@@ -18,15 +18,21 @@
 
 package org.apache.atlas.hive.hook.events;
 
+import com.google.common.collect.ImmutableMap;
 import org.apache.atlas.hive.hook.AtlasHiveHookContext;
 import org.apache.atlas.hive.hook.HiveHook.PreprocessAction;
+import org.apache.atlas.utils.PathExtractorContext;
 import org.apache.atlas.model.instance.AtlasEntity;
 import org.apache.atlas.model.instance.AtlasEntity.AtlasEntitiesWithExtInfo;
 import org.apache.atlas.model.instance.AtlasEntity.AtlasEntityWithExtInfo;
 import org.apache.atlas.model.instance.AtlasEntity.AtlasEntityExtInfo;
 import org.apache.atlas.model.instance.AtlasObjectId;
+import org.apache.atlas.model.instance.AtlasRelatedObjectId;
 import org.apache.atlas.model.instance.AtlasStruct;
 import org.apache.atlas.model.notification.HookNotification;
+import org.apache.atlas.repository.Constants;
+import org.apache.atlas.type.AtlasTypeUtil;
+import org.apache.atlas.utils.AtlasPathExtractorUtil;
 import org.apache.atlas.utils.HdfsNameServiceResolver;
 import org.apache.commons.collections.CollectionUtils;
 import org.apache.commons.collections.MapUtils;
@@ -49,43 +55,31 @@ import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
 import java.io.IOException;
+import java.net.InetAddress;
 import java.net.URI;
-import java.util.ArrayList;
-import java.util.Collection;
-import java.util.Collections;
-import java.util.Comparator;
-import java.util.HashMap;
-import java.util.HashSet;
-import java.util.List;
-import java.util.Map;
-import java.util.Set;
+import java.util.*;
 
-import static org.apache.atlas.hive.hook.AtlasHiveHookContext.QNAME_SEP_CLUSTER_NAME;
+import static org.apache.atlas.hive.bridge.HiveMetaStoreBridge.getDatabaseName;
+import static org.apache.atlas.hive.hook.AtlasHiveHookContext.QNAME_SEP_METADATA_NAMESPACE;
 import static org.apache.atlas.hive.hook.AtlasHiveHookContext.QNAME_SEP_ENTITY_NAME;
 import static org.apache.atlas.hive.hook.AtlasHiveHookContext.QNAME_SEP_PROCESS;
 
 public abstract class BaseHiveEvent {
     private static final Logger LOG = LoggerFactory.getLogger(BaseHiveEvent.class);
 
-    public static final String HIVE_TYPE_DB             = "hive_db";
-    public static final String HIVE_TYPE_TABLE          = "hive_table";
-    public static final String HIVE_TYPE_STORAGEDESC    = "hive_storagedesc";
-    public static final String HIVE_TYPE_COLUMN         = "hive_column";
-    public static final String HIVE_TYPE_PROCESS        = "hive_process";
-    public static final String HIVE_TYPE_COLUMN_LINEAGE = "hive_column_lineage";
-    public static final String HIVE_TYPE_SERDE          = "hive_serde";
-    public static final String HIVE_TYPE_ORDER          = "hive_order";
-    public static final String HDFS_TYPE_PATH           = "hdfs_path";
-    public static final String HBASE_TYPE_TABLE         = "hbase_table";
-    public static final String HBASE_TYPE_NAMESPACE     = "hbase_namespace";
-    public static final String AWS_S3_BUCKET            = "aws_s3_bucket";
-    public static final String AWS_S3_PSEUDO_DIR        = "aws_s3_pseudo_dir";
-    public static final String AWS_S3_OBJECT            = "aws_s3_object";
-
-    public static final String SCHEME_SEPARATOR         = "://";
-    public static final String S3_SCHEME                = "s3" + SCHEME_SEPARATOR;
-    public static final String S3A_SCHEME               = "s3a" + SCHEME_SEPARATOR;
-
+    public static final String HIVE_TYPE_DB                        = "hive_db";
+    public static final String HIVE_TYPE_TABLE                     = "hive_table";
+    public static final String HIVE_TYPE_STORAGEDESC               = "hive_storagedesc";
+    public static final String HIVE_TYPE_COLUMN                    = "hive_column";
+    public static final String HIVE_TYPE_PROCESS                   = "hive_process";
+    public static final String HIVE_TYPE_COLUMN_LINEAGE            = "hive_column_lineage";
+    public static final String HIVE_TYPE_SERDE                     = "hive_serde";
+    public static final String HIVE_TYPE_ORDER                     = "hive_order";
+    public static final String HIVE_TYPE_PROCESS_EXECUTION         = "hive_process_execution";
+    public static final String HIVE_DB_DDL                         = "hive_db_ddl";
+    public static final String HIVE_TABLE_DDL                      = "hive_table_ddl";
+    public static final String HBASE_TYPE_TABLE                    = "hbase_table";
+    public static final String HBASE_TYPE_NAMESPACE                = "hbase_namespace";
     public static final String ATTRIBUTE_QUALIFIED_NAME            = "qualifiedName";
     public static final String ATTRIBUTE_NAME                      = "name";
     public static final String ATTRIBUTE_DESCRIPTION               = "description";
@@ -126,6 +120,8 @@ public abstract class BaseHiveEvent {
     public static final String ATTRIBUTE_START_TIME                = "startTime";
     public static final String ATTRIBUTE_USER_NAME                 = "userName";
     public static final String ATTRIBUTE_QUERY_TEXT                = "queryText";
+    public static final String ATTRIBUTE_PROCESS                   = "process";
+    public static final String ATTRIBUTE_PROCESS_EXECUTIONS        = "processExecutions";
     public static final String ATTRIBUTE_QUERY_ID                  = "queryId";
     public static final String ATTRIBUTE_QUERY_PLAN                = "queryPlan";
     public static final String ATTRIBUTE_END_TIME                  = "endTime";
@@ -137,15 +133,30 @@ public abstract class BaseHiveEvent {
     public static final String ATTRIBUTE_URI                       = "uri";
     public static final String ATTRIBUTE_STORAGE_HANDLER           = "storage_handler";
     public static final String ATTRIBUTE_NAMESPACE                 = "namespace";
-    public static final String ATTRIBUTE_OBJECT_PREFIX             = "objectPrefix";
-    public static final String ATTRIBUTE_BUCKET                    = "bucket";
-
+    public static final String ATTRIBUTE_HOSTNAME                  = "hostName";
+    public static final String ATTRIBUTE_EXEC_TIME                 = "execTime";
+    public static final String ATTRIBUTE_DDL_QUERIES               = "ddlQueries";
+    public static final String ATTRIBUTE_SERVICE_TYPE              = "serviceType";
     public static final String HBASE_STORAGE_HANDLER_CLASS         = "org.apache.hadoop.hive.hbase.HBaseStorageHandler";
     public static final String HBASE_DEFAULT_NAMESPACE             = "default";
     public static final String HBASE_NAMESPACE_TABLE_DELIMITER     = ":";
     public static final String HBASE_PARAM_TABLE_NAME              = "hbase.table.name";
     public static final long   MILLIS_CONVERT_FACTOR               = 1000;
     public static final String HDFS_PATH_PREFIX                    = "hdfs://";
+    public static final String EMPTY_ATTRIBUTE_VALUE = "";
+
+    public static final String RELATIONSHIP_DATASET_PROCESS_INPUTS        = "dataset_process_inputs";
+    public static final String RELATIONSHIP_PROCESS_DATASET_OUTPUTS       = "process_dataset_outputs";
+    public static final String RELATIONSHIP_HIVE_PROCESS_COLUMN_LINEAGE   = "hive_process_column_lineage";
+    public static final String RELATIONSHIP_HIVE_TABLE_DB                 = "hive_table_db";
+    public static final String RELATIONSHIP_HIVE_TABLE_PART_KEYS          = "hive_table_partitionkeys";
+    public static final String RELATIONSHIP_HIVE_TABLE_COLUMNS            = "hive_table_columns";
+    public static final String RELATIONSHIP_HIVE_TABLE_STORAGE_DESC       = "hive_table_storagedesc";
+    public static final String RELATIONSHIP_HIVE_PROCESS_PROCESS_EXE      = "hive_process_process_executions";
+    public static final String RELATIONSHIP_HIVE_DB_DDL_QUERIES           = "hive_db_ddl_queries";
+    public static final String RELATIONSHIP_HIVE_TABLE_DDL_QUERIES        = "hive_table_ddl_queries";
+    public static final String RELATIONSHIP_HBASE_TABLE_NAMESPACE         = "hbase_table_namespace";
+
 
     public static final Map<Integer, String> OWNER_TYPE_TO_ENUM_VALUE = new HashMap<>();
 
@@ -179,12 +190,6 @@ public abstract class BaseHiveEvent {
         return table.getTTable() != null ? (table.getOwner()): "";
     }
 
-    public static AtlasObjectId getObjectId(AtlasEntity entity) {
-        String        qualifiedName = (String) entity.getAttribute(ATTRIBUTE_QUALIFIED_NAME);
-        AtlasObjectId ret           = new AtlasObjectId(entity.getGuid(), entity.getTypeName(), Collections.singletonMap(ATTRIBUTE_QUALIFIED_NAME, qualifiedName));
-
-        return ret;
-    }
 
     public static List<AtlasObjectId> getObjectIds(List<AtlasEntity> entities) {
         final List<AtlasObjectId> ret;
@@ -193,7 +198,7 @@ public abstract class BaseHiveEvent {
             ret = new ArrayList<>(entities.size());
 
             for (AtlasEntity entity : entities) {
-                ret.add(getObjectId(entity));
+                ret.add(AtlasTypeUtil.getObjectId(entity));
             }
         } else {
             ret = Collections.emptyList();
@@ -217,14 +222,14 @@ public abstract class BaseHiveEvent {
         }
     }
 
-    protected AtlasEntity getInputOutputEntity(Entity entity, AtlasEntityExtInfo entityExtInfo) throws Exception {
+    protected AtlasEntity getInputOutputEntity(Entity entity, AtlasEntityExtInfo entityExtInfo, boolean skipTempTables) throws Exception {
         AtlasEntity ret = null;
 
         switch(entity.getType()) {
             case TABLE:
             case PARTITION:
             case DFS_DIR: {
-                ret = toAtlasEntity(entity, entityExtInfo);
+                ret = toAtlasEntity(entity, entityExtInfo, skipTempTables);
             }
             break;
         }
@@ -232,13 +237,15 @@ public abstract class BaseHiveEvent {
         return ret;
     }
 
-    protected AtlasEntity toAtlasEntity(Entity entity, AtlasEntityExtInfo entityExtInfo) throws Exception {
+    protected AtlasEntity toAtlasEntity(Entity entity, AtlasEntityExtInfo entityExtInfo, boolean skipTempTables) throws Exception {
         AtlasEntity ret = null;
 
         switch (entity.getType()) {
             case DATABASE: {
-                if (!context.getIgnoreDummyDatabaseName().contains(entity.getDatabase().getName())) {
-                    Database db = getHive().getDatabase(entity.getDatabase().getName());
+                String dbName = getDatabaseName(entity.getDatabase());
+
+                if (!context.getIgnoreDummyDatabaseName().contains(dbName)) {
+                    Database db = getHive().getDatabase(dbName);
 
                     ret = toDbEntity(db);
                 }
@@ -255,6 +262,10 @@ public abstract class BaseHiveEvent {
                     skipTable = context.getIgnoreDummyTableName().contains(tableName) && context.getIgnoreDummyDatabaseName().contains(dbName);
                 }
 
+                if (!skipTable) {
+                    skipTable = skipTempTables && entity.getTable().isTemporary();
+                }
+
                 if (!skipTable) {
                     Table table = getHive().getTable(dbName, tableName);
 
@@ -282,10 +293,10 @@ public abstract class BaseHiveEvent {
     }
 
     protected AtlasEntity toDbEntity(Database db) throws Exception {
-        String  dbQualifiedName = getQualifiedName(db);
-        boolean isKnownDatabase = context.isKnownDatabase(dbQualifiedName);
-
-        AtlasEntity ret = context.getEntity(dbQualifiedName);
+        String      dbName          = getDatabaseName(db);
+        String      dbQualifiedName = getQualifiedName(db);
+        boolean     isKnownDatabase = context.isKnownDatabase(dbQualifiedName);
+        AtlasEntity ret             = context.getEntity(dbQualifiedName);
 
         if (ret == null) {
             ret = new AtlasEntity(HIVE_TYPE_DB);
@@ -298,11 +309,15 @@ public abstract class BaseHiveEvent {
             }
 
             ret.setAttribute(ATTRIBUTE_QUALIFIED_NAME, dbQualifiedName);
-            ret.setAttribute(ATTRIBUTE_NAME, db.getName().toLowerCase());
-            ret.setAttribute(ATTRIBUTE_DESCRIPTION, db.getDescription());
+            ret.setAttribute(ATTRIBUTE_NAME, dbName);
+
+            if (StringUtils.isNotEmpty(db.getDescription())) {
+                ret.setAttribute(ATTRIBUTE_DESCRIPTION, db.getDescription());
+            }
+
             ret.setAttribute(ATTRIBUTE_OWNER, db.getOwnerName());
 
-            ret.setAttribute(ATTRIBUTE_CLUSTER_NAME, getClusterName());
+            ret.setAttribute(ATTRIBUTE_CLUSTER_NAME, getMetadataNamespace());
             ret.setAttribute(ATTRIBUTE_LOCATION, HdfsNameServiceResolver.getPathWithNameServiceID(db.getLocationUri()));
             ret.setAttribute(ATTRIBUTE_PARAMETERS, db.getParameters());
 
@@ -350,7 +365,7 @@ public abstract class BaseHiveEvent {
             }
         }
 
-        AtlasEntity ret = toTableEntity(getObjectId(dbEntity), table, entityExtInfo);
+        AtlasEntity ret = toTableEntity(AtlasTypeUtil.getObjectId(dbEntity), table, entityExtInfo);
 
         return ret;
     }
@@ -379,7 +394,9 @@ public abstract class BaseHiveEvent {
                 long createTime     = getTableCreateTime(table);
                 long lastAccessTime = table.getLastAccessTime() > 0 ? (table.getLastAccessTime() * MILLIS_CONVERT_FACTOR) : createTime;
 
-                ret.setAttribute(ATTRIBUTE_DB, dbId);
+                AtlasRelatedObjectId dbRelatedObject =     new AtlasRelatedObjectId(dbId, RELATIONSHIP_HIVE_TABLE_DB);
+
+                ret.setRelationshipAttribute(ATTRIBUTE_DB, dbRelatedObject );
                 ret.setAttribute(ATTRIBUTE_QUALIFIED_NAME, tblQualifiedName);
                 ret.setAttribute(ATTRIBUTE_NAME, table.getTableName().toLowerCase());
                 ret.setAttribute(ATTRIBUTE_OWNER, table.getOwner());
@@ -404,10 +421,12 @@ public abstract class BaseHiveEvent {
                 if (pruneTable) {
                     LOG.info("ignoring details of table {}", tblQualifiedName);
                 } else {
-                    AtlasObjectId     tableId       = getObjectId(ret);
+                    AtlasObjectId     tableId       = AtlasTypeUtil.getObjectId(ret);
                     AtlasEntity       sd            = getStorageDescEntity(tableId, table);
-                    List<AtlasEntity> partitionKeys = getColumnEntities(tableId, table, table.getPartitionKeys());
-                    List<AtlasEntity> columns       = getColumnEntities(tableId, table, table.getCols());
+                    List<AtlasEntity> partitionKeys = getColumnEntities(tableId, table, table.getPartitionKeys(), RELATIONSHIP_HIVE_TABLE_PART_KEYS);
+                    List<AtlasEntity> columns       = getColumnEntities(tableId, table, table.getCols(), RELATIONSHIP_HIVE_TABLE_COLUMNS);
+
+
 
                     if (entityExtInfo != null) {
                         entityExtInfo.addReferredEntity(sd);
@@ -425,9 +444,10 @@ public abstract class BaseHiveEvent {
                         }
                     }
 
-                    ret.setAttribute(ATTRIBUTE_STORAGEDESC, getObjectId(sd));
-                    ret.setAttribute(ATTRIBUTE_PARTITION_KEYS, getObjectIds(partitionKeys));
-                    ret.setAttribute(ATTRIBUTE_COLUMNS, getObjectIds(columns));
+
+                    ret.setRelationshipAttribute(ATTRIBUTE_STORAGEDESC, AtlasTypeUtil.getAtlasRelatedObjectId(sd, RELATIONSHIP_HIVE_TABLE_STORAGE_DESC));
+                    ret.setRelationshipAttribute(ATTRIBUTE_PARTITION_KEYS, AtlasTypeUtil.getAtlasRelatedObjectIds(partitionKeys, RELATIONSHIP_HIVE_TABLE_PART_KEYS));
+                    ret.setRelationshipAttribute(ATTRIBUTE_COLUMNS, AtlasTypeUtil.getAtlasRelatedObjectIds(columns, RELATIONSHIP_HIVE_TABLE_COLUMNS));
                 }
 
                 context.putEntity(tblQualifiedName, ret);
@@ -455,7 +475,9 @@ public abstract class BaseHiveEvent {
 
             StorageDescriptor sd = table.getSd();
 
-            ret.setAttribute(ATTRIBUTE_TABLE, tableId);
+            AtlasRelatedObjectId tableRelatedObject =     new AtlasRelatedObjectId(tableId, RELATIONSHIP_HIVE_TABLE_STORAGE_DESC);
+
+            ret.setRelationshipAttribute(ATTRIBUTE_TABLE, tableRelatedObject);
             ret.setAttribute(ATTRIBUTE_QUALIFIED_NAME, sdQualifiedName);
             ret.setAttribute(ATTRIBUTE_PARAMETERS, sd.getParameters());
             ret.setAttribute(ATTRIBUTE_LOCATION, HdfsNameServiceResolver.getPathWithNameServiceID(sd.getLocation()));
@@ -465,7 +487,7 @@ public abstract class BaseHiveEvent {
             ret.setAttribute(ATTRIBUTE_NUM_BUCKETS, sd.getNumBuckets());
             ret.setAttribute(ATTRIBUTE_STORED_AS_SUB_DIRECTORIES, sd.isStoredAsSubDirectories());
 
-            if (sd.getBucketCols().size() > 0) {
+            if (sd.getBucketCols() != null && sd.getBucketCols().size() > 0) {
                 ret.setAttribute(ATTRIBUTE_BUCKET_COLS, sd.getBucketCols());
             }
 
@@ -501,137 +523,177 @@ public abstract class BaseHiveEvent {
         return ret;
     }
 
-    protected List<AtlasEntity> getColumnEntities(AtlasObjectId tableId, Table table, List<FieldSchema> fieldSchemas) {
-        List<AtlasEntity> ret          = new ArrayList<>();
-        boolean           isKnownTable = tableId.getGuid() == null;
+    protected List<AtlasEntity> getColumnEntities(AtlasObjectId tableId, Table table, List<FieldSchema> fieldSchemas, String relationshipType) {
+        List<AtlasEntity> ret            = new ArrayList<>();
+        boolean           isKnownTable   = tableId.getGuid() == null;
+        int               columnPosition = 0;
 
-        int columnPosition = 0;
-        for (FieldSchema fieldSchema : fieldSchemas) {
-            String      colQualifiedName = getQualifiedName(table, fieldSchema);
-            AtlasEntity column           = context.getEntity(colQualifiedName);
+        if (CollectionUtils.isNotEmpty(fieldSchemas)) {
+            for (FieldSchema fieldSchema : fieldSchemas) {
+                String      colQualifiedName = getQualifiedName(table, fieldSchema);
+                AtlasEntity column           = context.getEntity(colQualifiedName);
 
-            if (column == null) {
-                column = new AtlasEntity(HIVE_TYPE_COLUMN);
+                if (column == null) {
+                    column = new AtlasEntity(HIVE_TYPE_COLUMN);
 
-                // if column's table was sent in an earlier notification, set 'guid' to null - which will:
-                //  - result in this entity to be not included in 'referredEntities'
-                //  - cause Atlas server to resolve the entity by its qualifiedName
-                if (isKnownTable) {
-                    column.setGuid(null);
+                    // if column's table was sent in an earlier notification, set 'guid' to null - which will:
+                    //  - result in this entity to be not included in 'referredEntities'
+                    //  - cause Atlas server to resolve the entity by its qualifiedName
+                    if (isKnownTable) {
+                        column.setGuid(null);
+                    }
+                    AtlasRelatedObjectId relatedObjectId = new AtlasRelatedObjectId(tableId, relationshipType);
+                    column.setRelationshipAttribute(ATTRIBUTE_TABLE, (relatedObjectId));
+                    column.setAttribute(ATTRIBUTE_QUALIFIED_NAME, colQualifiedName);
+                    column.setAttribute(ATTRIBUTE_NAME, fieldSchema.getName());
+                    column.setAttribute(ATTRIBUTE_OWNER, table.getOwner());
+                    column.setAttribute(ATTRIBUTE_COL_TYPE, fieldSchema.getType());
+                    column.setAttribute(ATTRIBUTE_COL_POSITION, columnPosition++);
+                    column.setAttribute(ATTRIBUTE_COMMENT, fieldSchema.getComment());
+
+                    context.putEntity(colQualifiedName, column);
                 }
 
-                column.setAttribute(ATTRIBUTE_TABLE, tableId);
-                column.setAttribute(ATTRIBUTE_QUALIFIED_NAME, colQualifiedName);
-                column.setAttribute(ATTRIBUTE_NAME, fieldSchema.getName());
-                column.setAttribute(ATTRIBUTE_OWNER, table.getOwner());
-                column.setAttribute(ATTRIBUTE_COL_TYPE, fieldSchema.getType());
-                column.setAttribute(ATTRIBUTE_COL_POSITION, columnPosition++);
-                column.setAttribute(ATTRIBUTE_COMMENT, fieldSchema.getComment());
-
-                context.putEntity(colQualifiedName, column);
+                ret.add(column);
             }
-
-            ret.add(column);
         }
 
         return ret;
     }
 
     protected AtlasEntity getPathEntity(Path path, AtlasEntityExtInfo extInfo) {
-        AtlasEntity ret;
-        String strPath = path.toString();
+        String               strPath                  = path.toString();
+        String               metadataNamespace        = getMetadataNamespace();
+        boolean              isConvertPathToLowerCase = strPath.startsWith(HDFS_PATH_PREFIX) && context.isConvertHdfsPathToLowerCase();
+        PathExtractorContext pathExtractorContext     = new PathExtractorContext(metadataNamespace, context.getQNameToEntityMap(),
+                                                                                 isConvertPathToLowerCase, context.getAwsS3AtlasModelVersion());
 
-        if (strPath.startsWith(HDFS_PATH_PREFIX) && context.isConvertHdfsPathToLowerCase()) {
-            strPath = strPath.toLowerCase();
+        AtlasEntityWithExtInfo entityWithExtInfo = AtlasPathExtractorUtil.getPathEntity(path, pathExtractorContext);
+
+        if (entityWithExtInfo.getReferredEntities() != null){
+            for (AtlasEntity entity : entityWithExtInfo.getReferredEntities().values()) {
+                extInfo.addReferredEntity(entity);
+            }
         }
 
-        if (isS3Path(strPath)) {
-            String      bucketName          = path.toUri().getAuthority();
-            String      bucketQualifiedName = (path.toUri().getScheme() + SCHEME_SEPARATOR + path.toUri().getAuthority() + QNAME_SEP_CLUSTER_NAME).toLowerCase() + getClusterName();
-            String      pathQualifiedName   = (strPath + QNAME_SEP_CLUSTER_NAME).toLowerCase() + getClusterName();
-            AtlasEntity bucketEntity        = context.getEntity(bucketQualifiedName);
+        return entityWithExtInfo.getEntity();
+    }
 
-            ret = context.getEntity(pathQualifiedName);
+    protected AtlasEntity getHiveProcessEntity(List<AtlasEntity> inputs, List<AtlasEntity> outputs) throws Exception {
+        AtlasEntity ret           = new AtlasEntity(HIVE_TYPE_PROCESS);
+        String      queryStr      = getQueryString();
+        String      qualifiedName = getQualifiedName(inputs, outputs);
 
-            if (ret == null) {
-                if (bucketEntity == null) {
-                    bucketEntity = new AtlasEntity(AWS_S3_BUCKET);
+        if (queryStr != null) {
+            queryStr = queryStr.toLowerCase().trim();
+        }
 
-                    bucketEntity.setAttribute(ATTRIBUTE_QUALIFIED_NAME, bucketQualifiedName);
-                    bucketEntity.setAttribute(ATTRIBUTE_NAME, bucketName);
+        ret.setAttribute(ATTRIBUTE_OPERATION_TYPE, getOperationName());
 
-                    context.putEntity(bucketQualifiedName, bucketEntity);
-                }
+        if (context.isMetastoreHook()) {
+            HiveOperation operation = context.getHiveOperation();
 
-                extInfo.addReferredEntity(bucketEntity);
+            if (operation == HiveOperation.CREATETABLE || operation == HiveOperation.CREATETABLE_AS_SELECT) {
+                AtlasEntity table      = outputs.get(0);
+                long        createTime = Long.valueOf((Long)table.getAttribute(ATTRIBUTE_CREATE_TIME));
+                qualifiedName          =  (String) table.getAttribute(ATTRIBUTE_QUALIFIED_NAME) + QNAME_SEP_PROCESS + createTime;
 
-                ret = new AtlasEntity(AWS_S3_PSEUDO_DIR);
-
-                ret.setAttribute(ATTRIBUTE_BUCKET, getObjectId(bucketEntity));
-                ret.setAttribute(ATTRIBUTE_OBJECT_PREFIX, Path.getPathWithoutSchemeAndAuthority(path).toString().toLowerCase());
-                ret.setAttribute(ATTRIBUTE_QUALIFIED_NAME, pathQualifiedName);
-                ret.setAttribute(ATTRIBUTE_NAME, Path.getPathWithoutSchemeAndAuthority(path).toString().toLowerCase());
-
-                context.putEntity(pathQualifiedName, ret);
+                ret.setAttribute(ATTRIBUTE_NAME, "dummyProcess:" + UUID.randomUUID());
+                ret.setAttribute(ATTRIBUTE_OPERATION_TYPE, operation.getOperationName());
             }
+        }
+
+        ret.setAttribute(ATTRIBUTE_QUALIFIED_NAME, qualifiedName);
+        ret.setAttribute(ATTRIBUTE_NAME, qualifiedName);
+        ret.setRelationshipAttribute(ATTRIBUTE_INPUTS, AtlasTypeUtil.getAtlasRelatedObjectIds(inputs, RELATIONSHIP_DATASET_PROCESS_INPUTS));
+        ret.setRelationshipAttribute(ATTRIBUTE_OUTPUTS, AtlasTypeUtil.getAtlasRelatedObjectIds(outputs, RELATIONSHIP_PROCESS_DATASET_OUTPUTS));
+
+        // We are setting an empty value to these attributes, since now we have a new entity type called hive process
+        // execution which captures these values. We have to set empty values here because these attributes are
+        // mandatory attributes for hive process entity type.
+        ret.setAttribute(ATTRIBUTE_START_TIME, System.currentTimeMillis());
+        ret.setAttribute(ATTRIBUTE_END_TIME, System.currentTimeMillis());
+
+        if (context.isHiveProcessPopulateDeprecatedAttributes()) {
+            ret.setAttribute(ATTRIBUTE_USER_NAME, getUserName());
+            ret.setAttribute(ATTRIBUTE_QUERY_TEXT, queryStr);
+            ret.setAttribute(ATTRIBUTE_QUERY_ID, getQueryId());
         } else {
-            String nameServiceID     = HdfsNameServiceResolver.getNameServiceIDForPath(strPath);
-            String attrPath          = StringUtils.isEmpty(nameServiceID) ? strPath : HdfsNameServiceResolver.getPathWithNameServiceID(strPath);
-            String pathQualifiedName = getQualifiedName(attrPath);
-
-            ret = context.getEntity(pathQualifiedName);
-
-            if (ret == null) {
-                ret = new AtlasEntity(HDFS_TYPE_PATH);
-
-                if (StringUtils.isNotEmpty(nameServiceID)) {
-                    ret.setAttribute(ATTRIBUTE_NAMESERVICE_ID, nameServiceID);
-                }
-
-                String name = Path.getPathWithoutSchemeAndAuthority(path).toString();
-
-                if (strPath.startsWith(HDFS_PATH_PREFIX) && context.isConvertHdfsPathToLowerCase()) {
-                    name = name.toLowerCase();
-                }
-
-                ret.setAttribute(ATTRIBUTE_PATH, attrPath);
-                ret.setAttribute(ATTRIBUTE_QUALIFIED_NAME, pathQualifiedName);
-                ret.setAttribute(ATTRIBUTE_NAME, name);
-                ret.setAttribute(ATTRIBUTE_CLUSTER_NAME, getClusterName());
-
-                context.putEntity(pathQualifiedName, ret);
-            }
+            ret.setAttribute(ATTRIBUTE_USER_NAME, EMPTY_ATTRIBUTE_VALUE);
+            ret.setAttribute(ATTRIBUTE_QUERY_TEXT, EMPTY_ATTRIBUTE_VALUE);
+            ret.setAttribute(ATTRIBUTE_QUERY_ID, EMPTY_ATTRIBUTE_VALUE);
         }
 
+        ret.setAttribute(ATTRIBUTE_QUERY_PLAN, "Not Supported");
+        ret.setAttribute(ATTRIBUTE_RECENT_QUERIES, Collections.singletonList(queryStr));
+        ret.setAttribute(ATTRIBUTE_CLUSTER_NAME, getMetadataNamespace());
+
         return ret;
     }
 
-    protected AtlasEntity getHiveProcessEntity(List<AtlasEntity> inputs, List<AtlasEntity> outputs) throws Exception {
-        AtlasEntity ret         = new AtlasEntity(HIVE_TYPE_PROCESS);
+    protected AtlasEntity getHiveProcessExecutionEntity(AtlasEntity hiveProcess) throws Exception {
+        AtlasEntity ret         = new AtlasEntity(HIVE_TYPE_PROCESS_EXECUTION);
         String      queryStr    = getQueryString();
 
         if (queryStr != null) {
             queryStr = queryStr.toLowerCase().trim();
         }
 
-        ret.setAttribute(ATTRIBUTE_QUALIFIED_NAME, getQualifiedName(inputs, outputs));
-        ret.setAttribute(ATTRIBUTE_INPUTS, getObjectIds(inputs));
-        ret.setAttribute(ATTRIBUTE_OUTPUTS,  getObjectIds(outputs));
-        ret.setAttribute(ATTRIBUTE_NAME, queryStr);
-        ret.setAttribute(ATTRIBUTE_OPERATION_TYPE, getOperationName());
+        Long endTime = System.currentTimeMillis();
+        ret.setAttribute(ATTRIBUTE_QUALIFIED_NAME, hiveProcess.getAttribute(ATTRIBUTE_QUALIFIED_NAME).toString() +
+                QNAME_SEP_PROCESS + getQueryStartTime().toString() +
+                QNAME_SEP_PROCESS + endTime.toString());
+        ret.setAttribute(ATTRIBUTE_NAME, ret.getAttribute(ATTRIBUTE_QUALIFIED_NAME));
         ret.setAttribute(ATTRIBUTE_START_TIME, getQueryStartTime());
-        ret.setAttribute(ATTRIBUTE_END_TIME, System.currentTimeMillis());
+        ret.setAttribute(ATTRIBUTE_END_TIME, endTime);
         ret.setAttribute(ATTRIBUTE_USER_NAME, getUserName());
         ret.setAttribute(ATTRIBUTE_QUERY_TEXT, queryStr);
         ret.setAttribute(ATTRIBUTE_QUERY_ID, getQueryId());
         ret.setAttribute(ATTRIBUTE_QUERY_PLAN, "Not Supported");
-        ret.setAttribute(ATTRIBUTE_RECENT_QUERIES, Collections.singletonList(queryStr));
-
+        ret.setAttribute(ATTRIBUTE_HOSTNAME, getContext().getHostName()); //
+        AtlasRelatedObjectId hiveProcessRelationObjectId = AtlasTypeUtil.toAtlasRelatedObjectId(hiveProcess, RELATIONSHIP_HIVE_PROCESS_PROCESS_EXE);
+        ret.setRelationshipAttribute(ATTRIBUTE_PROCESS, hiveProcessRelationObjectId);
         return ret;
     }
 
-    protected String getClusterName() {
-        return context.getClusterName();
+    protected AtlasEntity createHiveDDLEntity(AtlasEntity dbOrTable) {
+        return createHiveDDLEntity(dbOrTable, false);
+    }
+
+    protected AtlasEntity createHiveDDLEntity(AtlasEntity dbOrTable, boolean excludeEntityGuid) {
+        AtlasObjectId objId   = AtlasTypeUtil.getObjectId(dbOrTable);
+        AtlasEntity   hiveDDL = null;
+
+        if (excludeEntityGuid) {
+            objId.setGuid(null);
+        }
+        AtlasRelatedObjectId objIdRelatedObject =     new AtlasRelatedObjectId(objId);
+
+        if (StringUtils.equals(objId.getTypeName(), HIVE_TYPE_DB)) {
+            hiveDDL = new AtlasEntity(HIVE_DB_DDL);
+            objIdRelatedObject.setRelationshipType(RELATIONSHIP_HIVE_DB_DDL_QUERIES);
+            hiveDDL.setRelationshipAttribute(ATTRIBUTE_DB, objIdRelatedObject);
+        } else if (StringUtils.equals(objId.getTypeName(), HIVE_TYPE_TABLE)) {
+            hiveDDL = new AtlasEntity(HIVE_TABLE_DDL);
+            objIdRelatedObject.setRelationshipType(RELATIONSHIP_HIVE_TABLE_DDL_QUERIES);
+            hiveDDL.setRelationshipAttribute( ATTRIBUTE_TABLE, objIdRelatedObject);
+        }
+
+        if (hiveDDL != null) {
+            hiveDDL.setAttribute(ATTRIBUTE_SERVICE_TYPE, "hive");
+            hiveDDL.setAttribute(ATTRIBUTE_EXEC_TIME, getQueryStartTime());
+            hiveDDL.setAttribute(ATTRIBUTE_QUERY_TEXT, getQueryString());
+            hiveDDL.setAttribute(ATTRIBUTE_USER_NAME, getUserName());
+            hiveDDL.setAttribute(ATTRIBUTE_NAME, getQueryString());
+            hiveDDL.setAttribute(ATTRIBUTE_QUALIFIED_NAME, dbOrTable.getAttribute(ATTRIBUTE_QUALIFIED_NAME).toString()
+                                                           + QNAME_SEP_PROCESS + getQueryStartTime().toString());
+        }
+
+        return hiveDDL;
+    }
+
+    protected String getMetadataNamespace() {
+        return context.getMetadataNamespace();
     }
 
     protected Database getDatabases(String dbName) throws Exception {
@@ -672,7 +734,7 @@ public abstract class BaseHiveEvent {
     }
 
     protected Long getQueryStartTime() {
-        return isHiveContextValid() ? context.getHiveContext().getQueryPlan().getQueryStartTime() : null;
+        return isHiveContextValid() ? context.getHiveContext().getQueryPlan().getQueryStartTime() : System.currentTimeMillis();
     }
 
     protected String getQueryId() {
@@ -749,7 +811,7 @@ public abstract class BaseHiveEvent {
     protected String getQualifiedName(Table table, FieldSchema column) {
         String tblQualifiedName = getQualifiedName(table);
 
-        int sepPos = tblQualifiedName.lastIndexOf(QNAME_SEP_CLUSTER_NAME);
+        int sepPos = tblQualifiedName.lastIndexOf(QNAME_SEP_METADATA_NAMESPACE);
 
         if (sepPos == -1) {
             return tblQualifiedName + QNAME_SEP_ENTITY_NAME + column.getName().toLowerCase();
@@ -767,19 +829,20 @@ public abstract class BaseHiveEvent {
     }
 
     protected String getQualifiedName(BaseColumnInfo column) {
-        String dbName    = column.getTabAlias().getTable().getDbName();
-        String tableName = column.getTabAlias().getTable().getTableName();
-        String colName   = column.getColumn() != null ? column.getColumn().getName() : null;
+        String dbName            = column.getTabAlias().getTable().getDbName();
+        String tableName         = column.getTabAlias().getTable().getTableName();
+        String colName           = column.getColumn() != null ? column.getColumn().getName() : null;
+        String metadataNamespace = getMetadataNamespace();
 
         if (colName == null) {
-            return (dbName + QNAME_SEP_ENTITY_NAME + tableName + QNAME_SEP_CLUSTER_NAME).toLowerCase() + getClusterName();
+            return (dbName + QNAME_SEP_ENTITY_NAME + tableName + QNAME_SEP_METADATA_NAMESPACE).toLowerCase() + metadataNamespace;
         } else {
-            return (dbName + QNAME_SEP_ENTITY_NAME + tableName + QNAME_SEP_ENTITY_NAME + colName + QNAME_SEP_CLUSTER_NAME).toLowerCase() + getClusterName();
+            return (dbName + QNAME_SEP_ENTITY_NAME + tableName + QNAME_SEP_ENTITY_NAME + colName + QNAME_SEP_METADATA_NAMESPACE).toLowerCase() + metadataNamespace;
         }
     }
 
     protected String getQualifiedName(String dbName, String tableName, String colName) {
-        return (dbName + QNAME_SEP_ENTITY_NAME + tableName + QNAME_SEP_ENTITY_NAME + colName + QNAME_SEP_CLUSTER_NAME).toLowerCase() + getClusterName();
+        return (dbName + QNAME_SEP_ENTITY_NAME + tableName + QNAME_SEP_ENTITY_NAME + colName + QNAME_SEP_METADATA_NAMESPACE).toLowerCase() + getMetadataNamespace();
     }
 
     protected String getQualifiedName(URI location) {
@@ -797,14 +860,14 @@ public abstract class BaseHiveEvent {
 
     protected String getQualifiedName(String path) {
         if (path.startsWith(HdfsNameServiceResolver.HDFS_SCHEME)) {
-            return path + QNAME_SEP_CLUSTER_NAME + getClusterName();
+            return path + QNAME_SEP_METADATA_NAMESPACE + getMetadataNamespace();
         }
 
         return path.toLowerCase();
     }
 
     protected String getColumnQualifiedName(String tblQualifiedName, String columnName) {
-        int sepPos = tblQualifiedName.lastIndexOf(QNAME_SEP_CLUSTER_NAME);
+        int sepPos = tblQualifiedName.lastIndexOf(QNAME_SEP_METADATA_NAMESPACE);
 
         if (sepPos == -1) {
             return tblQualifiedName + QNAME_SEP_ENTITY_NAME + columnName.toLowerCase();
@@ -859,23 +922,27 @@ public abstract class BaseHiveEvent {
     }
 
     protected AtlasEntity toReferencedHBaseTable(Table table, AtlasEntitiesWithExtInfo entities) {
-        AtlasEntity    ret            = null;
-        HBaseTableInfo hBaseTableInfo = new HBaseTableInfo(table);
-        String         hbaseNameSpace = hBaseTableInfo.getHbaseNameSpace();
-        String         hbaseTableName = hBaseTableInfo.getHbaseTableName();
+        AtlasEntity    ret               = null;
+        HBaseTableInfo hBaseTableInfo    = new HBaseTableInfo(table);
+        String         hbaseNameSpace    = hBaseTableInfo.getHbaseNameSpace();
+        String         hbaseTableName    = hBaseTableInfo.getHbaseTableName();
+        String         metadataNamespace = getMetadataNamespace();
 
         if (hbaseTableName != null) {
             AtlasEntity nsEntity = new AtlasEntity(HBASE_TYPE_NAMESPACE);
             nsEntity.setAttribute(ATTRIBUTE_NAME, hbaseNameSpace);
-            nsEntity.setAttribute(ATTRIBUTE_CLUSTER_NAME, getClusterName());
-            nsEntity.setAttribute(ATTRIBUTE_QUALIFIED_NAME, getHBaseNameSpaceQualifiedName(getClusterName(), hbaseNameSpace));
+            nsEntity.setAttribute(ATTRIBUTE_CLUSTER_NAME, metadataNamespace);
+            nsEntity.setAttribute(ATTRIBUTE_QUALIFIED_NAME, getHBaseNameSpaceQualifiedName(metadataNamespace, hbaseNameSpace));
 
             ret = new AtlasEntity(HBASE_TYPE_TABLE);
 
             ret.setAttribute(ATTRIBUTE_NAME, hbaseTableName);
             ret.setAttribute(ATTRIBUTE_URI, hbaseTableName);
-            ret.setAttribute(ATTRIBUTE_NAMESPACE, getObjectId(nsEntity));
-            ret.setAttribute(ATTRIBUTE_QUALIFIED_NAME, getHBaseTableQualifiedName(getClusterName(), hbaseNameSpace, hbaseTableName));
+
+            AtlasRelatedObjectId objIdRelatedObject = new AtlasRelatedObjectId(AtlasTypeUtil.getObjectId(nsEntity), RELATIONSHIP_HBASE_TABLE_NAMESPACE);
+
+            ret.setRelationshipAttribute(ATTRIBUTE_NAMESPACE, objIdRelatedObject);
+            ret.setAttribute(ATTRIBUTE_QUALIFIED_NAME, getHBaseTableQualifiedName(metadataNamespace, hbaseNameSpace, hbaseTableName));
 
             entities.addReferredEntity(nsEntity);
             entities.addEntity(ret);
@@ -897,12 +964,12 @@ public abstract class BaseHiveEvent {
         return ret;
     }
 
-    private static String getHBaseTableQualifiedName(String clusterName, String nameSpace, String tableName) {
-        return String.format("%s:%s@%s", nameSpace.toLowerCase(), tableName.toLowerCase(), clusterName);
+    private static String getHBaseTableQualifiedName(String metadataNamespace, String nameSpace, String tableName) {
+        return String.format("%s:%s@%s", nameSpace.toLowerCase(), tableName.toLowerCase(), metadataNamespace);
     }
 
-    private static String getHBaseNameSpaceQualifiedName(String clusterName, String nameSpace) {
-        return String.format("%s@%s", nameSpace.toLowerCase(), clusterName);
+    private static String getHBaseNameSpaceQualifiedName(String metadataNamespace, String nameSpace) {
+        return String.format("%s@%s", nameSpace.toLowerCase(), metadataNamespace);
     }
 
     private boolean ignoreHDFSPathsinProcessQualifiedName() {
@@ -1025,11 +1092,6 @@ public abstract class BaseHiveEvent {
         return false;
     }
 
-    private boolean isS3Path(String strPath) {
-        return strPath != null && (strPath.startsWith(S3_SCHEME) || strPath.startsWith(S3A_SCHEME));
-    }
-
-
     static final class EntityComparator implements Comparator<Entity> {
         @Override
         public int compare(Entity entity1, Entity entity2) {
