@@ -33,6 +33,7 @@ import org.slf4j.LoggerFactory;
 import java.util.Collections;
 import java.util.List;
 
+import static org.apache.atlas.hive.bridge.HiveMetaStoreBridge.getDatabaseName;
 import static org.apache.hadoop.hive.ql.hooks.Entity.Type.DATABASE;
 
 public class CreateDatabase extends BaseHiveEvent {
@@ -59,6 +60,10 @@ public class CreateDatabase extends BaseHiveEvent {
         CreateDatabaseEvent      dbEvent = (CreateDatabaseEvent) context.getMetastoreEvent();
         Database                 db      = dbEvent.getDatabase();
 
+        if (db != null) {
+            db = context.getMetastoreHandler().get_database(db.getName());
+        }
+
         if (db != null) {
             AtlasEntity dbEntity = toDbEntity(db);
 
@@ -80,13 +85,18 @@ public class CreateDatabase extends BaseHiveEvent {
                 Database db = entity.getDatabase();
 
                 if (db != null) {
-                    db = getHive().getDatabase(db.getName());
+                    db = getHive().getDatabase(getDatabaseName(db));
                 }
 
                 if (db != null) {
-                    AtlasEntity dbEntity = toDbEntity(db);
+                    AtlasEntity dbEntity    = toDbEntity(db);
+                    AtlasEntity dbDDLEntity = createHiveDDLEntity(dbEntity);
 
                     ret.addEntity(dbEntity);
+
+                    if (dbDDLEntity != null) {
+                        ret.addEntity(dbDDLEntity);
+                    }
                 } else {
                     LOG.error("CreateDatabase.getEntities(): failed to retrieve db");
                 }
