@@ -52,8 +52,9 @@ import java.util.Date;
 public class SqoopHook extends SqoopJobDataPublisher {
     private static final Logger LOG = LoggerFactory.getLogger(SqoopHook.class);
 
-    public static final String ATLAS_CLUSTER_NAME   = "atlas.cluster.name";
-    public static final String DEFAULT_CLUSTER_NAME = "primary";
+    public static final String CLUSTER_NAME_KEY           = "atlas.cluster.name";
+    public static final String ATLAS_METADATA_NAMESPACE   = "atlas.metadata.namespace";
+    public static final String DEFAULT_CLUSTER_NAME       = "primary";
 
     public static final String USER           = "userName";
     public static final String DB_STORE_TYPE  = "dbStoreType";
@@ -69,6 +70,10 @@ public class SqoopHook extends SqoopJobDataPublisher {
     public static final String OUTPUTS        = "outputs";
     public static final String ATTRIBUTE_DB   = "db";
 
+    public static final String RELATIONSHIP_HIVE_TABLE_DB = "hive_table_db";
+    public static final String RELATIONSHIP_DATASET_PROCESS_INPUTS = "dataset_process_inputs";
+    public static final String RELATIONSHIP_PROCESS_DATASET_OUTPUTS = "process_dataset_outputs";
+
     private static final AtlasHookImpl atlasHook;
 
     static {
@@ -80,12 +85,14 @@ public class SqoopHook extends SqoopJobDataPublisher {
     @Override
     public void publish(SqoopJobDataPublisher.Data data) throws AtlasHookException {
         try {
-            Configuration atlasProperties = ApplicationProperties.get();
-            String        clusterName     = atlasProperties.getString(ATLAS_CLUSTER_NAME, DEFAULT_CLUSTER_NAME);
-            AtlasEntity   entDbStore      = toSqoopDBStoreEntity(data);
-            AtlasEntity   entHiveDb       = toHiveDatabaseEntity(clusterName, data.getHiveDB());
-            AtlasEntity   entHiveTable    = data.getHiveTable() != null ? toHiveTableEntity(entHiveDb, data.getHiveTable()) : null;
-            AtlasEntity   entProcess      = toSqoopProcessEntity(entDbStore, entHiveDb, entHiveTable, data, clusterName);
+            Configuration atlasProperties   = ApplicationProperties.get();
+            String        metadataNamespace = atlasProperties.getString(ATLAS_METADATA_NAMESPACE, getClusterName(atlasProperties));
+
+            AtlasEntity entDbStore   = toSqoopDBStoreEntity(data);
+            AtlasEntity entHiveDb    = toHiveDatabaseEntity(metadataNamespace, data.getHiveDB());
+            AtlasEntity entHiveTable = data.getHiveTable() != null ? toHiveTableEntity(entHiveDb, data.getHiveTable()) : null;
+            AtlasEntity entProcess   = toSqoopProcessEntity(entDbStore, entHiveDb, entHiveTable, data, metadataNamespace);
+
 
             AtlasEntitiesWithExtInfo entities = new AtlasEntitiesWithExtInfo(entProcess);
 
@@ -105,11 +112,15 @@ public class SqoopHook extends SqoopJobDataPublisher {
         }
     }
 
-    private AtlasEntity toHiveDatabaseEntity(String clusterName, String dbName) {
-        AtlasEntity entHiveDb     = new AtlasEntity(HiveDataTypes.HIVE_DB.getName());
-        String      qualifiedName = HiveMetaStoreBridge.getDBQualifiedName(clusterName, dbName);
+    private String getClusterName(Configuration config) {
+        return config.getString(CLUSTER_NAME_KEY, DEFAULT_CLUSTER_NAME);
+    }
 
-        entHiveDb.setAttribute(AtlasConstants.CLUSTER_NAME_ATTRIBUTE, clusterName);
+    private AtlasEntity toHiveDatabaseEntity(String metadataNamespace, String dbName) {
+        AtlasEntity entHiveDb     = new AtlasEntity(HiveDataTypes.HIVE_DB.getName());
+        String      qualifiedName = HiveMetaStoreBridge.getDBQualifiedName(metadataNamespace, dbName);
+
+        entHiveDb.setAttribute(AtlasConstants.CLUSTER_NAME_ATTRIBUTE, metadataNamespace);
         entHiveDb.setAttribute(AtlasClient.NAME, dbName);
         entHiveDb.setAttribute(AtlasClient.REFERENCEABLE_ATTRIBUTE_NAME, qualifiedName);
 
@@ -122,7 +133,7 @@ public class SqoopHook extends SqoopJobDataPublisher {
 
         entHiveTable.setAttribute(AtlasClient.NAME, tableName.toLowerCase());
         entHiveTable.setAttribute(AtlasClient.REFERENCEABLE_ATTRIBUTE_NAME, qualifiedName);
-        entHiveTable.setAttribute(ATTRIBUTE_DB, AtlasTypeUtil.getAtlasObjectId(entHiveDb));
+        entHiveTable.setRelationshipAttribute(ATTRIBUTE_DB, AtlasTypeUtil.getAtlasRelatedObjectId(entHiveDb, RELATIONSHIP_HIVE_TABLE_DB));
 
         return entHiveTable;
     }
@@ -153,9 +164,10 @@ public class SqoopHook extends SqoopJobDataPublisher {
         return entDbStore;
     }
 
-    private AtlasEntity toSqoopProcessEntity(AtlasEntity entDbStore, AtlasEntity entHiveDb, AtlasEntity entHiveTable, SqoopJobDataPublisher.Data data, String clusterName) {
+    private AtlasEntity toSqoopProcessEntity(AtlasEntity entDbStore, AtlasEntity entHiveDb, AtlasEntity entHiveTable,
+                                             SqoopJobDataPublisher.Data data, String metadataNamespace) {
         AtlasEntity         entProcess       = new AtlasEntity(SqoopDataTypes.SQOOP_PROCESS.getName());
-        String              sqoopProcessName = getSqoopProcessName(data, clusterName);
+        String              sqoopProcessName = getSqoopProcessName(data, metadataNamespace);
         Map<String, String> sqoopOptionsMap  = new HashMap<>();
         Properties          options          = data.getOptions();
 
@@ -171,11 +183,11 @@ public class SqoopHook extends SqoopJobDataPublisher {
         List<AtlasObjectId> hiveObjects  = Collections.singletonList(AtlasTypeUtil.getAtlasObjectId(entHiveTable != null ? entHiveTable : entHiveDb));
 
         if (isImportOperation(data)) {
-            entProcess.setAttribute(SqoopHook.INPUTS, sqoopObjects);
-            entProcess.setAttribute(SqoopHook.OUTPUTS, hiveObjects);
+            entProcess.setRelationshipAttribute(SqoopHook.INPUTS, AtlasTypeUtil.getAtlasRelatedObjectIdList(sqoopObjects, RELATIONSHIP_DATASET_PROCESS_INPUTS));
+            entProcess.setRelationshipAttribute(SqoopHook.OUTPUTS, AtlasTypeUtil.getAtlasRelatedObjectIdList(hiveObjects, RELATIONSHIP_PROCESS_DATASET_OUTPUTS));
         } else {
-            entProcess.setAttribute(SqoopHook.INPUTS, hiveObjects);
-            entProcess.setAttribute(SqoopHook.OUTPUTS, sqoopObjects);
+            entProcess.setRelationshipAttribute(SqoopHook.INPUTS, AtlasTypeUtil.getAtlasRelatedObjectIdList(hiveObjects, RELATIONSHIP_DATASET_PROCESS_INPUTS));
+            entProcess.setRelationshipAttribute(SqoopHook.OUTPUTS, AtlasTypeUtil.getAtlasRelatedObjectIdList(sqoopObjects, RELATIONSHIP_PROCESS_DATASET_OUTPUTS));
         }
 
         entProcess.setAttribute(SqoopHook.USER, data.getUser());
@@ -190,7 +202,7 @@ public class SqoopHook extends SqoopJobDataPublisher {
         return data.getOperation().toLowerCase().equals("import");
     }
 
-    static String getSqoopProcessName(Data data, String clusterName) {
+    static String getSqoopProcessName(Data data, String metadataNamespace) {
         StringBuilder name = new StringBuilder(String.format("sqoop %s --connect %s", data.getOperation(), data.getUrl()));
 
         if (StringUtils.isNotEmpty(data.getHiveTable())) {
@@ -204,9 +216,9 @@ public class SqoopHook extends SqoopJobDataPublisher {
         }
 
         if (data.getHiveTable() != null) {
-            name.append(String.format(" --hive-%s --hive-database %s --hive-table %s --hive-cluster %s", data.getOperation(), data.getHiveDB().toLowerCase(), data.getHiveTable().toLowerCase(), clusterName));
+            name.append(String.format(" --hive-%s --hive-database %s --hive-table %s --hive-cluster %s", data.getOperation(), data.getHiveDB().toLowerCase(), data.getHiveTable().toLowerCase(), metadataNamespace));
         } else {
-            name.append(String.format("--hive-%s --hive-database %s --hive-cluster %s", data.getOperation(), data.getHiveDB(), clusterName));
+            name.append(String.format("--hive-%s --hive-database %s --hive-cluster %s", data.getOperation(), data.getHiveDB(), metadataNamespace));
         }
 
         return name.toString();
