@@ -16,7 +16,7 @@
  * limitations under the License.
  */
 
-define(['require', 'utils/Utils', 'modules/Modal', 'utils/Messages', 'utils/Enums', 'moment'], function(require, Utils, Modal, Messages, Enums, moment) {
+define(['require', 'utils/Utils', 'modules/Modal', 'utils/Messages', 'utils/Enums', 'moment', 'utils/Globals'], function(require, Utils, Modal, Messages, Enums, moment, Globals) {
     'use strict';
 
     var CommonViewFunction = {};
@@ -24,20 +24,27 @@ define(['require', 'utils/Utils', 'modules/Modal', 'utils/Messages', 'utils/Enum
         require(['models/VTag'], function(VTag) {
             if (options && options.guid && options.tagName) {
                 var tagModel = new VTag(),
+                    noticeRef = null,
                     notifyObj = {
                         modal: true,
+                        okCloses: false,
+                        okShowLoader: true,
                         text: options.msg,
                         title: options.titleMessage,
                         okText: options.okText,
-                        ok: function(argument) {
+                        ok: function(notice) {
+                            noticeRef = notice;
                             if (options.showLoader) {
                                 options.showLoader();
                             }
                             tagModel.deleteAssociation(options.guid, options.tagName, options.associatedGuid, {
-                                skipDefaultError: true,
+                                defaultErrorMessage: options.tagName + Messages.deleteErrorMessage,
                                 success: function(data) {
+                                    if (noticeRef) {
+                                        noticeRef.remove();
+                                    }
                                     Utils.notifySuccess({
-                                        content: "Classification " + options.tagName + Messages.removeSuccessMessage
+                                        content: "Classification " + options.tagName + Messages.getAbbreviationMsg(false, 'removeSuccessMessage')
                                     });
                                     if (options.callback) {
                                         options.callback();
@@ -48,16 +55,12 @@ define(['require', 'utils/Utils', 'modules/Modal', 'utils/Messages', 'utils/Enum
 
                                 },
                                 cust_error: function(model, response) {
-                                    var message = options.tagName + Messages.deleteErrorMessage;
-                                    if (response && response.responseJSON) {
-                                        message = response.responseJSON.errorMessage;
+                                    if (noticeRef) {
+                                        noticeRef.hideButtonLoader();
                                     }
                                     if (options.hideLoader) {
                                         options.hideLoader();
                                     }
-                                    Utils.notifyError({
-                                        content: message
-                                    });
                                 }
                             });
                         },
@@ -80,15 +83,39 @@ define(['require', 'utils/Utils', 'modules/Modal', 'utils/Messages', 'utils/Enum
             attributeDefs = options.attributeDefs,
             formatIntVal = options.formatIntVal,
             showListCount = options.showListCount || true,
+            highlightString = options.highlightString,
+            formatStringVal = options.formatStringVal,
             numberFormat = options.numberFormat || _.numberFormatWithComa;
 
         var table = "",
+            getHighlightedString = function(resultStr) {
+                if (highlightString && highlightString.length) {
+                    try {
+                        return resultStr.replace(new RegExp(highlightString, "gi"), function(foundStr) {
+                            return "<span class='searched-term-highlight'>" + foundStr + "</span>"
+                        });
+                    } catch (error) {
+                        return resultStr;
+                    }
+                } else {
+                    return resultStr;
+                }
+            },
             getValue = function(val) {
                 if (val) {
                     if ((_.isNumber(val) || !_.isNaN(parseInt(val))) && formatIntVal) {
                         return numberFormat(val);
                     } else {
-                        return val;
+                        var newVal = val;
+                        if (formatStringVal) {
+                            newVal = parseInt(val);
+                            if (_.isNaN(newVal)) {
+                                newVal = val;
+                            } else {
+                                newVal = numberFormat(newVal);
+                            }
+                        }
+                        return getHighlightedString(_.escape(newVal));
                     }
                 } else {
                     return "N/A";
@@ -104,7 +131,7 @@ define(['require', 'utils/Utils', 'modules/Modal', 'utils/Messages', 'utils/Enum
                         value = Utils.getName(data);
                         var id = "";
                         if (data.guid) {
-                            if (Enums.entityStateReadOnly[data.status]) {
+                            if (Enums.entityStateReadOnly[data.status || data.entityStatus]) {
                                 deleteButton += '<button title="Deleted" class="btn btn-action btn-md deleteBtn"><i class="fa fa-trash"></i></button>';
                             }
                             id = data.guid;
@@ -144,7 +171,7 @@ define(['require', 'utils/Utils', 'modules/Modal', 'utils/Messages', 'utils/Enum
                     var inputOutputField = keyValue[i],
                         id = inputOutputField.guid || (_.isObject(inputOutputField.id) ? inputOutputField.id.id : inputOutputField.id),
                         tempLink = "",
-                        status = inputOutputField.status || (_.isObject(inputOutputField.id) ? inputOutputField.id.state : inputOutputField.state),
+                        status = (inputOutputField.status || inputOutputField.entityStatus) || (_.isObject(inputOutputField.id) ? inputOutputField.id.state : inputOutputField.state),
                         readOnly = Enums.entityStateReadOnly[status];
                     if (!inputOutputField.attributes && inputOutputField.values) {
                         inputOutputField['attributes'] = inputOutputField.values;
@@ -152,7 +179,7 @@ define(['require', 'utils/Utils', 'modules/Modal', 'utils/Messages', 'utils/Enum
                     if (_.isString(inputOutputField) || _.isBoolean(inputOutputField) || _.isNumber(inputOutputField)) {
                         var tempVarfor$check = inputOutputField.toString();
                         if (tempVarfor$check.indexOf("$") == -1) {
-                            valueOfArray.push('<span class="json-string">' + getValue(_.escape(inputOutputField)) + '</span>');
+                            valueOfArray.push('<span class="json-string">' + getValue(inputOutputField) + '</span>');
                         }
                     } else if (_.isObject(inputOutputField) && !id) {
                         var attributesList = inputOutputField;
@@ -180,9 +207,9 @@ define(['require', 'utils/Utils', 'modules/Modal', 'utils/Messages', 'utils/Enum
                                     }
                                 }
                             });
-                            valueOfArray.push(Utils.JSONPrettyPrint(newAttributesList));
+                            valueOfArray.push(Utils.JSONPrettyPrint(newAttributesList, getValue));
                         } else {
-                            valueOfArray.push(Utils.JSONPrettyPrint(attributesList));
+                            valueOfArray.push(Utils.JSONPrettyPrint(attributesList, getValue));
                         }
                     }
                     if (id && inputOutputField) {
@@ -193,7 +220,11 @@ define(['require', 'utils/Utils', 'modules/Modal', 'utils/Messages', 'utils/Enum
                             fetchInputOutputValue(fetchId, defEntity);
                             tempLink += '<div data-id="' + fetchId + '"><div class="value-loader"></div></div>';
                         } else {
-                            tempLink += '<a href="#!/detailPage/' + id + '">' + name + '</a>'
+                            if (inputOutputField.typeName == "AtlasGlossaryTerm") {
+                                tempLink += '<a href="#!/glossary/' + id + '?guid=' + id + '&gType=term&viewType=term&fromView=entity">' + name + '</a>'
+                            } else {
+                                tempLink += '<a href="#!/detailPage/' + id + '">' + name + '</a>'
+                            }
                         }
                     }
                     if (readOnly) {
@@ -216,15 +247,13 @@ define(['require', 'utils/Utils', 'modules/Modal', 'utils/Messages', 'utils/Enum
                 if (valueOfArray.length) {
                     subLink = valueOfArray.join(', ');
                 }
-                return subLink;
+                return subLink === "" ? "N/A" : subLink;
             }
         var valueObjectKeysList = _.keys(valueObject);
         if (_.isUndefined(sortBy) || sortBy == true) {
             valueObjectKeysList = _.sortBy(valueObjectKeysList);
         }
         valueObjectKeysList.map(function(key) {
-
-            key = _.escape(key);
             if (key == "profileData") {
                 return;
             }
@@ -234,7 +263,7 @@ define(['require', 'utils/Utils', 'modules/Modal', 'utils/Messages', 'utils/Enum
             if (defEntity && defEntity.typeName) {
                 var defEntityType = defEntity.typeName.toLocaleLowerCase();
                 if (defEntityType === 'date') {
-                    keyValue = new Date(keyValue);
+                    keyValue = keyValue > 0 ? new Date(keyValue) : "";
                 } else if (_.isObject(keyValue)) {
                     keyValue = extractObject({ "keyValue": keyValue, "key": key, 'defEntity': defEntity });
                 }
@@ -246,15 +275,17 @@ define(['require', 'utils/Utils', 'modules/Modal', 'utils/Messages', 'utils/Enum
             var val = "";
             if (_.isObject(valueObject[key])) {
                 val = keyValue
-            } else if (Utils.isUrl(keyValue)) {
-                val = '<a target="_blank" class="blue-link" href="' + keyValue + '">' + keyValue + '</a>';
             } else if (key === 'guid' || key === "__guid") {
-                val = '<a title="' + key + '" href="#!/detailPage/' + keyValue + '">' + keyValue + '</a>';
+                if (options.fromAdminAudit) {
+                    val = getValue(keyValue);
+                } else {
+                    val = '<a title="' + key + '" href="#!/detailPage/' + _.escape(keyValue) + '">' + getValue(keyValue) + '</a>';
+                }
             } else {
-                val = _.escape(keyValue);
+                val = getValue(keyValue);
             }
             if (isTable) {
-                var value = getValue(val),
+                var value = val,
                     appendClass = (value == "N/A" ? "hide-row" : ""),
                     htmlTag = '<div class="scroll-y">' + value + '</div>';
                 if (_.isObject(valueObject[key]) && !_.isEmpty(valueObject[key])) {
@@ -270,7 +301,7 @@ define(['require', 'utils/Utils', 'modules/Modal', 'utils/Messages', 'utils/Enum
                 }
                 table += '<tr class="' + appendClass + '"><td>' + (_.escape(key) + listCount) + '</td><td>' + htmlTag + '</td></tr>';
             } else {
-                table += '<div>' + val + '</div>';
+                table += '<span>' + val + '</span>';
             }
 
         });
@@ -288,9 +319,9 @@ define(['require', 'utils/Utils', 'modules/Modal', 'utils/Messages', 'utils/Enum
                 var className = "btn btn-action btn-sm btn-blue btn-icon",
                     deleteIcon = "";
                 if (obj.guid === tag.entityGuid) {
-                    deleteIcon = '<i class="fa fa-times" data-id="delete"  data-assetname="' + entityName + '"data-name="' + tag.typeName + '" data-type="tag" data-guid="' + obj.guid + '" ></i>';
+                    deleteIcon = '<i class="fa fa-times" data-id="delete"  data-assetname="' + entityName + '" data-name="' + tag.typeName + '" data-type="tag" data-guid="' + obj.guid + '" ></i>';
                 } else if (obj.guid !== tag.entityGuid && tag.entityStatus === "DELETED") {
-                    deleteIcon = '<i class="fa fa-times" data-id="delete"  data-assetname="' + entityName + '"data-name="' + tag.typeName + '" data-type="tag" data-entityguid="' + tag.entityGuid + '" data-guid="' + obj.guid + '" ></i>';
+                    deleteIcon = '<i class="fa fa-times" data-id="delete"  data-assetname="' + entityName + '" data-name="' + tag.typeName + '" data-type="tag" data-entityguid="' + tag.entityGuid + '" data-guid="' + obj.guid + '" ></i>';
                 } else {
                     className += " propagte-classification";
                 }
@@ -303,7 +334,7 @@ define(['require', 'utils/Utils', 'modules/Modal', 'utils/Messages', 'utils/Enum
                 ++count;
             });
         }
-        if (!Enums.entityStateReadOnly[obj.status]) {
+        if (!Enums.entityStateReadOnly[obj.status || obj.entityStatus]) {
             if (obj.guid) {
                 addTag += '<a href="javascript:void(0)" data-id="addTag" class="btn btn-action btn-sm assignTag" data-guid="' + obj.guid + '" ><i class="fa fa-plus"></i></a>';
             } else {
@@ -324,9 +355,10 @@ define(['require', 'utils/Utils', 'modules/Modal', 'utils/Messages', 'utils/Enum
             entityName = Utils.getName(obj);
         if (terms) {
             terms.map(function(term) {
+                var displayText = _.escape(term.displayText);
                 var className = "btn btn-action btn-sm btn-blue btn-icon",
-                    deleteIcon = '<i class="fa fa-times" data-id="delete"  data-assetname="' + entityName + '"data-name="' + term.displayText + '" data-type="term" data-guid="' + obj.guid + '" data-termGuid="' + term.termGuid + '" ></i>',
-                    termString = '<a class="' + className + '" data-id="termClick"><span title="' + _.escape(term.displayText) + '">' + _.escape(term.displayText) + '</span>' + deleteIcon + '</a>';
+                    deleteIcon = '<i class="fa fa-times" data-id="delete"  data-assetname="' + entityName + '" data-name="' + displayText + '" data-type="term" data-guid="' + obj.guid + '" data-termGuid="' + term.termGuid + '" ></i>',
+                    termString = '<a class="' + className + '" data-id="termClick"><span title="' + displayText + '">' + displayText + '</span>' + deleteIcon + '</a>';
                 if (count >= 1) {
                     popTerm += termString;
                 } else {
@@ -335,7 +367,7 @@ define(['require', 'utils/Utils', 'modules/Modal', 'utils/Messages', 'utils/Enum
                 ++count;
             });
         }
-        if (!Enums.entityStateReadOnly[obj.status]) {
+        if (!Enums.entityStateReadOnly[obj.status || obj.entityStatus]) {
             if (obj.guid) {
                 addTerm += '<a href="javascript:void(0)" data-id="addTerm" class="btn btn-action btn-sm assignTag" data-guid="' + obj.guid + '" ><i class="fa fa-plus"></i></a>';
             } else {
@@ -348,6 +380,7 @@ define(['require', 'utils/Utils', 'modules/Modal', 'utils/Messages', 'utils/Enum
         return '<div class="tagList btn-inline btn-fixed-width">' + termHtml + addTerm + '</div>';
     }
     CommonViewFunction.generateQueryOfFilter = function(value) {
+        value = Utils.getUrlState.getQueryParams();
         var entityFilters = CommonViewFunction.attributeFilter.extractUrl({ "value": value.entityFilters, "formatDate": true }),
             tagFilters = CommonViewFunction.attributeFilter.extractUrl({ "value": value.tagFilters, "formatDate": true }),
             queryArray = [];
@@ -357,7 +390,7 @@ define(['require', 'utils/Utils', 'modules/Modal', 'utils/Messages', 'utils/Enum
                 if (_.has(obj, 'condition')) {
                     return '&nbsp<span class="operator">' + obj.condition + '</span>&nbsp' + '(' + objToString(obj) + ')';
                 } else {
-                    return '<span class="key">' + _.escape(obj.id) + '</span>&nbsp<span class="operator">' + _.escape(obj.operator) + '</span>&nbsp<span class="value">' + _.escape(obj.value) + "</span>";
+                    return '<span class="key">' + (Enums.systemAttributes[obj.id] ? Enums.systemAttributes[obj.id] : _.escape(obj.id)) + '</span>&nbsp<span class="operator">' + _.escape(obj.operator) + '</span>&nbsp<span class="value">' + (Enums[obj.id] ? Enums[obj.id][obj.value] : _.escape(obj.value)) + "</span>";
                 }
             });
             return generatedQuery;
@@ -442,22 +475,34 @@ define(['require', 'utils/Utils', 'modules/Modal', 'utils/Messages', 'utils/Enum
                                 val = val.join(',');
                             } else if (k == "tagFilters") {
                                 if (classificationDefCollection) {
-                                    var classificationDef = classificationDefCollection.fullCollection.findWhere({ 'name': value[skey].classification })
-                                    attributeDefs = Utils.getNestedSuperTypeObj({
-                                        collection: classificationDefCollection,
-                                        attrMerge: true,
-                                        data: classificationDef.toJSON()
-                                    });
+                                    var classificationDef = classificationDefCollection.fullCollection.findWhere({ 'name': value[skey].classification }),
+                                        attributeDefs = [];
+                                    if (classificationDef) {
+                                        attributeDefs = Utils.getNestedSuperTypeObj({
+                                            collection: classificationDefCollection,
+                                            attrMerge: true,
+                                            data: classificationDef.toJSON()
+                                        });
+                                    }
+                                    if (Globals[value[skey].typeName]) {
+                                        attributeDefs = Globals[value[skey].typeName].attributeDefs;
+                                    }
                                 }
                                 val = CommonViewFunction.attributeFilter.generateUrl({ "value": val, "attributeDefs": attributeDefs });
                             } else if (k == "entityFilters") {
                                 if (entityDefCollection) {
                                     var entityDef = entityDefCollection.fullCollection.findWhere({ 'name': value[skey].typeName }),
+                                        attributeDefs = [];
+                                    if (entityDef) {
                                         attributeDefs = Utils.getNestedSuperTypeObj({
                                             collection: entityDefCollection,
                                             attrMerge: true,
                                             data: entityDef.toJSON()
                                         });
+                                    }
+                                    if (Globals[value[skey].typeName]) {
+                                        attributeDefs = Globals[value[skey].typeName].attributeDefs;
+                                    }
                                 }
                                 val = CommonViewFunction.attributeFilter.generateUrl({ "value": val, "attributeDefs": attributeDefs });
                             } else if (_.contains(["includeDE", "excludeST", "excludeSC"], k)) {
@@ -591,13 +636,13 @@ define(['require', 'utils/Utils', 'modules/Modal', 'utils/Messages', 'utils/Enum
                                 rule = {};
                             if (apiObj) {
                                 rule = { attributeName: temp[0], operator: mapUiOperatorToAPI(temp[1]), attributeValue: _.trim(temp[2]) }
-                                rule.attributeValue = rule.type === 'date' && formatDate && rule.attributeValue.length ? moment(parseInt(rule.attributeValue)).format('MM/DD/YYYY h:mm A') : rule.attributeValue;
+                                rule.attributeValue = rule.type === 'date' && formatDate && rule.attributeValue.length ? moment(parseInt(rule.attributeValue)).format(Globals.dateTimeFormat) : rule.attributeValue;
                             } else {
                                 rule = { id: temp[0], operator: temp[1], value: _.trim(temp[2]) }
                                 if (temp[3]) {
                                     rule['type'] = temp[3];
                                 }
-                                rule.value = rule.type === 'date' && formatDate && rule.value.length ? moment(parseInt(rule.value)).format('MM/DD/YYYY h:mm A') : rule.value;
+                                rule.value = rule.type === 'date' && formatDate && rule.value.length ? moment(parseInt(rule.value)).format(Globals.dateTimeFormat) : rule.value;
                             }
                             return rule;
                         }
@@ -666,7 +711,7 @@ define(['require', 'utils/Utils', 'modules/Modal', 'utils/Messages', 'utils/Enum
                 }
             });
             modal.on('ok', function() {
-                modal.$el.find('button.ok').attr("disabled", true);
+                modal.$el.find('button.ok').showButtonLoader();
                 CommonViewFunction.createEditGlossaryCategoryTermSubmit(_.extend({ "ref": view, "modal": modal }, options));
             });
             modal.on('closeModal', function() {
@@ -688,7 +733,7 @@ define(['require', 'utils/Utils', 'modules/Modal', 'utils/Messages', 'utils/Enum
                 collection = options.collection,
                 isGlossaryView = options.isGlossaryView,
                 data = ref.ui[(isGlossaryView ? "glossaryForm" : "categoryTermForm")].serializeArray().reduce(function(obj, item) {
-                    obj[item.name] = item.value;
+                    obj[item.name] = item.value.trim();
                     return obj;
                 }, {}),
                 newModel = new options.collection.model(),
@@ -702,8 +747,9 @@ define(['require', 'utils/Utils', 'modules/Modal', 'utils/Messages', 'utils/Enum
         var ajaxOptions = {
             silent: true,
             success: function(rModel, response) {
+                var msgType = model ? "editSuccessMessage" : "addSuccessMessage";
                 Utils.notifySuccess({
-                    content: messageType + ref.ui.name.val() + Messages[model ? "editSuccessMessage" : "addSuccessMessage"]
+                    content: messageType + ref.ui.name.val() + Messages.getAbbreviationMsg(false, msgType)
                 });
                 if (options.callback) {
                     options.callback(rModel);
@@ -711,7 +757,7 @@ define(['require', 'utils/Utils', 'modules/Modal', 'utils/Messages', 'utils/Enum
                 modal.trigger('closeModal');
             },
             cust_error: function() {
-                modal.$el.find('button.ok').attr("disabled", false);
+                modal.$el.find('button.ok').hideButtonLoader();
             }
         }
         if (model) {
@@ -758,8 +804,12 @@ define(['require', 'utils/Utils', 'modules/Modal', 'utils/Messages', 'utils/Enum
                 collection = options.collection,
                 model = options.model,
                 newModel = new options.collection.model(),
+                noticeRef = null,
                 ajaxOptions = {
                     success: function(rModel, response) {
+                        if (noticeRef) {
+                            noticeRef.remove();
+                        }
                         Utils.notifySuccess({
                             content: ((isCategoryView || isEntityView ? "Term" : "Category") + " association is removed successfully")
                         });
@@ -768,6 +818,9 @@ define(['require', 'utils/Utils', 'modules/Modal', 'utils/Messages', 'utils/Enum
                         }
                     },
                     cust_error: function() {
+                        if (noticeRef) {
+                            noticeRef.hideButtonLoader();
+                        }
                         if (options.hideLoader) {
                             options.hideLoader();
                         }
@@ -775,10 +828,13 @@ define(['require', 'utils/Utils', 'modules/Modal', 'utils/Messages', 'utils/Enum
                 },
                 notifyObj = {
                     modal: true,
+                    okCloses: false,
+                    okShowLoader: true,
                     text: options.msg,
                     title: options.titleMessage,
                     okText: options.buttonText,
-                    ok: function(argument) {
+                    ok: function(notice) {
+                        noticeRef = notice;
                         if (options.showLoader) {
                             options.showLoader();
                         }
@@ -801,7 +857,7 @@ define(['require', 'utils/Utils', 'modules/Modal', 'utils/Messages', 'utils/Enum
                             }));
                         }
                     },
-                    cancel: function(argument) {}
+                    cancel: function() {}
                 };
             Utils.notifyConfirm(notifyObj);
         }
@@ -853,7 +909,15 @@ define(['require', 'utils/Utils', 'modules/Modal', 'utils/Messages', 'utils/Enum
                             CommonViewFunction.restCsrfCustomHeader = header;
                             CommonViewFunction.restCsrfMethodsToIgnore = {};
                             methods.map(function(method) { CommonViewFunction.restCsrfMethodsToIgnore[method] = true; });
+                            var statusCodeErrorFn = function(error) {
+                                Utils.defaultErrorHandler(null, error)
+                            }
                             Backbone.$.ajaxSetup({
+                                statusCode: {
+                                    401: statusCodeErrorFn,
+                                    419: statusCodeErrorFn,
+                                    403: statusCodeErrorFn
+                                },
                                 beforeSend: CommonViewFunction.addRestCsrfCustomHeader
                             });
                         }
@@ -867,5 +931,111 @@ define(['require', 'utils/Utils', 'modules/Modal', 'utils/Messages', 'utils/Enum
             });
         }
     }
+    CommonViewFunction.CheckDuplicateAndEmptyInput = function(elements, datalist) {
+        var keyMap = new Map(),
+            validation = true,
+            hasDup = [];
+        for (var i = 0; i < elements.length; i++) {
+            var input = elements[i],
+                pEl = input.nextElementSibling,
+                classes = 'form-control',
+                val = input.value.trim();
+            pEl.innerText = "";
+
+            if (val === '') {
+                validation = false;
+                pEl.innerText = 'Required!';
+            } else if (val.includes(':')) {
+                validation = false;
+                var errorText = $(".errorMsg[data-id='charSupportMsg']").text();
+                if (errorText && errorText.length === 0) {
+                    pEl.innerText = "These special character '(:)' are not supported.";
+                }
+            } else {
+                if (input.tagName === 'INPUT') {
+                    var duplicates = datalist.filter(function(c) {
+                        return c.key === val;
+                    });
+                    if (keyMap.has(val) || duplicates.length > 1) {
+                        classes = 'form-control errorClass';
+                        hasDup.push('duplicate');
+                        pEl.innerText = 'Duplicate key';
+                    } else {
+                        keyMap.set(val, val);
+                    }
+                }
+            }
+            if (validation === false) {
+                classes = 'form-control errorClass';
+            }
+            input.setAttribute('class', classes);
+        }
+        return {
+            validation: validation,
+            hasDuplicate: (hasDup.length === 0 ? false : true)
+        };
+    }
+    CommonViewFunction.getRandomIdAndAnchor = function() {
+        var randomId = "collapse_" + parseInt((Math.random() * 100)) + "_" + new Date().getUTCMilliseconds();
+        return {
+            id: randomId,
+            anchor: "#" + randomId
+        };
+    }
+    CommonViewFunction.udKeysStringParser = function(udKeys) {
+        var o = {};
+        _.each(udKeys.split(','), function(udKey) {
+            var ud = udKey.split(':');
+            o[ud[0]] = ud[1];
+        })
+        return o;
+    }
+    CommonViewFunction.udKeysObjectToStringParser = function(udKeys) {
+        var list = _.map(udKeys, function(udKey) {
+            var t = udKey.key + ':' + udKey.value;
+            return t;
+        });
+        return list.join(',');
+    }
+    CommonViewFunction.fetchRootEntityAttributes = function(options) {
+        $.ajax({
+            url: options.url,
+            methods: 'GET',
+            dataType: 'json',
+            cache: true,
+            success: function(response) {
+                if (response) {
+                    _.each(options.entity, function(rootEntity) {
+                        Globals[rootEntity] = $.extend(true, {}, response, { name: rootEntity, guid: rootEntity });
+                    });
+                }
+            },
+            complete: function(response) {
+                if (options.callback) {
+                    options.callback(response);
+                }
+            }
+        });
+    }
+    CommonViewFunction.fetchRootClassificationAttributes = function(options) {
+        $.ajax({
+            url: options.url,
+            methods: 'GET',
+            dataType: 'json',
+            cache: true,
+            success: function(response) {
+                if (response) {
+                    _.each(options.classification, function(rootClassification) {
+                        Globals[rootClassification] = $.extend(true, {}, response, { name: rootClassification, guid: rootClassification });
+                    });
+                }
+            },
+            complete: function(response) {
+                if (options.callback) {
+                    options.callback(response);
+                }
+            }
+        });
+    }
     return CommonViewFunction;
 });
\ No newline at end of file
