@@ -37,14 +37,27 @@ define(['require', 'utils/Enums', 'utils/Utils', 'underscore'], function(require
         classificationDefApiUrl: function(name) {
             return this.getDefApiUrl('classification', name);
         },
+        businessMetadataDefApiUrl: function(name) {
+            return this.getDefApiUrl('business_metadata', name);
+        },
         enumDefApiUrl: function(name) {
             return this.getDefApiUrl('enum', name);
         },
-        entityCountApi: function(){
+        metricsApiUrl: function() {
             return this.baseUrl + '/admin/metrics'
         },
+        migrationStatusApiUrl: function() {
+            return this.baseUrl + '/admin/status'
+        },
+        rootEntityDefUrl: function(name) {
+            return this.baseUrlV2 + '/types/entitydef/name/' + name;
+        },
+        rootClassificationDefUrl: function(name) {
+            return this.baseUrlV2 + '/types/classificationdef/name/' + name;
+        },
         getDefApiUrl: function(type, name) {
-            var defApiUrl = this.typedefsUrl(), defUrl;
+            var defApiUrl = this.typedefsUrl(),
+                defUrl;
             if (name) {
                 defUrl = defApiUrl.def + '/name/' + name;
             } else {
@@ -78,6 +91,9 @@ define(['require', 'utils/Enums', 'utils/Utils', 'underscore'], function(require
                 return entitiesUrl += '?minExtInfo=' + (minExtInfo);
             }
         },
+        entityLabelsAPIUrl: function(guid) {
+            return this.entitiesApiUrl({ guid: guid }) + "/labels";
+        },
         entityHeaderApiUrl: function(guid) {
             return this.entitiesApiUrl({ guid: guid }) + "/header"
         },
@@ -89,6 +105,11 @@ define(['require', 'utils/Enums', 'utils/Utils', 'underscore'], function(require
                 return this.baseUrlV2 + '/entity/bulk/classification';
             }
         },
+        entitiesBusinessMetadataApiUrl: function(guid) {
+            if (guid) {
+                return this.baseUrlV2 + '/entity/guid/' + guid + '/businessmetadata?isOverwrite=true';
+            }
+        },
         entityCollectionaudit: function(guid) {
             return this.baseUrlV2 + '/entity/' + guid + '/audit';
         },
@@ -175,6 +196,18 @@ define(['require', 'utils/Enums', 'utils/Utils', 'underscore'], function(require
                 return glossaryUrl;
             }
         },
+        glossaryImportTempUrl: function() {
+            return this.glossaryApiUrl() + '/import/template';
+        },
+        glossaryImportUrl: function() {
+            return this.glossaryApiUrl() + '/import';
+        },
+        businessMetadataImportTempUrl: function() {
+            return this.entitiesApiUrl() + '/businessmetadata/import/template';
+        },
+        businessMetadataImportUrl: function() {
+            return this.entitiesApiUrl() + '/businessmetadata/import';
+        },
         categoryApiUrl: function(options) {
             var guid = options && options.guid,
                 list = options && options.list,
@@ -216,6 +249,9 @@ define(['require', 'utils/Enums', 'utils/Utils', 'underscore'], function(require
         },
         sessionApiUrl: function() {
             return this.baseUrl + '/admin/session';
+        },
+        adminApiUrl: function() {
+            return this.baseUrl + '/admin/audits';
         }
 
     });
