@@ -22,20 +22,6 @@ define(['require', 'utils/Globals', 'pnotify', 'utils/Messages', 'utils/Enums',
     var Utils = {};
     var prevNetworkErrorTime = 0;
 
-    Utils.escapeHtml = function(string) {
-        var entityMap = {
-            "&": "&amp;",
-            "<": "&lt;",
-            ">": "&gt;",
-            '"': '&quot;',
-            "'": '&#39;',
-            "/": '&#x2F;'
-        };
-        return String(string).replace(/[&<>"'\/]/g, function(s) {
-            return entityMap[s];
-        });
-    }
-
     Utils.generatePopover = function(options) {
         if (options.el) {
             var defaultObj = {
@@ -184,7 +170,7 @@ define(['require', 'utils/Globals', 'pnotify', 'utils/Messages', 'utils/Enums',
                 confirm: true,
                 buttons: [{
                         text: options.cancelText || 'Cancel',
-                        addClass: 'btn-action btn-md',
+                        addClass: 'btn-action btn-md cancel',
                         click: function(notice) {
                             options.cancel(notice);
                             notice.remove();
@@ -192,10 +178,24 @@ define(['require', 'utils/Globals', 'pnotify', 'utils/Messages', 'utils/Enums',
                     },
                     {
                         text: options.okText || 'Ok',
-                        addClass: 'btn-atlas btn-md',
+                        addClass: 'btn-atlas btn-md ok',
                         click: function(notice) {
-                            options.ok(notice);
-                            notice.remove();
+                            if (options.ok) {
+                                options.ok($.extend({}, notice, {
+                                    hideButtonLoader: function() {
+                                        notice.container.find("button.ok").hideButtonLoader();
+                                    },
+                                    showButtonLoader: function() {
+                                        notice.container.find("button.ok").showButtonLoader();
+                                    }
+                                }));
+                            }
+                            if (options.okShowLoader) {
+                                notice.container.find("button.ok").showButtonLoader();
+                            }
+                            if (options.okCloses !== false) {
+                                notice.remove();
+                            }
                         }
                     }
                 ]
@@ -218,22 +218,24 @@ define(['require', 'utils/Globals', 'pnotify', 'utils/Messages', 'utils/Enums',
             }
         });
     }
-    Utils.defaultErrorHandler = function(model, error) {
+    Utils.defaultErrorHandler = function(model, error, options) {
+        var skipDefaultError = null,
+            defaultErrorMessage = null;
+        if (options) {
+            skipDefaultError = options.skipDefaultError;
+            defaultErrorMessage = options.defaultErrorMessage;
+        }
+        var redirectToLoginPage = function() {
+            Utils.localStorage.setValue("last_ui_load", "v1");
+            window.location = 'login.jsp';
+        }
         if (error && error.status) {
             if (error.status == 401) {
-                window.location = 'login.jsp'
+                redirectToLoginPage();
             } else if (error.status == 419) {
-                window.location = 'login.jsp'
+                redirectToLoginPage();
             } else if (error.status == 403) {
-                var message = "You are not authorized";
-                if (error.statusText) {
-                    try {
-                        message = JSON.parse(error.statusText).AuthorizationError;
-                    } catch (err) {}
-                    Utils.notifyError({
-                        content: message
-                    });
-                }
+                Utils.serverErrorHandler(error, "You are not authorized");
             } else if (error.status == "0" && error.statusText != "abort") {
                 var diffTime = (new Date().getTime() - prevNetworkErrorTime);
                 if (diffTime > 3000) {
@@ -243,22 +245,23 @@ define(['require', 'utils/Globals', 'pnotify', 'utils/Messages', 'utils/Enums',
                             "It seems you are not connected to the internet. Please check your internet connection and try again"
                     });
                 }
-            } else {
-                Utils.serverErrorHandler(model, error)
+            } else if (skipDefaultError !== true) {
+                Utils.serverErrorHandler(error, defaultErrorMessage);
             }
-        } else {
-            Utils.serverErrorHandler(model, error)
+        } else if (skipDefaultError !== true) {
+            Utils.serverErrorHandler(error, defaultErrorMessage);
         }
     };
-    Utils.serverErrorHandler = function(model, response) {
-        var responseJSON = response ? response.responseJSON : response;
-        if (response && responseJSON && (responseJSON.errorMessage || responseJSON.message || responseJSON.error)) {
+    Utils.serverErrorHandler = function(response, defaultErrorMessage) {
+        var responseJSON = response ? response.responseJSON : response,
+            message = defaultErrorMessage ? defaultErrorMessage : Messages.defaultErrorMessage
+        if (response && responseJSON) {
+            message = responseJSON.errorMessage || responseJSON.message || responseJSON.error || message
+        }
+        var existingError = $(".ui-pnotify-container.alert-danger .ui-pnotify-text").text();
+        if (existingError !== message) {
             Utils.notifyError({
-                content: responseJSON.errorMessage || responseJSON.message || responseJSON.error
-            });
-        } else {
-            Utils.notifyError({
-                content: Messages.defaultErrorMessage
+                content: message
             });
         }
     };
@@ -335,6 +338,8 @@ define(['require', 'utils/Globals', 'pnotify', 'utils/Messages', 'utils/Enums',
                     urlUpdate['searchUrl'] = options.url;
                 } else if (Utils.getUrlState.isGlossaryTab(options.url)) {
                     urlUpdate['glossaryUrl'] = options.url;
+                } else if (Utils.getUrlState.isAdministratorTab(options.url)) {
+                    urlUpdate['administratorUrl'] = options.url;
                 }
                 $.extend(Globals.saveApplicationState.tabState, urlUpdate);
             }
@@ -370,12 +375,22 @@ define(['require', 'utils/Globals', 'pnotify', 'utils/Messages', 'utils/Enums',
                 matchString: "tag"
             });
         },
+        isBSDetail: function(url) {
+            var quey = this.getQueryUrl(url);
+            return (quey.queyParams[0].indexOf("administrator/businessMetadata")) > -1 ? true : false;
+        },
         isSearchTab: function(url) {
             return this.checkTabUrl({
                 url: url,
                 matchString: "search"
             });
         },
+        isAdministratorTab: function(url) {
+            return this.checkTabUrl({
+                url: url,
+                matchString: "administrator"
+            });
+        },
         isGlossaryTab: function(url) {
             return this.checkTabUrl({
                 url: url,
@@ -535,6 +550,27 @@ define(['require', 'utils/Globals', 'pnotify', 'utils/Messages', 'utils/Enums',
         }
 
     }
+
+    Utils.backButtonClick = function() {
+        var queryParams = Utils.getUrlState.getQueryParams(),
+            urlPath = "searchUrl";
+        if (queryParams && queryParams.from) {
+            if (queryParams.from == "classification") {
+                urlPath = "tagUrl";
+            } else if (queryParams.from == "glossary") {
+                urlPath = "glossaryUrl";
+            } else if (queryParams.from == "bm") {
+                urlPath = "administratorUrl";
+            }
+        }
+        Utils.setUrl({
+            url: Globals.saveApplicationState.tabState[urlPath],
+            mergeBrowserUrl: false,
+            trigger: true,
+            updateTabState: true
+        });
+    }
+
     Utils.showTitleLoader = function(loaderEl, titleBoxEl) {
         loaderEl.css ? loaderEl.css({
             'display': 'block',
@@ -556,7 +592,10 @@ define(['require', 'utils/Globals', 'pnotify', 'utils/Messages', 'utils/Enums',
         var mergeObject = function(obj) {
             if (obj) {
                 if (obj.attributes) {
-                    Utils.findAndMergeRefEntity(obj.attributes, referredEntities);
+                    Utils.findAndMergeRefEntity({
+                        "attributeObject": obj.attributes,
+                        "referredEntities": referredEntities
+                    });
                 } else if (referredEntities[obj.guid]) {
                     _.extend(obj, referredEntities[obj.guid]);
                 }
@@ -576,23 +615,52 @@ define(['require', 'utils/Globals', 'pnotify', 'utils/Messages', 'utils/Enums',
             });
         }
     }
+
+    Utils.findAndMergeRelationShipEntity = function(options) {
+        var attributeObject = options.attributeObject,
+            relationshipAttributes = options.relationshipAttributes;
+        _.each(attributeObject, function(val, key) {
+            var attributVal = val;
+            if (relationshipAttributes && relationshipAttributes[key]) {
+                var relationShipVal = relationshipAttributes[key];
+                if (_.isObject(val)) {
+                    if (_.isArray(val)) {
+                        _.each(val, function(attr) {
+                            if (attr && attr.attributes === undefined) {
+                                var entityFound = _.find(relationShipVal, { guid: attr.guid });
+                                if (entityFound) {
+                                    attr.attributes = _.omit(entityFound, 'typeName', 'guid', 'entityStatus');
+                                    attr.status = entityFound.entityStatus;
+                                }
+                            }
+                        });
+                    } else if (relationShipVal && val.attributes === undefined) {
+                        val.attributes = _.omit(relationShipVal, 'typeName', 'guid', 'entityStatus');
+                        val.status = relationShipVal.entityStatus;
+                    }
+                }
+            }
+        })
+    }
+
     Utils.getNestedSuperTypes = function(options) {
         var data = options.data,
             collection = options.collection,
             superTypes = [];
 
         var getData = function(data, collection) {
-            superTypes = superTypes.concat(data.superTypes);
-
-            if (data.superTypes && data.superTypes.length) {
-                _.each(data.superTypes, function(superTypeName) {
-                    if (collection.fullCollection) {
-                        var collectionData = collection.fullCollection.findWhere({ name: superTypeName }).toJSON();
-                    } else {
-                        var collectionData = collection.findWhere({ name: superTypeName }).toJSON();
-                    }
-                    return getData(collectionData, collection);
-                });
+            if (data) {
+                superTypes = superTypes.concat(data.superTypes);
+                if (data.superTypes && data.superTypes.length) {
+                    _.each(data.superTypes, function(superTypeName) {
+                        if (collection.fullCollection) {
+                            var collectionData = collection.fullCollection.findWhere({ name: superTypeName }).toJSON();
+                        } else {
+                            var collectionData = collection.findWhere({ name: superTypeName }).toJSON();
+                        }
+                        getData(collectionData, collection);
+                    });
+                }
             }
         }
         getData(data, collection);
@@ -785,7 +853,7 @@ define(['require', 'utils/Globals', 'pnotify', 'utils/Messages', 'utils/Enums',
         return regexp.test(url);
     }
 
-    Utils.JSONPrettyPrint = function(obj) {
+    Utils.JSONPrettyPrint = function(obj, getValue) {
         var replacer = function(match, pIndent, pKey, pVal, pEnd) {
                 var key = '<span class=json-key>';
                 var val = '<span class=json-value>';
@@ -794,7 +862,7 @@ define(['require', 'utils/Globals', 'pnotify', 'utils/Messages', 'utils/Enums',
                 if (pKey)
                     r = r + key + pKey.replace(/[": ]/g, '') + '</span>: ';
                 if (pVal)
-                    r = r + (pVal[0] == '"' ? str : val) + pVal + '</span>';
+                    r = r + (pVal[0] == '"' ? str : val) + getValue(pVal) + '</span>';
                 return r + (pEnd || '');
             },
             jsonLine = /^( *)("[\w]+": )?("[^"]*"|[\w.+-]*)?([,[{])?$/mg;
@@ -836,7 +904,14 @@ define(['require', 'utils/Globals', 'pnotify', 'utils/Messages', 'utils/Enums',
         } else {
             tableEl.addClass('hide-empty-value');
         }
-
+    }
+    $.fn.showButtonLoader = function() {
+        $(this).attr("disabled", "true").addClass('button-loader');
+        $(this).siblings("button.cancel").prop("disabled", true);
+    }
+    $.fn.hideButtonLoader = function() {
+        $(this).removeClass('button-loader').removeAttr("disabled");
+        $(this).siblings("button.cancel").prop("disabled", false);
     }
     return Utils;
 });
\ No newline at end of file
