@@ -39,7 +39,6 @@ define(['require',
 
             /** ui selector cache */
             ui: {
-                auditCreate: "[data-id='auditCreate']",
                 previousAuditData: "[data-id='previousAuditData']",
                 nextAuditData: "[data-id='nextAuditData']",
                 pageRecordText: "[data-id='pageRecordText']",
@@ -48,7 +47,6 @@ define(['require',
             /** ui events hash */
             events: function() {
                 var events = {};
-                events["click " + this.ui.auditCreate] = "onClickAuditCreate";
                 events["click " + this.ui.nextAuditData] = "onClickNextAuditData";
                 events["click " + this.ui.previousAuditData] = "onClickPreviousAuditData";
                 return events;
@@ -70,6 +68,7 @@ define(['require',
                     includeFilter: false,
                     includePagination: false,
                     includePageSize: false,
+                    includeAtlasTableSorting: true,
                     includeFooterRecords: false,
                     gridOpts: {
                         className: "table table-hover backgrid table-quickMenu",
@@ -178,17 +177,41 @@ define(['require',
             getAuditTableColumns: function() {
                 var that = this;
                 return this.entityCollection.constructor.getTableCols({
+
+                    tool: {
+                        label: "",
+                        cell: "html",
+                        editable: false,
+                        sortable: false,
+                        fixWidth: "20",
+                        cell: Backgrid.ExpandableCell,
+                        accordion: false,
+                        expand: function(el, model) {
+                            el.attr('colspan', '4');
+                            require([
+                                'views/audit/CreateAuditTableLayoutView',
+                            ], function(CreateAuditTableLayoutView) {
+
+                                that.action = model.get('action');
+                                // $(el.target).attr('disabled', true);
+                                var eventModel = that.entityCollection.fullCollection.findWhere({ 'eventKey': model.get('eventKey') }).toJSON(),
+                                    collectionModel = new that.entityCollection.model(eventModel),
+                                    view = new CreateAuditTableLayoutView({ guid: that.guid, entityModel: collectionModel, action: that.action, entity: that.entity, entityName: that.entityName, attributeDefs: that.attributeDefs });
+                                view.render();
+                                $(el).append($('<div>').html(view.$el));
+                            });
+
+                        }
+                    },
                     user: {
                         label: "Users",
                         cell: "html",
                         editable: false,
-                        sortable: false,
                     },
                     timestamp: {
                         label: "Timestamp",
                         cell: "html",
                         editable: false,
-                        sortable: false,
                         formatter: _.extend({}, Backgrid.CellFormatter.prototype, {
                             fromRaw: function(rawValue, model) {
                                 return new Date(rawValue);
@@ -199,7 +222,6 @@ define(['require',
                         label: "Actions",
                         cell: "html",
                         editable: false,
-                        sortable: false,
                         formatter: _.extend({}, Backgrid.CellFormatter.prototype, {
                             fromRaw: function(rawValue, model) {
                                 if (Enums.auditAction[rawValue]) {
@@ -209,50 +231,10 @@ define(['require',
                                 }
                             }
                         })
-                    },
-                    tool: {
-                        label: "Tools",
-                        cell: "html",
-                        editable: false,
-                        sortable: false,
-                        formatter: _.extend({}, Backgrid.CellFormatter.prototype, {
-                            fromRaw: function(rawValue, model) {
-                                return '<div class="btn btn-action btn-sm" data-id="auditCreate" data-action="' + Enums.auditAction[model.get('action')] + '" data-modalId="' + model.get('eventKey') + '">Detail</div>';
-                            }
-                        })
-                    },
-
+                    }
                 }, this.entityCollection);
 
             },
-            onClickAuditCreate: function(e) {
-                var that = this;
-                require([
-                    'modules/Modal',
-                    'views/audit/CreateAuditTableLayoutView',
-                ], function(Modal, CreateAuditTableLayoutView) {
-                    that.action = $(e.target).data("action");
-                    $(e.target).attr('disabled', true);
-                    var eventModel = that.entityCollection.fullCollection.findWhere({ 'eventKey': $(e.currentTarget).data('modalid') }).toJSON(),
-                        collectionModel = new that.entityCollection.model(eventModel),
-                        view = new CreateAuditTableLayoutView({ guid: that.guid, entityModel: collectionModel, action: that.action, entity: that.entity, entityName: that.entityName, attributeDefs: that.attributeDefs });
-                    var modal = new Modal({
-                        title: that.action,
-                        content: view,
-                        okCloses: true,
-                        showFooter: true,
-                    }).open();
-                    view.on('closeModal', function() {
-                        modal.trigger('cancel');
-                    });
-                    view.$el.on('click', 'td a', function() {
-                        modal.trigger('cancel');
-                    });
-                    view.on('hidden.bs.modal', function() {
-                        that.$('.btn-action[data-id="auditCreate"]').attr('disabled', false);
-                    });
-                });
-            },
             onClickNextAuditData: function() {
                 var that = this;
                 this.ui.previousAuditData.removeAttr("disabled");
