@@ -74,7 +74,7 @@ define(['require',
              * @constructs
              */
             initialize: function(options) {
-                _.extend(this, _.pick(options, 'guid', 'callback', 'showLoader', 'entityDefCollection', 'typeHeaders'));
+                _.extend(this, _.pick(options, 'guid', 'callback', 'showLoader', 'entityDefCollection', 'typeHeaders', 'searchVent'));
                 var that = this,
                     entityTitle, okLabel;
                 this.selectStoreCollection = new Backbone.Collection();
@@ -103,6 +103,7 @@ define(['require',
                 }).open();
                 this.modal.$el.find('button.ok').attr("disabled", true);
                 this.modal.on('ok', function(e) {
+                    that.modal.$el.find('button.ok').showButtonLoader();
                     that.okButton();
                 });
                 this.modal.on('closeModal', function() {
@@ -435,9 +436,15 @@ define(['require',
                 var that = this;
                 this.$('input[data-type="date"]').each(function() {
                     if (!$(this).data('daterangepicker')) {
-                        var dateObj = { "singleDatePicker": true, "showDropdowns": true };
+                        var dateObj = {
+                            "singleDatePicker": true,
+                            "showDropdowns": true,
+                            locale: {
+                                format: Globals.dateFormat
+                            }
+                        };
                         if (that.guid) {
-                            dateObj["startDate"] = this.value
+                            dateObj["startDate"] = new Date(Number(this.value));
                         }
                         $(this).daterangepicker(dateObj);
                     }
@@ -461,7 +468,7 @@ define(['require',
             },
             initializeValidation: function() {
                 // IE9 allow input type number
-                var regex = /^[0-9]*((?=[^.]|$))?$/, // allow only numbers [0-9] 
+                var regex = /^[0-9]*((?=[^.]|$))?$/, // allow only numbers [0-9]
                     removeText = function(e, value) {
                         if (!regex.test(value)) {
                             var txtfld = e.currentTarget;
@@ -470,7 +477,7 @@ define(['require',
                         }
                     }
                 this.$('input[data-type="int"],input[data-type="long"]').on('keydown', function(e) {
-                    // allow only numbers [0-9] 
+                    // allow only numbers [0-9]
                     if (!regex.test(e.currentTarget.value)) {
                         return false;
                     }
@@ -495,10 +502,10 @@ define(['require',
             },
             getContainer: function(object) {
                 var value = object.value,
-                    entityLabel = this.capitalize(value.name);
+                    entityLabel = this.capitalize(_.escape(value.name));
 
                 return '<div class=" row ' + value.isOptional + '"><span class="col-sm-3">' +
-                    '<label><span class="' + (value.isOptional ? 'true' : 'false required') + '">' + entityLabel + '</span><span class="center-block ellipsis text-gray" title="Data Type : ' + value.typeName + '">' + '(' + Utils.escapeHtml(value.typeName) + ')' + '</span></label></span>' +
+                    '<label><span class="' + (value.isOptional ? 'true' : 'false required') + '">' + entityLabel + '</span><span class="center-block ellipsis-with-margin text-gray" title="Data Type : ' + value.typeName + '">' + '(' + _.escape(value.typeName) + ')' + '</span></label></span>' +
                     '<span class="col-sm-9">' + (this.getElement(object)) +
                     '</input></span></div>';
             },
@@ -512,7 +519,8 @@ define(['require',
             },
             getSelect: function(object) {
                 var value = object.value,
-                    entityValue = object.entityValue,
+                    name = _.escape(value.name),
+                    entityValue = _.escape(object.entityValue),
                     isAttribute = object.isAttribute,
                     isRelation = object.isRelation;
                 if (value.typeName === "boolean") {
@@ -520,7 +528,7 @@ define(['require',
                         '" data-type="' + value.typeName +
                         '" data-attribute="' + isAttribute +
                         '" data-relation="' + isRelation +
-                        '" data-key="' + value.name +
+                        '" data-key="' + name +
                         '" data-id="entityInput">' +
                         '<option value="">--Select true or false--</option><option value="true">true</option>' +
                         '<option value="false">false</option></select>';
@@ -535,14 +543,15 @@ define(['require',
                         '" data-type="' + value.typeName +
                         '" data-attribute="' + isAttribute +
                         '" data-relation="' + isRelation +
-                        '" data-key="' + value.name +
+                        '" data-key="' + name +
                         '" data-id="entitySelectData" data-queryData="' + splitTypeName + '">' + (this.guid ? entityValue : "") + '</select>';
                 }
 
             },
             getTextArea: function(object) {
                 var value = object.value,
-                    setValue = object.entityValue,
+                    name = _.escape(value.name),
+                    setValue = _.escape(object.entityValue),
                     isAttribute = object.isAttribute,
                     isRelation = object.isRelation,
                     structType = object.structType;
@@ -557,25 +566,26 @@ define(['require',
 
                 return '<textarea class="form-control entityInputBox ' + (value.isOptional === true ? "false" : "true") + '"' +
                     ' data-type="' + value.typeName + '"' +
-                    ' data-key="' + value.name + '"' +
+                    ' data-key="' + name + '"' +
                     ' data-attribute="' + isAttribute + '"' +
                     ' data-relation="' + isRelation + '"' +
-                    ' placeholder="' + value.name + '"' +
+                    ' placeholder="' + name + '"' +
                     ' data-id="entityInput">' + setValue + '</textarea>';
 
             },
             getInput: function(object) {
                 var value = object.value,
-                    entityValue = object.entityValue,
+                    name = _.escape(value.name),
+                    entityValue = _.escape(object.entityValue),
                     isAttribute = object.isAttribute,
                     isRelation = object.isRelation;
                 return '<input class="form-control entityInputBox ' + (value.isOptional === true ? "false" : "true") + '"' +
                     ' data-type="' + value.typeName + '"' +
                     ' value="' + entityValue + '"' +
-                    ' data-key="' + value.name + '"' +
+                    ' data-key="' + name + '"' +
                     ' data-attribute="' + isAttribute + '"' +
                     ' data-relation="' + isRelation + '"' +
-                    ' placeholder="' + value.name + '"' +
+                    ' placeholder="' + name + '"' +
                     ' data-id="entityInput">';
             },
             getElement: function(object) {
@@ -594,9 +604,9 @@ define(['require',
                         }
                         if (value.typeName === "date") {
                             if (dataValue) {
-                                entityValue = moment(dataValue).format("MM/DD/YYYY");
+                                entityValue = moment(dataValue);
                             } else {
-                                entityValue = moment().format("MM/DD/YYYY");
+                                entityValue = moment().format(Globals.dateFormat);
                             }
                         }
                     }
@@ -687,6 +697,7 @@ define(['require',
                                     $(this).addClass('errorClass');
                                 }
                                 that.hideLoader();
+                                that.modal.$el.find('button.ok').hideButtonLoader();
                                 throw new Error("Please fill the required fields");
                                 return;
                             }
@@ -762,19 +773,27 @@ define(['require',
                         data: JSON.stringify(entityJson),
                         type: "POST",
                         success: function(model, response) {
+                            that.modal.$el.find('button.ok').hideButtonLoader();
                             that.modal.close();
+                            var msgType = (model.mutatedEntities && model.mutatedEntities.UPDATE) ? 'editSuccessMessage' : 'addSuccessMessage';
                             Utils.notifySuccess({
-                                content: "Entity " + Messages[that.guid ? 'editSuccessMessage' : 'addSuccessMessage']
+                                content: "Entity " + Messages.getAbbreviationMsg(false, msgType)
                             });
                             if (that.guid && that.callback) {
                                 that.callback();
                             } else {
-                                if (model.mutatedEntities && model.mutatedEntities.CREATE && _.isArray(model.mutatedEntities.CREATE) && model.mutatedEntities.CREATE[0] && model.mutatedEntities.CREATE[0].guid) {
-                                    Utils.setUrl({
-                                        url: '#!/detailPage/' + (model.mutatedEntities.CREATE[0].guid),
-                                        mergeBrowserUrl: false,
-                                        trigger: true
-                                    });
+                                if (model.mutatedEntities) {
+                                    var mutatedEntities = model.mutatedEntities.CREATE || model.mutatedEntities.UPDATE;
+                                    if (mutatedEntities && _.isArray(mutatedEntities) && mutatedEntities[0] && mutatedEntities[0].guid) {
+                                        Utils.setUrl({
+                                            url: '#!/detailPage/' + (mutatedEntities[0].guid),
+                                            mergeBrowserUrl: false,
+                                            trigger: true
+                                        });
+                                    }
+                                    if (that.searchVent) {
+                                        that.searchVent.trigger('entityList:refresh');
+                                    }
                                 }
                             }
                         },
@@ -782,9 +801,9 @@ define(['require',
                             that.hideLoader({
                                 editVisiblityOfEntitySelectionBox: true
                             });
+                            that.modal.$el.find('button.ok').hideButtonLoader();
                         }
                     });
-
                 } catch (e) {
                     Utils.notifyError({
                         content: e.message
