@@ -22,9 +22,10 @@ define(['require',
     'utils/Utils',
     'utils/Messages',
     'utils/Globals',
+    'utils/UrlLinks',
     'utils/CommonViewFunction',
     'jstree'
-], function(require, Backbone, GlossaryLayoutViewTmpl, Utils, Messages, Globals, CommonViewFunction) {
+], function(require, Backbone, GlossaryLayoutViewTmpl, Utils, Messages, Globals, UrlLinks, CommonViewFunction) {
     'use strict';
 
     var GlossaryLayoutView = Backbone.Marionette.LayoutView.extend(
@@ -39,6 +40,7 @@ define(['require',
             templateHelpers: function() {
                 return {
                     isAssignView: this.isAssignView,
+                    importTmplUrl: UrlLinks.glossaryImportTempUrl(),
                     isAssignAttributeRelationView: this.isAssignAttributeRelationView
                 };
             },
@@ -51,7 +53,10 @@ define(['require',
                 searchCategory: "[data-id='searchCategory']",
                 glossaryView: 'input[name="glossaryView"]',
                 termTree: "[data-id='termTree']",
-                categoryTree: "[data-id='categoryTree']"
+                categoryTree: "[data-id='categoryTree']",
+                importGlossary: "[data-id='importGlossary']",
+                glossaryTreeLoader: "[data-id='glossaryTreeLoader']",
+                glossaryTreeView: "[data-id='glossaryTreeView']"
             },
             /** ui events hash */
             events: function() {
@@ -74,9 +79,13 @@ define(['require',
                         }
                     })
                 };
-                events["click " + this.ui.refreshGlossary] = 'getGlossary';
+                events["click " + this.ui.refreshGlossary] = function() {
+                    this.ui.refreshGlossary.attr("disabled", true);
+                    this.getGlossary();
+                };
+                events["click " + this.ui.importGlossary] = 'onClickImportGlossary';
                 events["keyup " + this.ui.searchTerm] = function() {
-                    this.ui.termTree.jstree("search", this.ui.searchTerm.val());
+                    this.ui.termTree.jstree("search", _.escape(this.ui.searchTerm.val()));
                 };
                 events["keyup " + this.ui.searchCategory] = function() {
                     this.ui.categoryTree.jstree("search", this.ui.searchCategory.val());
@@ -88,7 +97,7 @@ define(['require',
              * @constructs
              */
             initialize: function(options) {
-                _.extend(this, _.pick(options, 'associatedTerms', 'guid', 'value', 'glossaryCollection', 'glossary', 'isAssignTermView', 'isAssignCategoryView', 'isAssignEntityView', 'isAssignAttributeRelationView'));
+                _.extend(this, _.pick(options, 'associatedTerms', 'guid', 'value', 'glossaryCollection', 'glossary', 'isAssignTermView', 'isAssignCategoryView', 'isAssignEntityView', 'isAssignAttributeRelationView', 'importVent'));
                 this.viewType = "term";
                 this.isAssignView = this.isAssignTermView || this.isAssignCategoryView || this.isAssignEntityView || this.isAssignAttributeRelationView;
                 this.bindEvents();
@@ -106,6 +115,8 @@ define(['require',
                 this.listenTo(this.glossaryCollection.fullCollection, "reset add change", function(skip) {
                     this.generateTree();
                     this.setValues();
+                    this.changeLoaderState(false);
+                    this.ui.refreshGlossary.attr("disabled", false);
                 }, this);
                 this.listenTo(this.glossaryCollection, "update:details", function(options) {
                     var isGlossaryUpdate = options.isGlossaryUpdate;
@@ -123,6 +134,7 @@ define(['require',
                             this.setValues({ trigger: false });
                         }
                     }
+                    this.changeLoaderState(false);
                 }, this);
                 if (!this.isAssignView) {
                     $('body').on('click', '.termPopoverOptions li, .categoryPopoverOptions li', function(e) {
@@ -130,8 +142,14 @@ define(['require',
                         that[$(this).find('a').data('fn')](e)
                     });
                 }
+                if (this.importVent) {
+                    this.importVent.on("Import:Glossary:Update", function(options) {
+                        that.getGlossary();
+                    });
+                }
             },
             onRender: function() {
+                this.changeLoaderState(true);
                 if (this.isAssignCategoryView) {
                     this.$('.category-view').show();
                     this.$('.term-view').hide();
@@ -142,6 +160,15 @@ define(['require',
                     this.getGlossary();
                 }
             },
+            changeLoaderState: function(showLoader) {
+                if (showLoader) {
+                    this.ui.glossaryTreeLoader.show();
+                    this.ui.glossaryTreeView.hide();
+                } else {
+                    this.ui.glossaryTreeLoader.hide();
+                    this.ui.glossaryTreeView.show();
+                }
+            },
             setValues: function(options) {
                 if (this.viewType == "category") {
                     if (!this.ui.glossaryView.prop("checked")) {
@@ -209,6 +236,7 @@ define(['require',
                 }
             },
             getGlossary: function() {
+                this.changeLoaderState(true);
                 this.glossaryCollection.fetch({ reset: true });
             },
             generateCategoryData: function(options) {
@@ -449,8 +477,11 @@ define(['require',
                     treeLoaded = function(options) {
                         if (that.query[options.type].isNodeNotFoundAtLoad == true) {
                             var id = that.glossary.selectedItem.guid;
-                            options.$el.jstree('activate_node', id);
+                            if (id) {
+                                options.$el.jstree('activate_node', id);
+                            }
                         }
+                        that.changeLoaderState(false);
                     },
                     createAction = function(options) {
                         var $el = options.el,
@@ -509,9 +540,7 @@ define(['require',
                             }).on("clear_search.jstree", function(e, data) {
                                 createAction(_.extend({}, options, data));
                             }).bind('loaded.jstree', function(e, data) {
-                                if (that.query[type].isNodeNotFoundAtLoad == true) {
-                                    treeLoaded({ "$el": $el, "type": type });
-                                }
+                                treeLoaded({ "$el": $el, "type": type });
                             });
                     },
                     initializeTermTree = function() {
@@ -652,7 +681,7 @@ define(['require',
                                 }
                             }
                             Utils.notifySuccess({
-                                content: messageType + Messages.deleteSuccessMessage
+                                content: messageType + Messages.getAbbreviationMsg(false, 'deleteSuccessMessage')
                             });
                             var url = gId ? '#!/glossary/' + gId : '#!/glossary';
                             if (gId == null) {
@@ -676,11 +705,17 @@ define(['require',
                                 }) : null,
                                 updateTabState: true
                             });
+                        },
+                        complete: function() {
+                            that.notificationModal.hideButtonLoader();
+                            that.notificationModal.remove();
                         }
                     },
                     notifyObj = {
                         modal: true,
-                        ok: function(argument) {
+                        ok: function(obj) {
+                            that.notificationModal = obj;
+                            obj.showButtonLoader();
                             if (type == "Glossary") {
                                 that.glossaryCollection.fullCollection.get(guid).destroy(options, { silent: true, reset: false });
                             } else if (type == "GlossaryCategory") {
@@ -689,6 +724,7 @@ define(['require',
                                 new that.glossaryCollection.model().deleteTerm(guid, options);
                             }
                         },
+                        okCloses: false,
                         cancel: function(argument) {}
                     };
                 if (type == "Glossary") {
@@ -740,6 +776,19 @@ define(['require',
                         updateTabState: true
                     });
                 }
+            },
+            onClickImportGlossary: function() {
+                var that = this;
+                require([
+                    'views/import/ImportLayoutView'
+                ], function(ImportLayoutView) {
+                    var view = new ImportLayoutView({
+                        callback: function() {
+                            that.getGlossary();
+                        },
+                        isGlossary: true
+                    });
+                });
             }
         });
     return GlossaryLayoutView;
