@@ -151,7 +151,7 @@ define(['require',
                 fromEntity = this.lineageData.guidEntityMap[obj.fromEntityId],
                 toEntity = this.lineageData.guidEntityMap[obj.toEntityId];
             if (fromEntity && toEntity) {
-                this.ui.edgeDetailName.html(fromEntity.displayText + " <span class='navigation-font'><i class='fa fa-long-arrow-right fa-color'></i></span> " + toEntity.displayText);
+                this.ui.edgeDetailName.html(_.escape(fromEntity.displayText) + " <span class='navigation-font'><i class='fa fa-long-arrow-right fa-color'></i></span> " + _.escape(toEntity.displayText));
             }
             if (obj && obj.relationshipId) {
                 this.showLoader();
@@ -236,9 +236,6 @@ define(['require',
                 entityId = that.ui.propagationOptions.attr('entity-id'),
                 PropagationValue = this.$("input[name='propagateRelation']:checked").val(),
                 relationshipProp = {};
-            if (PropagationValue === this.ui.propagationOptions.attr("propagation")) {
-                return;
-            }
             this.ui.propagationOptions.attr("propagation", PropagationValue);
             if (this.viewType == "flow") {
                 relationshipProp = {
@@ -306,11 +303,12 @@ define(['require',
             this.ui.PropagatedClassificationTable.append(_.isEmpty(propagationStringValue) ? "No Records Found." : classificationTableValue);
         },
         showLoader: function() {
-            this.modal.$el.find('button.ok').attr("disabled", true);
+            this.modal.$el.find('button.ok').showButtonLoader();
             this.$('.overlay').removeClass('hide').addClass('show');
         },
         hideLoader: function(options) {
             var buttonDisabled = options && options.buttonDisabled;
+            this.modal.$el.find('button.ok').hideButtonLoader();
             this.modal.$el.find('button.ok').attr("disabled", buttonDisabled ? buttonDisabled : false);
             this.$('.overlay').removeClass('show').addClass('hide');
         },
@@ -318,7 +316,7 @@ define(['require',
             var that = this,
                 notifyObj = {
                     modal: true,
-                    text: "It looks like you have been edited something. If you leave before saving, your changes will be lost.",
+                    text: "It looks like you have edited something. If you leave before saving, your changes will be lost.",
                     ok: function(argument) {
                         that.viewType = that.ui.editPropagationType.is(":checked") ? "flow" : "table";
                         that.ui.editPropagationType.prop("checked", that.viewType === "flow" ? false : true).trigger("change");
