@@ -142,12 +142,12 @@ define(['require',
             updateRelationshipDetails: function(options) {
                 var data = options.obj.value,
                     typeName = data.typeName || options.obj.name,
-                    searchString = options.searchString,
+                    searchString = _.escape(options.searchString),
                     listString = "",
                     getEntityTypelist = function(options) {
                         var activeEntityColor = "#4a90e2",
                             deletedEntityColor = "#BB5838",
-                            entityTypeHtml = '',
+                            entityTypeHtml = '<pre>',
                             getdefault = function(obj) {
                                 var options = obj.options,
                                     status = (Enums.entityStateReadOnly[options.entityStatus || options.status] ? " deleted-relation" : ''),
@@ -155,10 +155,15 @@ define(['require',
                                     entityColor = obj.color,
                                     name = obj.name,
                                     typeName = options.typeName;
-
-                                return "<li class=" + status + ">" +
-                                    "<a style='color:" + entityColor + "' href=#!/detailPage/" + guid + "?tabActive=relationship>" + name + " (" + typeName + ")</a>" +
-                                    "</li>";
+                                if (typeName === "AtlasGlossaryTerm") {
+                                    return '<li class=' + status + '>' +
+                                        '<a style="color:' + entityColor + '" href="#!/glossary/' + guid + '?guid=' + guid + '&gType=term&viewType=term&fromView=entity">' + name + ' (' + typeName + ')</a>' +
+                                        '</li>';
+                                } else {
+                                    return "<li class=" + status + ">" +
+                                        "<a style='color:" + entityColor + "' href=#!/detailPage/" + guid + "?tabActive=relationship>" + name + " (" + typeName + ")</a>" +
+                                        "</li>";
+                                }
                             },
                             getWithButton = function(obj) {
                                 var options = obj.options,
@@ -188,13 +193,13 @@ define(['require',
                                 entityTypeHtml = getdefault({
                                     "color": activeEntityColor,
                                     "options": options,
-                                    "name": _.escape(name)
+                                    "name": name
                                 });
                             } else if (options.relationshipStatus == "DELETED") {
                                 entityTypeHtml = getWithButton({
                                     "color": activeEntityColor,
                                     "options": options,
-                                    "name": _.escape(name),
+                                    "name": name,
                                     "relationship": true
                                 })
                             }
@@ -202,18 +207,17 @@ define(['require',
                             entityTypeHtml = getWithButton({
                                 "color": deletedEntityColor,
                                 "options": options,
-                                "name": _.escape(name),
+                                "name": name,
                                 "entity": true
                             })
                         } else {
-
                             entityTypeHtml = getdefault({
                                 "color": activeEntityColor,
                                 "options": options,
-                                "name": _.escape(name)
+                                "name": name
                             });
                         }
-                        return entityTypeHtml;
+                        return entityTypeHtml + '</pre>';
                     };
                 this.ui.searchNode.hide();
                 this.$("[data-id='typeName']").text(typeName);
@@ -260,10 +264,10 @@ define(['require',
                     .links(data.links)
                     .size([width, height])
                     .linkDistance(200)
-                    .gravity(0.0)
+                    .gravity(0.5)
                     .friction(0.1)
                     .charge(function(d) {
-                        var charge = -500;
+                        var charge = -2000;
                         if (d.index === 0) charge = 100
                         return charge;
                     })
