@@ -23,8 +23,9 @@ define(['require',
     'utils/Globals',
     'utils/Enums',
     'collection/VSearchList',
-    'utils/CommonViewFunction'
-], function(require, Backbone, SearchLayoutViewTmpl, Utils, UrlLinks, Globals, Enums, VSearchList, CommonViewFunction) {
+    'utils/CommonViewFunction',
+    'modules/Modal'
+], function(require, Backbone, SearchLayoutViewTmpl, Utils, UrlLinks, Globals, Enums, VSearchList, CommonViewFunction, Modal) {
     'use strict';
 
     var SearchLayoutView = Backbone.Marionette.LayoutView.extend(
@@ -89,31 +90,34 @@ define(['require',
              * @constructs
              */
             initialize: function(options) {
-                _.extend(this, _.pick(options, 'value', 'typeHeaders', 'searchVent', 'entityDefCollection', 'enumDefCollection', 'classificationDefCollection', 'searchTableColumns', 'searchTableFilters', 'entityCountCollection'));
+                _.extend(this, _.pick(options, 'value', 'typeHeaders', 'searchVent', 'entityDefCollection', 'enumDefCollection', 'classificationDefCollection', 'businessMetadataDefCollection', 'searchTableColumns', 'searchTableFilters', 'metricCollection'));
                 this.type = "basic";
-                this.entityCountObj = _.first(this.entityCountCollection.toJSON());
-                this.filterTypeSelected = [];
+                this.entityCountObj = _.first(this.metricCollection.toJSON());
+                this.selectedFilter = {
+                    'basic': [],
+                    'dsl': []
+                }
                 var param = Utils.getUrlState.getQueryParams();
                 this.query = {
                     dsl: {
-                        query: null,
-                        type: null,
-                        pageOffset: null,
-                        pageLimit: null
+                        query: this.value ? this.value.query : null,
+                        type: this.value ? this.value.type : null,
+                        pageOffset: this.value ? this.value.pageOffset : null,
+                        pageLimit: this.value ? this.value.pageLimit : null
                     },
                     basic: {
-                        query: null,
-                        type: null,
-                        tag: null,
-                        term: null,
-                        attributes: null,
-                        tagFilters: null,
-                        pageOffset: null,
-                        pageLimit: null,
-                        entityFilters: null,
-                        includeDE: null,
-                        excludeST: null,
-                        excludeSC: null
+                        query: this.value ? this.value.query : null,
+                        type: this.value ? this.value.type : null,
+                        tag: this.value ? this.value.tag : null,
+                        term: this.value ? this.value.term : null,
+                        attributes: this.value ? this.value.attributes : null,
+                        tagFilters: this.value ? this.value.tagFilters : null,
+                        pageOffset: this.value ? this.value.pageOffset : null,
+                        pageLimit: this.value ? this.value.pageLimit : null,
+                        entityFilters: this.value ? this.value.entityFilters : null,
+                        includeDE: this.value ? this.value.includeDE : null,
+                        excludeST: this.value ? this.value.excludeST : null,
+                        excludeSC: this.value ? this.value.excludeSC : null
                     }
                 };
                 if (!this.value) {
@@ -124,6 +128,13 @@ define(['require',
                     this.type = param.searchType;
                     this.updateQueryObject(param);
                 }
+                if ((this.value && this.value.type) || (this.value && this.value.tag && this.value.searchType === "basic")) {
+                    this.setInitialEntityVal = false;
+                } else {
+                    this.setInitialEntityVal = true;
+                }
+                this.tagEntityCheck = false;
+                this.typeEntityCheck = false;
                 this.bindEvents();
             },
             renderSaveSearch: function() {
@@ -199,6 +210,9 @@ define(['require',
                 this.listenTo(this.typeHeaders, "reset", function(value) {
                     this.initializeValues();
                 }, this);
+                this.listenTo(this.searchVent, "entityList:refresh", function(model, response) {
+                    this.onRefreshButton();
+                }, this);
             },
             initializeValues: function() {
                 this.renderTypeTagList();
@@ -206,6 +220,9 @@ define(['require',
                 this.setValues();
                 this.checkForButtonVisiblity();
                 this.renderSaveSearch();
+                if (this.setInitialEntityVal) {
+                    this.setInitialEntityVal = false;
+                }
             },
             makeFilterButtonActive: function(filtertypeParam) {
                 var filtertype = ['entityFilters', 'tagFilters'],
@@ -231,7 +248,8 @@ define(['require',
                     }
                 }
                 var tagCheck = function(filterObj, type) {
-                    if (that.value.tag && !_.contains(Enums.addOnClassification, that.value.tag)) {
+                    var filterAddOn = Enums.addOnClassification.filter(function(a) { a !== Enums.addOnClassification[0] });
+                    if (that.value.tag && !_.contains(filterAddOn, that.value.tag)) {
                         that.ui.tagAttrFilter.prop('disabled', false);
                         if (filterObj && filterObj.length) {
                             that.ui.tagAttrFilter.addClass('active');
@@ -358,16 +376,33 @@ define(['require',
                         includeDE: null
                     }), param);
             },
-            fetchCollection: function(value) {
-                this.typeHeaders.fetch({ reset: true });
-            },
             onRefreshButton: function() {
-                this.fetchCollection();
-                //to check url query param contain type or not
-                var checkURLValue = Utils.getUrlState.getQueryParams(this.url);
-                if (this.searchVent && (_.has(checkURLValue, "tag") || _.has(checkURLValue, "type") || _.has(checkURLValue, "query"))) {
-                    this.searchVent.trigger('search:refresh');
-                }
+                var that = this,
+                    apiCount = 2,
+                    updateSearchList = function() {
+                        if (apiCount === 0) {
+                            that.initializeValues();
+                            var checkURLValue = Utils.getUrlState.getQueryParams(that.url);
+                            if (that.searchVent && (_.has(checkURLValue, "tag") || _.has(checkURLValue, "type") || _.has(checkURLValue, "query"))) {
+                                that.searchVent.trigger('search:refresh');
+                            }
+                        }
+                    };
+                this.metricCollection.fetch({
+                    complete: function() {
+                        --apiCount;
+                        that.entityCountObj = _.first(that.metricCollection.toJSON());
+                        updateSearchList();
+                    }
+                });
+
+                this.typeHeaders.fetch({
+                    silent: true,
+                    complete: function() {
+                        --apiCount;
+                        updateSearchList();
+                    }
+                });
             },
             advancedInfo: function(e) {
                 require([
@@ -399,7 +434,9 @@ define(['require',
                         entityDefCollection: that.entityDefCollection,
                         enumDefCollection: that.enumDefCollection,
                         classificationDefCollection: that.classificationDefCollection,
-                        searchTableFilters: that.searchTableFilters
+                        businessMetadataDefCollection: that.businessMetadataDefCollection,
+                        searchTableFilters: that.searchTableFilters,
+
                     });
                     that.attrModal.on('ok', function(scope, e) {
                         that.okAttrFilterButton(e);
@@ -407,7 +444,8 @@ define(['require',
                 });
             },
             okAttrFilterButton: function(e) {
-                var isTag = this.attrModal.tag ? true : false,
+                var that = this,
+                    isTag = this.attrModal.tag ? true : false,
                     filtertype = isTag ? 'tagFilters' : 'entityFilters',
                     queryBuilderRef = this.attrModal.RQueryBuilder.currentView.ui.builder,
                     col = [];
@@ -435,6 +473,32 @@ define(['require',
                         }
                         this.searchTableColumns[this.value.type] = _.sortBy(_.union(this.searchTableColumns[this.value.type], getIdFromRuleObject(rule)));
                     }
+                    if (rule.rules) {
+                        if (!isTag && !that.tagEntityCheck) {
+                            var state = _.find(rule.rules, { id: "__state" });
+                            if (state) {
+                                that.typeEntityCheck = (state.value === "ACTIVE" && state.operator === "=") || (state.value === "DELETED" && state.operator === "!=") ? false : true;
+                                that.value.includeDE = that.typeEntityCheck;
+                            } else if (that.typeEntityCheck) {
+                                that.typeEntityCheck = false;
+                                if (!that.tagEntityCheck) {
+                                    that.value.includeDE = false;
+                                }
+                            }
+                        }
+                        if (isTag && !that.typeEntityCheck) {
+                            var entityStatus = _.find(rule.rules, { id: "__entityStatus" });
+                            if (entityStatus) {
+                                that.tagEntityCheck = (entityStatus.value === "ACTIVE" && entityStatus.operator === "=") || (entityStatus.value === "DELETED" && entityStatus.operator === "!=") ? false : true;
+                                that.value.includeDE = that.tagEntityCheck
+                            } else if (that.tagEntityCheck) {
+                                that.tagEntityCheck = false;
+                                if (!that.typeEntityCheck) {
+                                    that.value.includeDE = false;
+                                }
+                            }
+                        }
+                    }
                     this.attrModal.modal.close();
                     if ($(e.currentTarget).hasClass('search')) {
                         this.findSearchResult();
@@ -442,7 +506,11 @@ define(['require',
                 }
             },
             manualRender: function(paramObj) {
+                if (paramObj) {
+                    this.value = paramObj;
+                }
                 this.updateQueryObject(paramObj);
+                this.renderTypeTagList();
                 this.setValues(paramObj);
             },
             getFilterBox: function() {
@@ -455,8 +523,9 @@ define(['require',
                         serviceArr.push(serviceType);
                     }
                 });
+
                 _.each(_.uniq(serviceArr), function(service) {
-                    serviceStr += '<li><div class="pretty p-switch p-fill"><input type="checkbox" class="pull-left" data-value="' + (service) + '" value="" ' + (_.contains(that.filterTypeSelected, service) ? "checked" : "") + '/><div class="state p-primary"><label>' + (service.toUpperCase()) + '</label></div></div></li>';
+                    serviceStr += '<li><div class="pretty p-switch p-fill"><input type="checkbox" class="pull-left" data-value="' + (service) + '" value="" ' + (_.contains(that.selectedFilter[that.type], service) ? "checked" : "") + '/><div class="state p-primary"><label>' + (service.toUpperCase()) + '</label></div></div></li>';
                 });
                 var templt = serviceStr + '<hr class="hr-filter"/><div class="text-right"><div class="divider"></div><button class="btn btn-action btn-sm filterDone">Done</button></div>';
                 return templt;
@@ -465,29 +534,64 @@ define(['require',
                 var that = this;
                 var serviceTypeToBefiltered = (options && options.filterList);
                 var isTypeOnly = options && options.isTypeOnly;
+                if (this.selectedFilter[this.type].length) {
+                    serviceTypeToBefiltered = this.selectedFilter[this.type];
+                }
                 this.ui.typeLov.empty();
                 var typeStr = '<option></option>',
-                    tagStr = typeStr;
+                    tagStr = typeStr,
+                    foundNewClassification = false;
                 this.typeHeaders.fullCollection.each(function(model) {
                     var name = Utils.getName(model.toJSON(), 'name');
                     if (model.get('category') == 'ENTITY' && (serviceTypeToBefiltered && serviceTypeToBefiltered.length ? _.contains(serviceTypeToBefiltered, model.get('serviceType')) : true)) {
-                        var entityCount = (that.entityCountObj.entity.entityActive[name] + (that.entityCountObj.entity.entityDeleted[name] ? that.entityCountObj.entity.entityDeleted[name] : 0));
+                        var entityCount = (that.entityCountObj.entity.entityActive[name] || 0) + (that.entityCountObj.entity.entityDeleted[name] || 0);
                         typeStr += '<option value="' + (name) + '" data-name="' + (name) + '">' + (name) + ' ' + (entityCount ? "(" + _.numberFormatWithComa(entityCount) + ")" : '') + '</option>';
                     }
                     if (isTypeOnly == undefined && model.get('category') == 'CLASSIFICATION') {
                         var tagEntityCount = that.entityCountObj.tag.tagEntities[name];
+                        if (that.value && that.value.tag) { // to check if wildcard classification is present in our data
+                            if (name === that.value.tag) {
+                                foundNewClassification = true;
+                            }
+                        }
                         tagStr += '<option value="' + (name) + '" data-name="' + (name) + '">' + (name) + ' ' + (tagEntityCount ? "(" + _.numberFormatWithComa(tagEntityCount) + ")" : '') + '</option>';
                     }
                 });
+
+                if (this.type !== "dsl") {
+                    _.each(Enums.addOnEntities, function(entity) {
+                        typeStr += '<option  value="' + (entity) + '" data-name="' + (entity) + '">' + entity + '</option>';
+                    });
+                }
+                if (!foundNewClassification && that.value) {
+                    if (that.value.tag) {
+                        var classificationValue = decodeURIComponent(that.value.tag);
+                        tagStr += '<option  value="' + (classificationValue) + '" data-name="' + (classificationValue) + '">' + classificationValue + '</option>';
+                    }
+                }
                 if (_.isUndefined(isTypeOnly)) {
                     //to insert extra classification list
-                    _.each(Enums.addOnClassification, function(classificationName) {
-                        tagStr += '<option  value="' + (classificationName) + '" data-name="' + (classificationName) + '">' + classificationName + '</option>';
-                    });
+                    if (that.value) {
+                        _.each(Enums.addOnClassification, function(classificationName) {
+                            if (classificationName !== that.value.tag) {
+                                tagStr += '<option  value="' + (classificationName) + '" data-name="' + (classificationName) + '">' + classificationName + '</option>';
+                            }
+                        });
+                    }
                     that.ui.tagLov.html(tagStr);
                     this.ui.tagLov.select2({
                         placeholder: "Select Classification",
-                        allowClear: true
+                        allowClear: true,
+                        tags: true,
+                        createTag: function(tag) {
+                            if (tag.term.indexOf('*') != -1) {
+                                return {
+                                    id: tag.term,
+                                    text: tag.term,
+                                    isNew: true
+                                };
+                            }
+                        }
                     });
                 }
                 that.ui.typeLov.html(typeStr);
@@ -497,16 +601,21 @@ define(['require',
                     allowClear: true,
                     getFilterBox: this.getFilterBox.bind(this),
                     onFilterSubmit: function(options) {
-                        that.filterTypeSelected = options.filterVal;
+                        that.selectedFilter[that.type] = options.filterVal;
                         that.renderTypeTagList({ "filterList": options.filterVal, isTypeOnly: true })
                     }
                 });
                 typeLovSelect2.on("select2:close", function() {
                     typeLovSelect2.trigger("hideFilter");
                 });
-                if (typeLovSelect2 && serviceTypeToBefiltered) {
+                if (typeLovSelect2 && isTypeOnly) {
                     typeLovSelect2.select2('open').trigger("change", { 'manual': true });
                 }
+                if (that.setInitialEntityVal) {
+                    var defaultEntity = Enums.addOnEntities[0];
+                    that.value.type = defaultEntity;
+                    that.ui.typeLov.val(defaultEntity, null);
+                }
             },
             renderTermList: function() {
                 var getTypeAheadData = function(data, params) {
@@ -515,7 +624,7 @@ define(['require',
                     _.each(dataList, function(obj) {
                         if (obj) {
                             if (obj.guid) {
-                                obj['id'] = Utils.getName(obj, 'qualifiedName');
+                                obj['id'] = obj.attributes['qualifiedName'];
                             }
                             foundOptions.push(obj);
                         }
@@ -585,6 +694,7 @@ define(['require',
                         }
                     }
 
+
                     if (!this.dsl) {
                         this.ui.tagLov.val(this.value.tag);
                         if (this.ui.tagLov.data('select2')) {
@@ -596,8 +706,9 @@ define(['require',
                                 this.ui.tagLov.trigger("change", { 'manual': true });
                             }
                         }
+
                         if (this.value.term) {
-                            this.ui.termLov.append('<option value="' + this.value.term + '" selected="selected">' + this.value.term + '</option>');
+                            this.ui.termLov.append('<option value="' + _.escape(this.value.term) + '" selected="selected">' + _.escape(this.value.term) + '</option>');
                         }
                         if (this.ui.termLov.data('select2')) {
                             if (this.ui.termLov.val() !== this.value.term) {
@@ -623,8 +734,8 @@ define(['require',
                         tagFilters: null,
                         entityFilters: null
                     },
-                    typeLovValue = this.ui.typeLov.find(':selected').data('name'),
-                    tagLovValue = this.ui.tagLov.find(':selected').data('name'),
+                    typeLovValue = this.ui.typeLov.find(':selected').data('name'), // to get count of selected element used data
+                    tagLovValue = this.ui.tagLov.find(':selected').data('name') || this.ui.tagLov.val(),
                     termLovValue = this.ui.termLov.select2('val')
                 params['type'] = typeLovValue || null;
                 if (!this.dsl) {
@@ -704,7 +815,7 @@ define(['require',
                 }
             },
             clearSearchData: function() {
-                this.filterTypeSelected = [];
+                this.selectedFilter[this.type] = [];
                 this.renderTypeTagList();
                 this.updateQueryObject();
                 this.ui.typeLov.val("").trigger("change");
