@@ -18,6 +18,7 @@
 
 package org.apache.atlas.repository.graphdb.janus;
 
+import com.google.common.annotations.VisibleForTesting;
 import com.google.common.collect.ImmutableMap;
 import org.apache.atlas.ApplicationProperties;
 import org.apache.atlas.AtlasException;
@@ -83,6 +84,10 @@ public class AtlasJanusGraphDatabase implements GraphDatabase<AtlasJanusVertex,
 
         Configuration configProperties = ApplicationProperties.get();
 
+        if (isEmbeddedSolr()) { // AtlasJanusGraphIndexClient.performRequestHandlerAction() fails for embedded-solr; disable freetext until this issue is resolved
+            configProperties.setProperty(ApplicationProperties.ENABLE_FREETEXT_SEARCH_CONF, false);
+        }
+
         configProperties.setProperty(SOLR_ZOOKEEPER_URLS, configProperties.getStringArray(SOLR_ZOOKEEPER_URL));
 
         Configuration janusConfig = ApplicationProperties.getSubsetConfiguration(configProperties, GRAPH_PREFIX);
@@ -161,24 +166,32 @@ public class AtlasJanusGraphDatabase implements GraphDatabase<AtlasJanusVertex,
                         throw new RuntimeException(e);
                     }
 
-                    try {
-                        graphInstance = JanusGraphFactory.open(config);
-                    } catch (JanusGraphException e) {
-                        LOG.warn("JanusGraphException: {}", e.getMessage());
-                        if (e.getMessage().startsWith(OLDER_STORAGE_EXCEPTION)) {
-                            LOG.info("Newer client is being used with older janus storage version. Setting allow-upgrade=true and reattempting connection");
-                            config.addProperty("graph.allow-upgrade", true);
-                            graphInstance = JanusGraphFactory.open(config);
-                        }
-                    }
+                    graphInstance = initJanusGraph(config);
                     atlasGraphInstance = new AtlasJanusGraph();
                     validateIndexBackend(config);
+
                 }
             }
         }
         return graphInstance;
     }
 
+    @VisibleForTesting
+    static JanusGraph initJanusGraph(Configuration config) {
+        try {
+            return JanusGraphFactory.open(config);
+        } catch (JanusGraphException e) {
+            LOG.warn("JanusGraphException: {}", e.getMessage());
+            if (e.getMessage().startsWith(OLDER_STORAGE_EXCEPTION)) {
+                LOG.info("Newer client is being used with older janus storage version. Setting allow-upgrade=true and reattempting connection");
+                config.addProperty("graph.allow-upgrade", true);
+                return JanusGraphFactory.open(config);
+            } else {
+                throw new RuntimeException(e);
+            }
+        }
+    }
+
     public static JanusGraph getBulkLoadingGraphInstance() {
         try {
             Configuration cfg = getConfiguration();
