@@ -24,6 +24,8 @@ import java.io.IOException;
 import java.io.StringReader;
 import java.io.UncheckedIOException;
 import java.lang.reflect.Constructor;
+import java.math.BigDecimal;
+import java.math.BigInteger;
 import java.text.DateFormat;
 import java.text.SimpleDateFormat;
 import java.time.Instant;
@@ -111,7 +113,7 @@ import org.janusgraph.diskstorage.indexing.RawQuery;
 import org.janusgraph.diskstorage.solr.transform.GeoToWktConverter;
 import org.janusgraph.diskstorage.util.DefaultTransaction;
 import org.janusgraph.graphdb.configuration.PreInitializeConfigOptions;
-import org.janusgraph.graphdb.database.serialize.AttributeUtil;
+import org.janusgraph.graphdb.database.serialize.AttributeUtils;
 import org.janusgraph.graphdb.internal.Order;
 import org.janusgraph.graphdb.query.JanusGraphPredicate;
 import org.janusgraph.graphdb.query.condition.And;
@@ -135,11 +137,13 @@ public class Solr6Index implements IndexProvider {
 
     private static final Logger logger = LoggerFactory.getLogger(Solr6Index.class);
 
-
     private static final String DEFAULT_ID_FIELD  = "id";
     private static final char   CHROOT_START_CHAR = '/';
 
-    private enum Mode {
+    private static Solr6Index instance = null;
+    public static final ConfigOption<Boolean> CREATE_SOLR_CLIENT_PER_REQUEST = new ConfigOption(SOLR_NS, "create-client-per-request", "when false, allows the sharing of solr client across other components.", org.janusgraph.diskstorage.configuration.ConfigOption.Type.LOCAL, false);
+
+    public enum Mode {
         HTTP, CLOUD;
 
         public static Mode parse(String mode) {
@@ -167,6 +171,7 @@ public class Solr6Index implements IndexProvider {
             .build();
 
     private static final Map<Geo, String> SPATIAL_PREDICATES = spatialPredicates();
+    private static boolean createSolrClientPerRequest;
 
     private final SolrClient solrClient;
     private final Configuration configuration;
@@ -200,40 +205,104 @@ public class Solr6Index implements IndexProvider {
             logger.debug("KERBEROS_ENABLED name is " + KERBEROS_ENABLED.getName() + " and it is" + (KERBEROS_ENABLED.isOption() ? " " : " not") + " an option.");
             logger.debug("KERBEROS_ENABLED type is " + KERBEROS_ENABLED.getType().name());
         }
+
+        solrClient = createSolrClient();
+        createSolrClientPerRequest = config.get(CREATE_SOLR_CLIENT_PER_REQUEST);
+        if(createSolrClientPerRequest) {
+            logger.info("A new Solr Client will be created for direct interation with SOLR.");
+        } else {
+            logger.info("Solr Client will be shared for direct interation with SOLR.");
+        }
+        Solr6Index.instance = this;
+    }
+
+    public static Mode getSolrMode() {
+        Solr6Index solr6Index = Solr6Index.instance;
+        Mode       ret        = (solr6Index != null) ? Mode.parse(solr6Index.configuration.get(SOLR_MODE)) : null;
+
+        if (ret == null) {
+            logger.warn("SolrMode is not set. Assuming {}", Mode.CLOUD);
+
+            ret = Mode.CLOUD;
+        }
+
+        return ret;
+    }
+
+    public static SolrClient getSolrClient() {
+        if (Solr6Index.instance != null) {
+            if (createSolrClientPerRequest) {
+                logger.debug("Creating a new Solr Client.");
+                return Solr6Index.instance.createSolrClient();
+            } else {
+                logger.debug("Returning the solr client owned by Solr6Index.");
+                return Solr6Index.instance.solrClient;
+            }
+        } else {
+            logger.debug(" No Solr6Index available. Will return null");
+            return null;
+        }
+    }
+
+    public static void releaseSolrClient(SolrClient solrClient) {
+        if(createSolrClientPerRequest) {
+            if (solrClient != null) {
+                try {
+                    solrClient.close();
+
+                    if(logger.isDebugEnabled()) {
+                        logger.debug("Closed the solr client successfully.");
+                    }
+                } catch (IOException excp) {
+                    logger.warn("Failed to close SolrClient.", excp);
+                }
+            }
+        } else {
+            if(logger.isDebugEnabled()) {
+                logger.debug("Ignoring the closing of solr client as it is owned by Solr6Index.");
+            }
+        }
+    }
+
+    private SolrClient createSolrClient() {
+        if(logger.isDebugEnabled()) {
+            logger.debug("HttpClientBuilder = {}", HttpClientUtil.getHttpClientBuilder(), new Exception());
+        }
         final ModifiableSolrParams clientParams = new ModifiableSolrParams();
+        SolrClient solrClient = null;
+
+        Mode mode = Mode.parse(configuration.get(SOLR_MODE));
         switch (mode) {
             case CLOUD:
-                /* ATLAS-2920: Update JanusGraph Solr clients to use all zookeeper entries – start */
-                final List<String> zookeeperUrls = getZookeeperURLs(config);
-                /* ATLAS-2920: end */
                 final CloudSolrClient cloudServer = new CloudSolrClient.Builder()
                         .withLBHttpSolrClientBuilder(
                                 new LBHttpSolrClient.Builder()
                                         .withHttpSolrClientBuilder(new HttpSolrClient.Builder().withInvariantParams(clientParams))
-                                        .withBaseSolrUrls(config.get(HTTP_URLS))
+                                        .withBaseSolrUrls(configuration.get(HTTP_URLS))
                         )
-                        .withZkHost(zookeeperUrls)
+                        .withZkHost(getZookeeperURLs(configuration))
                         .sendUpdatesOnlyToShardLeaders()
                         .build();
                 cloudServer.connect();
                 solrClient = cloudServer;
+                logger.info("Created solr client using Cloud based configuration.");
                 break;
             case HTTP:
-                clientParams.add(HttpClientUtil.PROP_ALLOW_COMPRESSION, config.get(HTTP_ALLOW_COMPRESSION).toString());
-                clientParams.add(HttpClientUtil.PROP_CONNECTION_TIMEOUT, config.get(HTTP_CONNECTION_TIMEOUT).toString());
-                clientParams.add(HttpClientUtil.PROP_MAX_CONNECTIONS_PER_HOST, config.get(HTTP_MAX_CONNECTIONS_PER_HOST).toString());
-                clientParams.add(HttpClientUtil.PROP_MAX_CONNECTIONS, config.get(HTTP_GLOBAL_MAX_CONNECTIONS).toString());
+                clientParams.add(HttpClientUtil.PROP_ALLOW_COMPRESSION, configuration.get(HTTP_ALLOW_COMPRESSION).toString());
+                clientParams.add(HttpClientUtil.PROP_CONNECTION_TIMEOUT, configuration.get(HTTP_CONNECTION_TIMEOUT).toString());
+                clientParams.add(HttpClientUtil.PROP_MAX_CONNECTIONS_PER_HOST, configuration.get(HTTP_MAX_CONNECTIONS_PER_HOST).toString());
+                clientParams.add(HttpClientUtil.PROP_MAX_CONNECTIONS, configuration.get(HTTP_GLOBAL_MAX_CONNECTIONS).toString());
                 final HttpClient client = HttpClientUtil.createClient(clientParams);
                 solrClient = new LBHttpSolrClient.Builder()
                         .withHttpClient(client)
-                        .withBaseSolrUrls(config.get(HTTP_URLS))
+                        .withBaseSolrUrls(configuration.get(HTTP_URLS))
                         .build();
-
-
+                logger.info("Created solr client using HTTP based configuration.");
                 break;
             default:
                 throw new IllegalArgumentException("Unsupported Solr operation mode: " + mode);
         }
+        return solrClient;
     }
 
     private void configureSolrClientsForKerberos() throws PermanentBackendException {
@@ -314,6 +383,11 @@ public class Solr6Index implements IndexProvider {
         String analyzer = ParameterType.STRING_ANALYZER.findParameter(information.getParameters(), null);
         if (analyzer != null) {
             //If the key have a tokenizer, we try to get it by reflection
+            // some referred classes might not be able to be found via SystemClassLoader
+            // since they might be associated with other classloader, in this situation
+            // ClassNotFound exception will be thrown. instead of using SystemClassLoader
+            // for all classes, we find its classloader first and then load the class, please
+            // call - instantiateUsingClassLoader()
             try {
                 ((Constructor<Tokenizer>) ClassLoader.getSystemClassLoader().loadClass(analyzer)
                         .getConstructor()).newInstance();
@@ -333,6 +407,17 @@ public class Solr6Index implements IndexProvider {
         }
     }
 
+    private void instantiateUsingClassLoader(String analyzer) throws PermanentBackendException {
+        if (analyzer == null) return;
+        try {
+            Class analyzerClass = Class.forName(analyzer);
+            ClassLoader cl = analyzerClass.getClassLoader();
+            ((Constructor<Tokenizer>) cl.loadClass(analyzer).getConstructor()).newInstance();
+        } catch (final ReflectiveOperationException e) {
+            throw new PermanentBackendException(e.getMessage(),e);
+        }
+    }
+
     @Override
     public void mutate(Map<String, Map<String, IndexMutation>> mutations, KeyInformation.IndexRetriever information,
                        BaseTransaction tx) throws BackendException {
@@ -553,12 +638,7 @@ public class Solr6Index implements IndexProvider {
         final String queryFilter = buildQueryFilter(query.getCondition(), information.get(collection));
         solrQuery.addFilterQuery(queryFilter);
         if (!query.getOrder().isEmpty()) {
-            final List<IndexQuery.OrderEntry> orders = query.getOrder();
-            for (final IndexQuery.OrderEntry order1 : orders) {
-                final String item = order1.getKey();
-                final SolrQuery.ORDER order = order1.getOrder() == Order.ASC ? SolrQuery.ORDER.asc : SolrQuery.ORDER.desc;
-                solrQuery.addSort(new SolrQuery.SortClause(item, order));
-            }
+            addOrderToQuery(solrQuery, query.getOrder());
         }
         solrQuery.setStart(0);
         if (query.hasLimit()) {
@@ -570,6 +650,14 @@ public class Solr6Index implements IndexProvider {
                 doc -> doc.getFieldValue(keyIdField).toString());
     }
 
+    private void addOrderToQuery(SolrQuery solrQuery, List<IndexQuery.OrderEntry> orders) {
+        for (final IndexQuery.OrderEntry order1 : orders) {
+            final String item = order1.getKey();
+            final SolrQuery.ORDER order = order1.getOrder() == Order.ASC ? SolrQuery.ORDER.asc : SolrQuery.ORDER.desc;
+            solrQuery.addSort(new SolrQuery.SortClause(item, order));
+        }
+    }
+
     private <E> Stream<E> executeQuery(Integer limit, int offset, String collection, SolrQuery solrQuery,
                                        Function<SolrDocument, E> function) throws PermanentBackendException {
         try {
@@ -598,6 +686,9 @@ public class Solr6Index implements IndexProvider {
         } else {
             solrQuery.setRows(batchSize);
         }
+        if (!query.getOrders().isEmpty()) {
+            addOrderToQuery(solrQuery, query.getOrders());
+        }
 
         for(final Parameter parameter: query.getParameters()) {
             if (parameter.value() instanceof String[]) {
@@ -929,8 +1020,8 @@ public class Solr6Index implements IndexProvider {
     public boolean supports(KeyInformation information, JanusGraphPredicate predicate) {
         final Class<?> dataType = information.getDataType();
         final Mapping mapping = Mapping.getMapping(information);
-        if (mapping!=Mapping.DEFAULT && !AttributeUtil.isString(dataType) &&
-                !(mapping==Mapping.PREFIX_TREE && AttributeUtil.isGeo(dataType))) return false;
+        if (mapping!=Mapping.DEFAULT && !AttributeUtils.isString(dataType) &&
+                !(mapping==Mapping.PREFIX_TREE && AttributeUtils.isGeo(dataType))) return false;
 
         if (Number.class.isAssignableFrom(dataType)) {
             return predicate instanceof Cmp;
@@ -941,7 +1032,7 @@ public class Solr6Index implements IndexProvider {
                 case PREFIX_TREE:
                     return predicate == Geo.INTERSECT || predicate == Geo.WITHIN || predicate == Geo.CONTAINS;
             }
-        } else if (AttributeUtil.isString(dataType)) {
+        } else if (AttributeUtils.isString(dataType)) {
             switch(mapping) {
                 case DEFAULT:
                 case TEXT:
@@ -969,9 +1060,9 @@ public class Solr6Index implements IndexProvider {
         if (Number.class.isAssignableFrom(dataType) || dataType == Date.class || dataType == Instant.class
                 || dataType == Boolean.class || dataType == UUID.class) {
             return mapping == Mapping.DEFAULT;
-        } else if (AttributeUtil.isString(dataType)) {
+        } else if (AttributeUtils.isString(dataType)) {
             return mapping == Mapping.DEFAULT || mapping == Mapping.TEXT || mapping == Mapping.STRING;
-        } else if (AttributeUtil.isGeo(dataType)) {
+        } else if (AttributeUtils.isGeo(dataType)) {
             return mapping == Mapping.DEFAULT || mapping == Mapping.PREFIX_TREE;
         }
         return false;
@@ -986,19 +1077,23 @@ public class Solr6Index implements IndexProvider {
         if (ParameterType.MAPPED_NAME.hasParameter(keyInfo.getParameters())) return key;
         String postfix;
         final Class dataType = keyInfo.getDataType();
-        if (AttributeUtil.isString(dataType)) {
+        if (AttributeUtils.isString(dataType)) {
             final Mapping map = getStringMapping(keyInfo);
             switch (map) {
                 case TEXT: postfix = "_t"; break;
                 case STRING: postfix = "_s"; break;
                 default: throw new IllegalArgumentException("Unsupported string mapping: " + map);
             }
-        } else if (AttributeUtil.isWholeNumber(dataType)) {
+        } else if (AttributeUtils.isWholeNumber(dataType)) {
             if (dataType.equals(Long.class)) postfix = "_l";
             else postfix = "_i";
-        } else if (AttributeUtil.isDecimal(dataType)) {
+        } else if (AttributeUtils.isDecimal(dataType)) {
             if (dataType.equals(Float.class)) postfix = "_f";
             else postfix = "_d";
+        } else if (dataType.equals(BigInteger.class)) {
+            postfix = "_bi";
+        } else if (dataType.equals(BigDecimal.class)) {
+            postfix = "_bd";
         } else if (dataType.equals(Geoshape.class)) {
             postfix = "_g";
         } else if (dataType.equals(Date.class) || dataType.equals(Instant.class)) {
@@ -1008,6 +1103,7 @@ public class Solr6Index implements IndexProvider {
         } else if (dataType.equals(UUID.class)) {
             postfix = "_uuid";
         } else throw new IllegalArgumentException("Unsupported data type ["+dataType+"] for field: " + key);
+
         if (keyInfo.getCardinality() == Cardinality.SET || keyInfo.getCardinality() == Cardinality.LIST) {
             postfix += "s";
         }
@@ -1039,7 +1135,7 @@ public class Solr6Index implements IndexProvider {
      */
 
     private static Mapping getStringMapping(KeyInformation information) {
-        assert AttributeUtil.isString(information.getDataType());
+        assert AttributeUtils.isString(information.getDataType());
         Mapping map = Mapping.getMapping(information);
         if (map==Mapping.DEFAULT) map = Mapping.TEXT;
         return map;
@@ -1146,7 +1242,7 @@ public class Solr6Index implements IndexProvider {
     }
 
     /* ATLAS-2920: Update JanusGraph Solr clients to use all zookeeper entries – start */
-    private List<String> getZookeeperURLs(Configuration config) {
+    private static List<String> getZookeeperURLs(Configuration config) {
         List<String> ret     = null;
         String[]     zkHosts = config.get(ZOOKEEPER_URLS);
 
