@@ -19,11 +19,11 @@ package org.apache.atlas.type;
 
 import org.apache.atlas.AtlasErrorCode;
 import org.apache.atlas.exception.AtlasBaseException;
+import org.apache.atlas.model.instance.AtlasEntity;
 import org.apache.atlas.model.instance.AtlasStruct;
 import org.apache.atlas.model.typedef.AtlasStructDef;
 import org.apache.atlas.model.typedef.AtlasStructDef.AtlasAttributeDef;
 import org.apache.atlas.model.typedef.AtlasStructDef.AtlasAttributeDef.Cardinality;
-import org.apache.atlas.type.AtlasStructType.AtlasAttribute;
 import org.apache.atlas.model.typedef.AtlasStructDef.AtlasConstraintDef;
 import org.apache.commons.collections.CollectionUtils;
 import org.apache.commons.collections.MapUtils;
@@ -31,16 +31,11 @@ import org.apache.commons.lang.StringUtils;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
-import java.util.Collection;
-import java.util.Collections;
-import java.util.HashMap;
-import java.util.Iterator;
-import java.util.List;
-import java.util.Map;
+import java.util.*;
 
-import static org.apache.atlas.model.typedef.AtlasStructDef.AtlasConstraintDef.CONSTRAINT_PARAM_ATTRIBUTE;
-import static org.apache.atlas.model.typedef.AtlasStructDef.AtlasConstraintDef.CONSTRAINT_TYPE_INVERSE_REF;
-import static org.apache.atlas.model.typedef.AtlasStructDef.AtlasConstraintDef.CONSTRAINT_TYPE_OWNED_REF;
+import static org.apache.atlas.model.TypeCategory.OBJECT_ID_TYPE;
+import static org.apache.atlas.model.typedef.AtlasBaseTypeDef.ATLAS_TYPE_STRING;
+import static org.apache.atlas.model.typedef.AtlasStructDef.AtlasConstraintDef.*;
 
 /**
  * class that implements behaviour of a struct-type.
@@ -48,6 +43,8 @@ import static org.apache.atlas.model.typedef.AtlasStructDef.AtlasConstraintDef.C
 public class AtlasStructType extends AtlasType {
     private static final Logger LOG = LoggerFactory.getLogger(AtlasStructType.class);
 
+    public static final String UNIQUE_ATTRIBUTE_SHADE_PROPERTY_PREFIX = "__u_";
+
     private final AtlasStructDef structDef;
 
     protected Map<String, AtlasAttribute> allAttributes  = Collections.emptyMap();
@@ -101,6 +98,22 @@ public class AtlasStructType extends AtlasType {
 
                 arrayType.setMinCount(attributeDef.getValuesMinCount());
                 arrayType.setMaxCount(attributeDef.getValuesMaxCount());
+                arrayType.setCardinality(cardinality);
+            }
+
+            //check if attribute type is not classification
+            if (attrType instanceof AtlasArrayType) {
+                attrType = ((AtlasArrayType) attrType).getElementType();
+            } else if (attrType instanceof AtlasMapType) {
+                attrType = ((AtlasMapType) attrType).getValueType();
+            }
+
+            if (attrType instanceof AtlasClassificationType) {
+                throw new AtlasBaseException(AtlasErrorCode.ATTRIBUTE_TYPE_INVALID, getTypeName(), attributeDef.getName());
+            }
+
+            if (attrType instanceof AtlasBusinessMetadataType) {
+                throw new AtlasBaseException(AtlasErrorCode.ATTRIBUTE_TYPE_INVALID, getTypeName(), attributeDef.getName());
             }
 
             a.put(attributeDef.getName(), attribute);
@@ -172,9 +185,10 @@ public class AtlasStructType extends AtlasType {
                 continue;
             }
             // Set the inverse reference attribute.
-            AtlasType referencedType = typeRegistry.getType(attribute.getAttributeDef().getTypeName());
+            AtlasType       referencedType       = typeRegistry.getType(attribute.getAttributeDef().getTypeName());
             AtlasEntityType referencedEntityType = getReferencedEntityType(referencedType);
-            AtlasAttribute inverseReference = referencedEntityType.getAttribute(attribute.getInverseRefAttributeName());
+            AtlasAttribute  inverseReference     = referencedEntityType.getAttribute(attribute.getInverseRefAttributeName());
+
             attribute.setInverseRefAttribute(inverseReference);
          }
     }
@@ -188,6 +202,15 @@ public class AtlasStructType extends AtlasType {
         return  ret;
     }
 
+    @Override
+    public Object createDefaultValue(Object defaultValue) {
+        AtlasStruct ret = new AtlasStruct(structDef.getName());
+
+        populateDefaultValues(ret);
+
+        return  ret;
+    }
+
     public Map<String, AtlasAttribute> getAllAttributes() {
         return allAttributes;
     }
@@ -197,7 +220,25 @@ public class AtlasStructType extends AtlasType {
     }
 
     public AtlasAttribute getAttribute(String attributeName) {
-        return allAttributes.get(attributeName);
+        AtlasAttribute ret = allAttributes.get(attributeName);
+
+        if (ret == null) {
+            ret = getSystemAttribute(attributeName);
+        }
+
+        if (ret == null) {
+            ret = getBusinesAAttribute(attributeName);
+        }
+
+        return ret;
+    }
+
+    public AtlasAttribute getSystemAttribute(String attributeName) {
+        return null;
+    }
+
+    public AtlasAttribute getBusinesAAttribute(String attributeName) {
+        return null;
     }
 
     @Override
@@ -359,13 +400,24 @@ public class AtlasStructType extends AtlasType {
                         if (value != null) {
                             ret = dataType.validateValue(value, fieldName, messages) && ret;
                         } else if (!attributeDef.getIsOptional()) {
-                            ret = false;
-                            messages.add(fieldName + ": mandatory attribute value missing in type " + getTypeName());
+                            // if required attribute is null, check if attribute value specified in relationship
+                            if (structObj instanceof AtlasEntity) {
+                                AtlasEntity entityObj = (AtlasEntity) structObj;
+
+                                if (entityObj.getRelationshipAttribute(attrName) == null) {
+                                    ret = false;
+                                    messages.add(fieldName + ": mandatory attribute value missing in type " + getTypeName());
+                                }
+                            } else {
+                                ret = false;
+                                messages.add(fieldName + ": mandatory attribute value missing in type " + getTypeName());
+                            }
                         }
                     }
                 }
             } else if (obj instanceof Map) {
-                Map attributes = AtlasTypeUtil.toStructAttributes((Map)obj);
+                Map attributes             = AtlasTypeUtil.toStructAttributes((Map)obj);
+                Map relationshipAttributes = AtlasTypeUtil.toRelationshipAttributes((Map)obj);
 
                 for (AtlasAttributeDef attributeDef : structDef.getAttributeDefs()) {
                     String             attrName  = attributeDef.getName();
@@ -379,8 +431,11 @@ public class AtlasStructType extends AtlasType {
                         if (value != null) {
                             ret = dataType.validateValue(value, fieldName, messages) && ret;
                         } else if (!attributeDef.getIsOptional()) {
-                            ret = false;
-                            messages.add(fieldName + ": mandatory attribute value missing in type " + getTypeName());
+                            // if required attribute is null, check if attribute value specified in relationship
+                            if (MapUtils.isEmpty(relationshipAttributes) || !relationshipAttributes.containsKey(attrName)) {
+                                ret = false;
+                                messages.add(fieldName + ": mandatory attribute value missing in type " + getTypeName());
+                            }
                         }
                     }
                 }
@@ -520,7 +575,7 @@ public class AtlasStructType extends AtlasType {
             if (attribute != null) {
                 AtlasType dataType = attribute.getAttributeType();
 
-                ret = dataType.createDefaultValue();
+                ret = dataType.createDefaultValue(attributeDef.getDefaultValue());
             }
         }
 
@@ -597,9 +652,11 @@ public class AtlasStructType extends AtlasType {
         return null;
     }
 
-    public String getQualifiedAttributeName(String attrName) throws AtlasBaseException {
-        if ( allAttributes.containsKey(attrName)) {
-            return allAttributes.get(attrName).getQualifiedName();
+    public String getVertexPropertyName(String attrName) throws AtlasBaseException {
+        AtlasAttribute attribute = getAttribute(attrName);
+
+        if (attribute != null) {
+            return attribute.getVertexPropertyName();
         }
 
         throw new AtlasBaseException(AtlasErrorCode.UNKNOWN_ATTRIBUTE, attrName, structDef.getName());
@@ -613,7 +670,7 @@ public class AtlasStructType extends AtlasType {
         throw new AtlasBaseException(AtlasErrorCode.UNKNOWN_ATTRIBUTE, attrName, structDef.getName());
     }
 
-    private AtlasEntityType getReferencedEntityType(AtlasType type) {
+    AtlasEntityType getReferencedEntityType(AtlasType type) {
         if (type instanceof AtlasArrayType) {
             type = ((AtlasArrayType)type).getElementType();
         }
@@ -661,26 +718,40 @@ public class AtlasStructType extends AtlasType {
         return ret;
     }
 
-
     public static class AtlasAttribute {
-        private final AtlasStructType   definedInType;
-        private final AtlasType         attributeType;
-        private final AtlasAttributeDef attributeDef;
-        private final String            qualifiedName;
-        private final String            vertexPropertyName;
-        private final boolean           isOwnedRef;
-        private final String            inverseRefAttributeName;
-        private AtlasAttribute          inverseRefAttribute;
+        public static final Object VERTEX_PROPERTY_PREFIX_STRING_INDEX_TYPE = "__s_";
+        private final AtlasStructType          definedInType;
+        private final AtlasType                attributeType;
+        private final AtlasAttributeDef        attributeDef;
+        private final String                   qualifiedName;
+        private final String                   vertexPropertyName;
+        private final String                   vertexUniquePropertyName;
+        private final boolean                  isOwnedRef;
+        private final boolean                  isObjectRef;
+        private final String                   inverseRefAttributeName;
+        private AtlasAttribute                 inverseRefAttribute;
+        private String                         relationshipName;
+        private String                         relationshipEdgeLabel;
+        private AtlasRelationshipEdgeDirection relationshipEdgeDirection;
+        private boolean                        isLegacyAttribute;
+        private String                         indexFieldName;
 
-        public AtlasAttribute(AtlasStructType definedInType, AtlasAttributeDef attrDef, AtlasType attributeType) {
-            this.definedInType      = definedInType;
-            this.attributeDef       = attrDef;
-            this.attributeType      = attributeType.getTypeForAttribute();
-            this.qualifiedName      = getQualifiedAttributeName(definedInType.getStructDef(), attributeDef.getName());
-            this.vertexPropertyName = encodePropertyKey(this.qualifiedName);
+        private boolean isDynAttribute            = false;
+        private boolean isDynAttributeEvalTrigger = false;
 
-            boolean isOwnedRef          = false;
-            String  inverseRefAttribute = null;
+        public AtlasAttribute(AtlasStructType definedInType, AtlasAttributeDef attrDef, AtlasType attributeType, String relationshipName, String relationshipLabel) {
+            this.definedInType            = definedInType;
+            this.attributeDef             = attrDef;
+            this.attributeType            = attributeType.getTypeForAttribute();
+            this.qualifiedName            = getQualifiedAttributeName(definedInType.getStructDef(), attributeDef.getName());
+            this.vertexPropertyName       = generateVertexPropertyName(definedInType.getStructDef(), attributeDef, qualifiedName);
+            this.vertexUniquePropertyName = attrDef.getIsUnique() ? encodePropertyKey(getQualifiedAttributeName(definedInType.getStructDef(), UNIQUE_ATTRIBUTE_SHADE_PROPERTY_PREFIX + attributeDef.getName())) : null;
+            this.relationshipName         = relationshipName;
+            this.relationshipEdgeLabel    = getRelationshipEdgeLabel(relationshipLabel);
+            boolean isOwnedRef            = false;
+            String  inverseRefAttribute   = null;
+
+            LOG.debug("Attribute {} will use the vertext property name {}.", qualifiedName, vertexPropertyName);
 
             if (CollectionUtils.isNotEmpty(attributeDef.getConstraints())) {
                 for (AtlasConstraintDef constraint : attributeDef.getConstraints()) {
@@ -698,8 +769,55 @@ public class AtlasStructType extends AtlasType {
                 }
             }
 
-            this.isOwnedRef          = isOwnedRef;
-            this.inverseRefAttributeName = inverseRefAttribute;
+            this.isOwnedRef                = isOwnedRef;
+            this.inverseRefAttributeName   = inverseRefAttribute;
+            this.relationshipEdgeDirection = AtlasRelationshipEdgeDirection.OUT;
+
+            switch (this.attributeType.getTypeCategory()) {
+                case OBJECT_ID_TYPE:
+                    isObjectRef = true;
+                    break;
+
+                case MAP:
+                    AtlasMapType mapType = (AtlasMapType) this.attributeType;
+
+                    isObjectRef = mapType.getValueType().getTypeCategory() == OBJECT_ID_TYPE;
+                    break;
+
+                case ARRAY:
+                    AtlasArrayType arrayType = (AtlasArrayType) this.attributeType;
+
+                    isObjectRef = arrayType.getElementType().getTypeCategory() == OBJECT_ID_TYPE;
+                    break;
+
+                default:
+                    isObjectRef = false;
+                    break;
+            }
+        }
+
+        public AtlasAttribute(AtlasStructType definedInType, AtlasAttributeDef attrDef, AtlasType attributeType) {
+            this(definedInType, attrDef, attributeType, null, null);
+        }
+
+        public AtlasAttribute(AtlasAttribute other) {
+            this.definedInType             = other.definedInType;
+            this.attributeType             = other.attributeType;
+            this.attributeDef              = other.attributeDef;
+            this.qualifiedName             = other.qualifiedName;
+            this.vertexPropertyName        = other.vertexPropertyName;
+            this.vertexUniquePropertyName  = other.vertexUniquePropertyName;
+            this.isOwnedRef                = other.isOwnedRef;
+            this.isObjectRef               = other.isObjectRef;
+            this.inverseRefAttributeName   = other.inverseRefAttributeName;
+            this.inverseRefAttribute       = other.inverseRefAttribute;
+            this.relationshipName          = other.relationshipName;
+            this.relationshipEdgeLabel     = other.relationshipEdgeLabel;
+            this.relationshipEdgeDirection = other.relationshipEdgeDirection;
+            this.isLegacyAttribute         = other.isLegacyAttribute;
+            this.indexFieldName            = other.indexFieldName;
+            this.isDynAttribute            = false;
+            this.isDynAttributeEvalTrigger = false;
         }
 
         public AtlasStructType getDefinedInType() { return definedInType; }
@@ -722,13 +840,76 @@ public class AtlasStructType extends AtlasType {
 
         public String getVertexPropertyName() { return vertexPropertyName; }
 
+        public String getVertexUniquePropertyName() { return vertexUniquePropertyName; }
+
         public boolean isOwnedRef() { return isOwnedRef; }
 
+        public boolean isObjectRef() { return isObjectRef; }
+
         public String getInverseRefAttributeName() { return inverseRefAttributeName; }
 
         public AtlasAttribute getInverseRefAttribute() { return inverseRefAttribute; }
 
-        public void setInverseRefAttribute(AtlasAttribute inverseAttr) { inverseRefAttribute = inverseAttr; };
+        public void setInverseRefAttribute(AtlasAttribute inverseAttr) { inverseRefAttribute = inverseAttr; }
+
+        public String getRelationshipName() { return relationshipName; }
+
+        public void setRelationshipName(String relationshipName) { this.relationshipName = relationshipName; }
+
+        public String getRelationshipEdgeLabel() { return relationshipEdgeLabel; }
+
+        public void setRelationshipEdgeLabel(String relationshipEdgeLabel) { this.relationshipEdgeLabel = relationshipEdgeLabel; }
+
+        public AtlasRelationshipEdgeDirection getRelationshipEdgeDirection() { return relationshipEdgeDirection; }
+
+        public void setRelationshipEdgeDirection(AtlasRelationshipEdgeDirection relationshipEdgeDirection) {
+            this.relationshipEdgeDirection = relationshipEdgeDirection;
+        }
+
+        public boolean isLegacyAttribute() { return isLegacyAttribute; }
+
+        public void setLegacyAttribute(boolean legacyAttribute) { isLegacyAttribute = legacyAttribute; }
+
+        public String getIndexFieldName() { return indexFieldName; }
+
+        public void setIndexFieldName(String indexFieldName) { this.indexFieldName = indexFieldName; }
+
+        public int getSearchWeight() { return attributeDef.getSearchWeight(); }
+
+        public AtlasAttributeDef.IndexType getIndexType() { return attributeDef.getIndexType();}
+
+        public boolean getIsDynAttribute() { return isDynAttribute; }
+
+        public void setIsDynAttribute(boolean isDynAttribute){ this.isDynAttribute = isDynAttribute; }
+
+        public boolean getIsDynAttributeEvalTrigger() { return isDynAttributeEvalTrigger; }
+
+        public void setIsDynAttributeEvalTrigger(boolean isDynAttributeEvalTrigger) { this.isDynAttributeEvalTrigger = isDynAttributeEvalTrigger; }
+
+        public Set<String> getOptionSet(String optionName) {
+            String      strValue = attributeDef.getOption(optionName);
+            Set<String> ret      = StringUtils.isBlank(strValue) ? null : AtlasType.fromJson(strValue, Set.class);
+
+            return ret;
+        }
+
+        public Integer getOptionInt(String optionName) {
+            String  strValue = attributeDef.getOption(optionName);
+            Integer ret      = StringUtils.isBlank(strValue) ? null : Integer.parseInt(strValue);
+
+            return ret;
+        }
+
+        public String getOptionString(String optionName) {
+            String  strValue = attributeDef.getOption(optionName);
+            String ret      = StringUtils.isBlank(strValue) ? null : strValue;
+
+            return ret;
+        }
+
+        public static String getEdgeLabel(String property) {
+            return "__" + property;
+        }
 
         public static String encodePropertyKey(String key) {
             if (StringUtils.isBlank(key)) {
@@ -795,17 +976,47 @@ public class AtlasStructType extends AtlasType {
             return ret;
         }
 
-        private static String getQualifiedAttributeName(AtlasStructDef structDef, String attrName) {
-            final String typeName = structDef.getName();
-            return attrName.contains(".") ? attrName : String.format("%s.%s", typeName, attrName);
+        private String getRelationshipEdgeLabel(String relationshipLabel) {
+            return (relationshipLabel == null) ? getEdgeLabel(qualifiedName) : relationshipLabel;
         }
 
-        private static String[][] RESERVED_CHAR_ENCODE_MAP = new String[][] {
-                new String[] { "{",  "_o" },
-                new String[] { "}",  "_c" },
-                new String[] { "\"", "_q" },
-                new String[] { "$",  "_d" },
-                new String[] { "%", "_p"  },
+        public static String getQualifiedAttributeName(AtlasStructDef structDef, String attrName) {
+            if (isRootType(structDef)) {
+                return attrName;
+            } else {
+                return attrName.contains(".") ? attrName : String.format("%s.%s", structDef.getName(), attrName);
+            }
+        }
+
+        public static String generateVertexPropertyName(AtlasStructDef structDef, AtlasAttributeDef attrDef, String qualifiedName) {
+            String vertexPropertyName = qualifiedName;
+            String attrName           = attrDef.getName();
+            if (isRootType(structDef)) {
+                return attrName;
+            } else {
+                if(!attrDef.getName().contains(".") &&
+                    AtlasAttributeDef.IndexType.STRING.equals(attrDef.getIndexType()) &&
+                    ATLAS_TYPE_STRING.equalsIgnoreCase(attrDef.getTypeName())) {
+                    vertexPropertyName = String.format("%s.%s%s", structDef.getName(), VERTEX_PROPERTY_PREFIX_STRING_INDEX_TYPE, attrDef.getName());
+                }
+            }
+            return encodePropertyKey(vertexPropertyName);
+        }
+
+        private static boolean isRootType(AtlasStructDef structDef) {
+            return StringUtils.equals(structDef.getName(), AtlasEntityType.ENTITY_ROOT.getTypeName()) ||
+                   StringUtils.equals(structDef.getName(), AtlasClassificationType.CLASSIFICATION_ROOT.getTypeName());
+        }
+
+        // Keys copied from org.janusgraph.graphdb.types.system.SystemTypeManager.RESERVED_CHARS
+        // JanusGraph checks that these chars are not part of any keys hence encoding
+        // also including Titan reserved characters to support migrated property keys
+        private static String[][] RESERVED_CHAR_ENCODE_MAP = new String[][]{
+                new String[] {"{", "_o"},
+                new String[] {"}", "_c"},
+                new String[] {"\"", "_q"},
+                new String[] {"$", "_d"}, //titan reserved character
+                new String[] {"%", "_p"}, //titan reserved characters
         };
 
         private static final char[] IDX_QRY_OFFENDING_CHARS = { '@', '/', ' ', '-' };
@@ -813,5 +1024,7 @@ public class AtlasStructType extends AtlasType {
         private static final char   BRACE_CLOSE_CHAR        = ')';
         private static final char   DOUBLE_QUOTE_CHAR       = '"';
         private static final char   SPACE_CHAR              = ' ';
+
+        public enum AtlasRelationshipEdgeDirection { IN, OUT, BOTH }
     }
 }
