@@ -24,6 +24,7 @@ import org.apache.atlas.model.instance.AtlasEntity.AtlasEntityWithExtInfo;
 import org.apache.atlas.model.instance.AtlasObjectId;
 import org.apache.atlas.model.instance.AtlasStruct;
 import org.apache.atlas.model.typedef.AtlasBaseTypeDef;
+import org.apache.atlas.model.typedef.AtlasBusinessMetadataDef;
 import org.apache.atlas.model.typedef.AtlasClassificationDef;
 import org.apache.atlas.model.typedef.AtlasEntityDef;
 import org.apache.atlas.model.typedef.AtlasEnumDef;
@@ -34,8 +35,17 @@ import org.apache.atlas.model.typedef.AtlasStructDef.AtlasConstraintDef;
 import org.apache.atlas.model.typedef.AtlasTypesDef;
 import org.apache.atlas.type.AtlasTypeUtil;
 import org.apache.commons.collections.CollectionUtils;
+import org.apache.commons.io.FileUtils;
 import org.apache.commons.lang.RandomStringUtils;
+import org.apache.commons.lang.StringUtils;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 
+import java.io.File;
+import java.io.FileInputStream;
+import java.io.FileNotFoundException;
+import java.io.IOException;
+import java.io.InputStream;
 import java.math.BigDecimal;
 import java.math.BigInteger;
 import java.util.ArrayList;
@@ -58,13 +68,14 @@ import static org.apache.atlas.type.AtlasTypeUtil.createRequiredAttrDef;
 import static org.apache.atlas.type.AtlasTypeUtil.createStructTypeDef;
 import static org.apache.atlas.type.AtlasTypeUtil.createUniqueRequiredAttrDef;
 import static org.apache.atlas.type.AtlasTypeUtil.getAtlasObjectId;
+import static org.apache.atlas.type.AtlasTypeUtil.createBusinessMetadataDef;
 
 
 /**
  * Test utility class.
  */
 public final class TestUtilsV2 {
-
+    private static final Logger LOG = LoggerFactory.getLogger(TestUtilsV2.class);
     public static final long TEST_DATE_IN_LONG = 1418265358440L;
 
     public static final String TEST_USER   = "testUser";
@@ -544,15 +555,18 @@ public final class TestUtilsV2 {
     public static final String COLUMN_TYPE = "column_type";
     public static final String TABLE_NAME = "bar";
     public static final String CLASSIFICATION = "classification";
+    public static final String FETL_CLASSIFICATION = "fetl" + CLASSIFICATION;
+
     public static final String PII = "PII";
     public static final String PHI = "PHI";
-    public static final String SUPER_TYPE_NAME = "Base";
+    public static final String SUPER_TYPE_NAME = "Referenceable";
     public static final String STORAGE_DESC_TYPE = "hive_storagedesc";
     public static final String PARTITION_STRUCT_TYPE = "partition_struct_type";
     public static final String PARTITION_CLASS_TYPE = "partition_class_type";
     public static final String SERDE_TYPE = "serdeType";
     public static final String COLUMNS_MAP = "columnsMap";
     public static final String COLUMNS_ATTR_NAME = "columns";
+    public static final String ENTITY_TYPE_WITH_SIMPLE_ATTR = "entity_with_simple_attr";
     public static final String ENTITY_TYPE_WITH_NESTED_COLLECTION_ATTR = "entity_with_nested_collection_attr";
     public static final String ENTITY_TYPE_WITH_COMPLEX_COLLECTION_ATTR = "entity_with_complex_collection_attr";
     public static final String ENTITY_TYPE_WITH_COMPLEX_COLLECTION_ATTR_DELETE = "entity_with_complex_collection_attr_delete";
@@ -621,6 +635,119 @@ public final class TestUtilsV2 {
         return ret;
     }
 
+    public static AtlasTypesDef defineSimpleAttrType() {
+        AtlasAttributeDef attrPuArray = new AtlasAttributeDef("puArray", "array<string>", true, SINGLE, 1, 1, false, false, false, null);
+        AtlasAttributeDef attrPuMap   = new AtlasAttributeDef("puMap", "map<string,string>",  true, SINGLE, 1,1, false, false, false, null);
+
+        attrPuArray.setOption(AtlasAttributeDef.ATTRDEF_OPTION_APPEND_ON_PARTIAL_UPDATE, "true");
+        attrPuMap.setOption(AtlasAttributeDef.ATTRDEF_OPTION_APPEND_ON_PARTIAL_UPDATE, "true");
+
+
+        AtlasEntityDef simpleAttributesEntityType =
+            createClassTypeDef(ENTITY_TYPE_WITH_SIMPLE_ATTR, ENTITY_TYPE_WITH_SIMPLE_ATTR + "_description", null,
+                createUniqueRequiredAttrDef("name", "string"),
+
+                new AtlasAttributeDef("stringAtrr", "string",
+                    true,
+                    SINGLE, 1, 1,
+                    false, false, false, null),
+
+                new AtlasAttributeDef("arrayOfStrings",
+                    "array<string>", true, SINGLE, 1, 1,
+                    false, false, false, null),
+
+                new AtlasAttributeDef("mapOfStrings", "map<string,string>",
+                    true, SINGLE, 1,1, false, false, false, null),
+
+                attrPuArray,
+
+                attrPuMap
+            );
+
+        AtlasTypesDef ret = AtlasTypeUtil.getTypesDef(Collections.<AtlasEnumDef>emptyList(),
+            Collections.<AtlasStructDef>emptyList(),
+            Collections.<AtlasClassificationDef>emptyList(),
+            Collections.singletonList(simpleAttributesEntityType));
+
+        return ret;
+    }
+
+    public static AtlasEntityWithExtInfo createSimpleAttrTypeEntity() {
+        AtlasEntity entity = new AtlasEntity(ENTITY_TYPE_WITH_SIMPLE_ATTR);
+
+        entity.setAttribute(NAME, ENTITY_TYPE_WITH_SIMPLE_ATTR);
+        entity.setAttribute("stringAtrr", "DummyThree");
+        entity.setAttribute("arrayOfStrings", Arrays.asList("DummyOne", "DummyTwo"));
+        entity.setAttribute("mapOfStrings", Collections.singletonMap("one", "DummyString"));
+        entity.setAttribute("puArray", Arrays.asList("DummyOne", "DummyTwo"));
+        entity.setAttribute("puMap", Collections.singletonMap("one", "DummyString"));
+
+        return new AtlasEntityWithExtInfo(entity);
+    }
+
+    public static AtlasTypesDef defineEnumTypes() {
+        String _description = "_description";
+        AtlasEnumDef myEnum =
+                new AtlasEnumDef("ENUM_1", "ENUM_1" + _description, "1.0",
+                        Arrays.asList(
+                                new AtlasEnumElementDef("USER", "Element" + _description, 1),
+                                new AtlasEnumElementDef("ROLE", "Element" + _description, 2),
+                                new AtlasEnumElementDef("GROUP", "Element" + _description, 3)
+                        ));
+
+        AtlasTypesDef ret = AtlasTypeUtil.getTypesDef(Collections.singletonList(myEnum),
+                new ArrayList<>(), new ArrayList<>(), new ArrayList<>(), new ArrayList<>(), new ArrayList<>());
+
+        populateSystemAttributes(ret);
+
+        return ret;
+    }
+
+    public static AtlasTypesDef defineBusinessMetadataTypes() {
+        String _description = "_description";
+        Map<String, String> options = new HashMap<>();
+        options.put("maxStrLength", "20");
+
+        AtlasBusinessMetadataDef bmNoApplicableTypes = createBusinessMetadataDef("bmNoApplicableTypes", _description, "1.0",
+                createOptionalAttrDef("attr0", "string", options, _description));
+
+        AtlasBusinessMetadataDef bmNoAttributes = createBusinessMetadataDef("bmNoAttributes", _description, "1.0", null);
+
+        options.put("applicableEntityTypes", "[\"" + DATABASE_TYPE + "\",\"" + TABLE_TYPE + "\"]");
+
+        AtlasBusinessMetadataDef bmWithAllTypes = createBusinessMetadataDef("bmWithAllTypes", _description, "1.0",
+                createOptionalAttrDef("attr1", AtlasBusinessMetadataDef.ATLAS_TYPE_BOOLEAN, options, _description),
+                createOptionalAttrDef("attr2", AtlasBusinessMetadataDef.ATLAS_TYPE_BYTE, options, _description),
+                createOptionalAttrDef("attr3", AtlasBusinessMetadataDef.ATLAS_TYPE_SHORT, options, _description),
+                createOptionalAttrDef("attr4", AtlasBusinessMetadataDef.ATLAS_TYPE_INT, options, _description),
+                createOptionalAttrDef("attr5", AtlasBusinessMetadataDef.ATLAS_TYPE_LONG, options, _description),
+                createOptionalAttrDef("attr6", AtlasBusinessMetadataDef.ATLAS_TYPE_FLOAT, options, _description),
+                createOptionalAttrDef("attr7", AtlasBusinessMetadataDef.ATLAS_TYPE_DOUBLE, options, _description),
+                createOptionalAttrDef("attr8", AtlasBusinessMetadataDef.ATLAS_TYPE_STRING, options, _description),
+                createOptionalAttrDef("attr9", AtlasBusinessMetadataDef.ATLAS_TYPE_DATE, options, _description),
+                createOptionalAttrDef("attr10", "ENUM_1", options, _description));
+
+        AtlasBusinessMetadataDef bmWithAllTypesMV = createBusinessMetadataDef("bmWithAllTypesMV", _description, "1.0",
+                createOptionalAttrDef("attr11", "array<boolean>", options, _description),
+                createOptionalAttrDef("attr12", "array<byte>", options, _description),
+                createOptionalAttrDef("attr13", "array<short>", options, _description),
+                createOptionalAttrDef("attr14", "array<int>", options, _description),
+                createOptionalAttrDef("attr15", "array<long>", options, _description),
+                createOptionalAttrDef("attr16", "array<float>", options, _description),
+                createOptionalAttrDef("attr17", "array<double>", options, _description),
+                createOptionalAttrDef("attr18", "array<string>", options, _description),
+                createOptionalAttrDef("attr19", "array<date>", options, _description),
+                createOptionalAttrDef("attr20", "array<ENUM_1>", options, _description));
+
+        AtlasTypesDef ret = AtlasTypeUtil.getTypesDef(new ArrayList<>(),
+                new ArrayList<>(), new ArrayList<>(),
+                new ArrayList<>(), new ArrayList<>(),
+                Arrays.asList(bmNoApplicableTypes, bmNoAttributes, bmWithAllTypes, bmWithAllTypesMV));
+
+        populateSystemAttributes(ret);
+
+        return ret;
+    }
 
     public static AtlasTypesDef defineHiveTypes() {
         String _description = "_description";
@@ -747,7 +874,7 @@ public final class TestUtilsV2 {
                         createUniqueRequiredAttrDef("name", "string"),
                         createOptionalAttrDef("description", "string"),
                         createRequiredAttrDef("type", "string"),
-                        createOptionalAttrDef("created", "date"),
+                        createOptionalAttrDef("created", "string"),
                         // enum
                         new AtlasAttributeDef("tableType", "tableType", false,
                                 SINGLE, 1, 1,
@@ -1412,6 +1539,7 @@ public final class TestUtilsV2 {
         populateSystemAttributes(typesDef.getClassificationDefs());
         populateSystemAttributes(typesDef.getEntityDefs());
         populateSystemAttributes(typesDef.getRelationshipDefs());
+        populateSystemAttributes(typesDef.getBusinessMetadataDefs());
     }
 
     public static void populateSystemAttributes(List<? extends AtlasBaseTypeDef> typeDefs) {
@@ -1426,4 +1554,35 @@ public final class TestUtilsV2 {
         typeDef.setCreatedBy(TestUtilsV2.TEST_USER);
         typeDef.setUpdatedBy(TestUtilsV2.TEST_USER);
     }
+
+    public static InputStream getFile(String subDir, String fileName){
+        final String userDir  = System.getProperty("user.dir");
+        String filePath = getTestFilePath(userDir, subDir, fileName);
+        File file = new File(filePath);
+        InputStream  fs       = null;
+
+        try {
+            fs = new FileInputStream(file);
+        } catch (FileNotFoundException e) {
+            LOG.error("File could not be found at: {}", filePath, e);
+        }
+
+        return fs;
+    }
+
+    public static String getFileData(String subDir, String fileName)throws IOException {
+        final String userDir  = System.getProperty("user.dir");
+        String       filePath = getTestFilePath(userDir, subDir, fileName);
+        File f        = new File(filePath);
+        String ret = FileUtils.readFileToString(f, "UTF-8");
+        return ret;
+    }
+
+    private static String getTestFilePath(String startPath, String subDir, String fileName) {
+        if (StringUtils.isNotEmpty(subDir)) {
+            return startPath + "/src/test/resources/" + subDir + "/" + fileName;
+        } else {
+            return startPath + "/src/test/resources/" + fileName;
+        }
+    }
 }
