@@ -19,44 +19,48 @@ package org.apache.atlas.discovery;
 
 import org.apache.atlas.ApplicationProperties;
 import org.apache.atlas.AtlasException;
+import org.apache.atlas.SortOrder;
 import org.apache.atlas.exception.AtlasBaseException;
 import org.apache.atlas.model.discovery.SearchParameters;
 import org.apache.atlas.model.discovery.SearchParameters.FilterCriteria;
 import org.apache.atlas.model.discovery.SearchParameters.FilterCriteria.Condition;
 import org.apache.atlas.model.typedef.AtlasBaseTypeDef;
 import org.apache.atlas.repository.Constants;
+import org.apache.atlas.repository.graph.GraphHelper;
 import org.apache.atlas.repository.graphdb.AtlasGraphQuery;
 import org.apache.atlas.repository.graphdb.AtlasIndexQuery;
 import org.apache.atlas.repository.graphdb.AtlasVertex;
 import org.apache.atlas.repository.store.graph.v2.AtlasGraphUtilsV2;
-import org.apache.atlas.type.AtlasArrayType;
-import org.apache.atlas.type.AtlasEntityType;
-import org.apache.atlas.type.AtlasEnumType;
-import org.apache.atlas.type.AtlasStructType;
+import org.apache.atlas.type.*;
 import org.apache.atlas.type.AtlasStructType.AtlasAttribute;
-import org.apache.atlas.type.AtlasType;
 import org.apache.atlas.util.AtlasGremlinQueryProvider;
+import org.apache.atlas.util.SearchPredicateUtil;
 import org.apache.atlas.util.SearchPredicateUtil.*;
 import org.apache.commons.collections.CollectionUtils;
 import org.apache.commons.collections.Predicate;
 import org.apache.commons.collections.PredicateUtils;
 import org.apache.commons.lang.StringUtils;
+import org.apache.tinkerpop.gremlin.process.traversal.Order;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
 import java.math.BigDecimal;
 import java.math.BigInteger;
-import java.util.ArrayList;
-import java.util.Date;
-import java.util.HashMap;
-import java.util.HashSet;
-import java.util.Iterator;
-import java.util.LinkedList;
-import java.util.List;
-import java.util.Map;
-import java.util.Set;
+import java.util.*;
 import java.util.regex.Pattern;
 
+import static org.apache.atlas.SortOrder.ASCENDING;
+import static org.apache.atlas.discovery.SearchContext.MATCH_ALL_CLASSIFICATION_TYPES;
+import static org.apache.atlas.discovery.SearchContext.MATCH_ALL_CLASSIFIED;
+import static org.apache.atlas.discovery.SearchContext.MATCH_ALL_NOT_CLASSIFIED;
+import static org.apache.atlas.discovery.SearchContext.MATCH_ALL_WILDCARD_CLASSIFICATION;
+import static org.apache.atlas.repository.Constants.CLASSIFICATION_NAMES_KEY;
+import static org.apache.atlas.repository.Constants.CLASSIFICATION_NAME_DELIMITER;
+import static org.apache.atlas.repository.Constants.CUSTOM_ATTRIBUTES_PROPERTY_KEY;
+import static org.apache.atlas.repository.Constants.LABELS_PROPERTY_KEY;
+import static org.apache.atlas.repository.Constants.PROPAGATED_CLASSIFICATION_NAMES_KEY;
+import static org.apache.atlas.repository.Constants.PROPAGATED_TRAIT_NAMES_PROPERTY_KEY;
+import static org.apache.atlas.repository.Constants.TRAIT_NAMES_PROPERTY_KEY;
 import static org.apache.atlas.util.SearchPredicateUtil.*;
 
 public abstract class SearchProcessor {
@@ -74,7 +78,10 @@ public abstract class SearchProcessor {
     public static final String  SPACE_STRING               = " ";
     public static final String  BRACE_OPEN_STR             = "(";
     public static final String  BRACE_CLOSE_STR            = ")";
-
+    public static final String  ALL_TYPE_QUERY             = "[* TO *]";
+    public static final char    CUSTOM_ATTR_SEPARATOR      = '=';
+    public static final String  CUSTOM_ATTR_SEARCH_FORMAT  = "\"\\\"%s\\\":\\\"%s\\\"\"";
+    public static final String  CUSTOM_ATTR_SEARCH_FORMAT_GRAPH = "\"%s\":\"%s\"";
     private static final Map<SearchParameters.Operator, String>                            OPERATOR_MAP           = new HashMap<>();
     private static final Map<SearchParameters.Operator, VertexAttributePredicateGenerator> OPERATOR_PREDICATE_MAP = new HashMap<>();
 
@@ -113,6 +120,8 @@ public abstract class SearchProcessor {
         OPERATOR_MAP.put(SearchParameters.Operator.CONTAINS, INDEX_SEARCH_PREFIX + "\"%s\": (*%s*)");
         OPERATOR_PREDICATE_MAP.put(SearchParameters.Operator.CONTAINS, getContainsPredicateGenerator());
 
+        OPERATOR_PREDICATE_MAP.put(SearchParameters.Operator.NOT_CONTAINS, getNotContainsPredicateGenerator());
+
         // TODO: Add contains any, contains all mappings here
 
         OPERATOR_MAP.put(SearchParameters.Operator.IS_NULL, "(*:* NOT " + INDEX_SEARCH_PREFIX + "\"%s\":[* TO *])");
@@ -122,13 +131,14 @@ public abstract class SearchProcessor {
         OPERATOR_PREDICATE_MAP.put(SearchParameters.Operator.NOT_NULL, getNotNullPredicateGenerator());
     }
 
-    protected final SearchContext   context;
-    protected       SearchProcessor nextProcessor;
-    protected       Predicate       inMemoryPredicate;
-
+    protected final SearchContext          context;
+    protected       SearchProcessor        nextProcessor;
+    protected       Predicate              inMemoryPredicate;
+    protected       GraphIndexQueryBuilder graphIndexQueryBuilder;
 
     protected SearchProcessor(SearchContext context) {
         this.context = context;
+        this.graphIndexQueryBuilder = new GraphIndexQueryBuilder(context);
     }
 
     public void addProcessor(SearchProcessor processor) {
@@ -140,6 +150,26 @@ public abstract class SearchProcessor {
     }
 
     public abstract List<AtlasVertex> execute();
+    public abstract long getResultCount();
+
+    protected boolean isEntityRootType() {
+        return context.getEntityType() == SearchContext.MATCH_ALL_ENTITY_TYPES;
+    }
+
+    protected boolean isClassificationRootType() {
+        return context.getClassificationType() == SearchContext.MATCH_ALL_CLASSIFICATION_TYPES;
+    }
+
+    protected boolean isSystemAttribute(String attrName) {
+        return AtlasEntityType.getEntityRoot().hasAttribute(attrName) || AtlasClassificationType.getClassificationRoot().hasAttribute(attrName);
+    }
+
+    protected boolean isPipeSeparatedSystemAttribute(String attrName) {
+        return StringUtils.equals(attrName, CLASSIFICATION_NAMES_KEY) ||
+               StringUtils.equals(attrName, PROPAGATED_CLASSIFICATION_NAMES_KEY) ||
+               StringUtils.equals(attrName, LABELS_PROPERTY_KEY) ||
+               StringUtils.equals(attrName, CUSTOM_ATTRIBUTES_PROPERTY_KEY);
+    }
 
     protected int collectResultVertices(final List<AtlasVertex> ret, final int startIdx, final int limit, int resultIdx, final List<AtlasVertex> entityVertices) {
         for (AtlasVertex entityVertex : entityVertices) {
@@ -165,6 +195,27 @@ public abstract class SearchProcessor {
         }
     }
 
+    protected Predicate buildTraitPredict(AtlasClassificationType classificationType) {
+        Predicate traitPredicate;
+        if (classificationType == MATCH_ALL_WILDCARD_CLASSIFICATION || classificationType == MATCH_ALL_CLASSIFIED || classificationType == MATCH_ALL_CLASSIFICATION_TYPES) {
+            traitPredicate = PredicateUtils.orPredicate(SearchPredicateUtil.getNotEmptyPredicateGenerator().generatePredicate(TRAIT_NAMES_PROPERTY_KEY, null, List.class),
+                SearchPredicateUtil.getNotEmptyPredicateGenerator().generatePredicate(PROPAGATED_TRAIT_NAMES_PROPERTY_KEY, null, List.class));
+        } else if (classificationType == MATCH_ALL_NOT_CLASSIFIED) {
+            traitPredicate = PredicateUtils.andPredicate(SearchPredicateUtil.getIsNullOrEmptyPredicateGenerator().generatePredicate(TRAIT_NAMES_PROPERTY_KEY, null, List.class),
+                SearchPredicateUtil.getIsNullOrEmptyPredicateGenerator().generatePredicate(PROPAGATED_TRAIT_NAMES_PROPERTY_KEY, null, List.class));
+        } else if (context.isWildCardSearch()) {
+            //For wildcard search __classificationNames which of String type is taken instead of _traitNames which is of Array type
+            //No need to escape, as classification Names only support letters,numbers,space and underscore
+            String regexString = getRegexString("\\|" + context.getClassificationName() + "\\|");
+            traitPredicate = PredicateUtils.orPredicate(SearchPredicateUtil.getRegexPredicateGenerator().generatePredicate(CLASSIFICATION_NAMES_KEY, regexString, String.class),
+                    SearchPredicateUtil.getRegexPredicateGenerator().generatePredicate(PROPAGATED_CLASSIFICATION_NAMES_KEY, regexString, String.class));
+        } else {
+            traitPredicate = PredicateUtils.orPredicate(SearchPredicateUtil.getContainsAnyPredicateGenerator().generatePredicate(TRAIT_NAMES_PROPERTY_KEY, context.getClassificationTypes(), List.class),
+                SearchPredicateUtil.getContainsAnyPredicateGenerator().generatePredicate(PROPAGATED_TRAIT_NAMES_PROPERTY_KEY, context.getClassificationTypes(), List.class));
+        }
+        return traitPredicate;
+    }
+
 
     protected void processSearchAttributes(AtlasStructType structType, FilterCriteria filterCriteria, Set<String> indexFiltered, Set<String> graphFiltered, Set<String> allAttributes) {
         if (structType == null || filterCriteria == null) {
@@ -179,18 +230,52 @@ public abstract class SearchProcessor {
                 processSearchAttributes(structType, criteria, indexFiltered, graphFiltered, allAttributes);
             }
         } else if (StringUtils.isNotEmpty(filterCriteria.getAttributeName())) {
-            try {
-                String attributeName = filterCriteria.getAttributeName();
+            String attributeName = filterCriteria.getAttributeName();
 
+            if (StringUtils.equals(attributeName, Constants.IS_INCOMPLETE_PROPERTY_KEY)) {
+                // when entity is incomplete (i.e. shell entity):
+                //   vertex property IS_INCOMPLETE_PROPERTY_KEY will be set to INCOMPLETE_ENTITY_VALUE
+                // when entity is not incomplete (i.e. not a shell entity):
+                //   vertex property IS_INCOMPLETE_PROPERTY_KEY will not be set
+
+                String attributeValue = filterCriteria.getAttributeValue();
+
+                switch (filterCriteria.getOperator()) {
+                    case EQ:
+                        if (attributeValue == null || StringUtils.equals(attributeValue, "0") || StringUtils.equalsIgnoreCase(attributeValue, "false")) {
+                            filterCriteria.setOperator(SearchParameters.Operator.IS_NULL);
+                        } else {
+                            filterCriteria.setOperator(SearchParameters.Operator.EQ);
+                            filterCriteria.setAttributeValue(Constants.INCOMPLETE_ENTITY_VALUE.toString());
+                        }
+                    break;
+
+                    case NEQ:
+                        if (attributeValue == null || StringUtils.equals(attributeValue, "0") || StringUtils.equalsIgnoreCase(attributeValue, "false")) {
+                            filterCriteria.setOperator(SearchParameters.Operator.EQ);
+                            filterCriteria.setAttributeValue(Constants.INCOMPLETE_ENTITY_VALUE.toString());
+                        } else {
+                            filterCriteria.setOperator(SearchParameters.Operator.IS_NULL);
+                        }
+                    break;
+
+                    case NOT_NULL:
+                        filterCriteria.setOperator(SearchParameters.Operator.EQ);
+                        filterCriteria.setAttributeValue(Constants.INCOMPLETE_ENTITY_VALUE.toString());
+                    break;
+                }
+            }
+
+            try {
                 if (isIndexSearchable(filterCriteria, structType)) {
                     indexFiltered.add(attributeName);
                 } else {
-                    LOG.warn("not using index-search for attribute '{}'; might cause poor performance", structType.getQualifiedAttributeName(attributeName));
+                    LOG.warn("not using index-search for attribute '{}'; might cause poor performance", structType.getVertexPropertyName(attributeName));
 
                     graphFiltered.add(attributeName);
                 }
 
-                if (structType instanceof AtlasEntityType) {
+                if (structType instanceof AtlasEntityType && !isSystemAttribute(attributeName)) {
                     // Capture the entity attributes
                     context.getEntityAttributes().add(attributeName);
                 }
@@ -214,7 +299,7 @@ public abstract class SearchProcessor {
     //      (AND (OR idx-att1=x idx-attr1=y) non-idx-attr=z (AND idx-attr2=xyz idx-attr2=abc))
     //
     protected boolean canApplyIndexFilter(AtlasStructType structType, FilterCriteria filterCriteria, boolean insideOrCondition) {
-        if (filterCriteria == null) {
+        if (!context.hasAttributeFilter(filterCriteria)) {
             return true;
         }
 
@@ -250,13 +335,31 @@ public abstract class SearchProcessor {
         return ret;
     }
 
-    protected void constructTypeTestQuery(StringBuilder indexQuery, String typeAndAllSubTypesQryStr) {
-        if (StringUtils.isNotEmpty(typeAndAllSubTypesQryStr)) {
-            if (indexQuery.length() > 0) {
-                indexQuery.append(AND_STR);
-            }
+    protected void filterWhiteSpaceClassification(List<AtlasVertex> entityVertices) {
+        if (CollectionUtils.isNotEmpty(entityVertices)) {
+            final Iterator<AtlasVertex> it              = entityVertices.iterator();
+            final Set<String>           typeAndSubTypes = context.getClassificationTypes();
 
-            indexQuery.append(INDEX_SEARCH_PREFIX + "\"").append(Constants.TYPE_NAME_PROPERTY_KEY).append("\":").append(typeAndAllSubTypesQryStr);
+            while (it.hasNext()) {
+                AtlasVertex  entityVertex        = it.next();
+                List<String> classificationNames = AtlasGraphUtilsV2.getClassificationNames(entityVertex);
+
+                if (CollectionUtils.isNotEmpty(classificationNames)) {
+                    if (typeAndSubTypes.isEmpty() || CollectionUtils.containsAny(classificationNames, typeAndSubTypes)) {
+                        continue;
+                    }
+                }
+
+                List<String> propagatedClassificationNames = AtlasGraphUtilsV2.getPropagatedClassificationNames(entityVertex);
+
+                if (CollectionUtils.isNotEmpty(propagatedClassificationNames)) {
+                    if (typeAndSubTypes.isEmpty() || CollectionUtils.containsAny(propagatedClassificationNames, typeAndSubTypes)) {
+                        continue;
+                    }
+                }
+
+                it.remove();
+            }
         }
     }
 
@@ -333,16 +436,8 @@ public abstract class SearchProcessor {
         }
     }
 
-    protected void constructStateTestQuery(StringBuilder indexQuery) {
-        if (indexQuery.length() > 0) {
-            indexQuery.append(AND_STR);
-        }
-
-        indexQuery.append(INDEX_SEARCH_PREFIX + "\"").append(Constants.STATE_PROPERTY_KEY).append("\":ACTIVE");
-    }
-
     private boolean isIndexSearchable(FilterCriteria filterCriteria, AtlasStructType structType) throws AtlasBaseException {
-        String      qualifiedName = structType.getQualifiedAttributeName(filterCriteria.getAttributeName());
+        String      qualifiedName = structType.getVertexPropertyName(filterCriteria.getAttributeName());
         Set<String> indexedKeys   = context.getIndexedKeys();
         boolean     ret           = indexedKeys != null && indexedKeys.contains(qualifiedName);
 
@@ -359,7 +454,7 @@ public abstract class SearchProcessor {
                     }
 
                     ret = false;
-                } else if (hasIndexQuerySpecialChar(filterCriteria.getAttributeValue())) {
+                } else if (hasIndexQuerySpecialChar(filterCriteria.getAttributeValue()) && !isPipeSeparatedSystemAttribute(filterCriteria.getAttributeName())) {
                     if (LOG.isDebugEnabled()) {
                         LOG.debug("special characters found in filter value {}, deferring to in-memory or graph query (might cause poor performance)", filterCriteria.getAttributeValue());
                     }
@@ -433,22 +528,76 @@ public abstract class SearchProcessor {
                     return PredicateUtils.anyPredicate(predicates);
                 }
             }
-        } else if (indexAttributes.contains(criteria.getAttributeName())){
-            return toInMemoryPredicate(type, criteria.getAttributeName(), criteria.getOperator(), criteria.getAttributeValue());
+        } else if (indexAttributes.contains(criteria.getAttributeName())) {
+            String                    attrName  = criteria.getAttributeName();
+            String                    attrValue = criteria.getAttributeValue();
+            SearchParameters.Operator operator  = criteria.getOperator();
+
+            //process attribute value and attribute operator for pipeSeperated fields
+            if (isPipeSeparatedSystemAttribute(attrName)) {
+                FilterCriteria processedCriteria = processPipeSeperatedSystemAttribute(attrName, operator, attrValue);
+                attrValue                        = processedCriteria.getAttributeValue();
+                operator                         = processedCriteria.getOperator();
+            }
+
+            return toInMemoryPredicate(type, attrName, operator, attrValue);
         }
 
         return null;
     }
 
+    private FilterCriteria processPipeSeperatedSystemAttribute(String attrName, SearchParameters.Operator op, String attrVal) {
+
+        FilterCriteria ret = new FilterCriteria();
+
+        if (op != null && attrVal != null) {
+            switch (op) {
+                case STARTS_WITH:
+                    attrVal = CLASSIFICATION_NAME_DELIMITER + attrVal;
+                    op      = SearchParameters.Operator.CONTAINS;
+                    break;
+                case ENDS_WITH:
+                    attrVal = attrVal + CLASSIFICATION_NAME_DELIMITER;
+                    op      = SearchParameters.Operator.CONTAINS;
+                    break;
+                case EQ:
+                    attrVal = GraphHelper.getDelimitedClassificationNames(Collections.singleton(attrVal));
+                    op      = SearchParameters.Operator.CONTAINS;
+                    break;
+                case NEQ:
+                    attrVal = GraphHelper.getDelimitedClassificationNames(Collections.singleton(attrVal));
+                    op      = SearchParameters.Operator.NOT_CONTAINS;
+                    break;
+                case CONTAINS:
+                    if (attrName.equals(CUSTOM_ATTRIBUTES_PROPERTY_KEY)) {
+                        attrVal = getCustomAttributeIndexQueryValue(attrVal, true);
+                    }
+                    break;
+            }
+        }
+
+        ret.setAttributeName(attrName);
+        ret.setOperator(op);
+        ret.setAttributeValue(attrVal);
+
+        return ret;
+    }
+
     private String toIndexExpression(AtlasStructType type, String attrName, SearchParameters.Operator op, String attrVal) {
         String ret = EMPTY_STRING;
 
         try {
             if (OPERATOR_MAP.get(op) != null) {
-                String qualifiedName         = type.getQualifiedAttributeName(attrName);
+                String qualifiedName         = type.getVertexPropertyName(attrName);
                 String escapeIndexQueryValue = AtlasAttribute.escapeIndexQueryValue(attrVal);
 
-                ret = String.format(OPERATOR_MAP.get(op), qualifiedName, escapeIndexQueryValue);
+                // map '__customAttributes' 'CONTAINS' operator to 'EQ' operator (solr limitation for json serialized string search)
+                // map '__customAttributes' value from 'key1=value1' to '\"key1\":\"value1\"' (escape special characters and surround with quotes)
+                if (attrName.equals(CUSTOM_ATTRIBUTES_PROPERTY_KEY) && op == SearchParameters.Operator.CONTAINS) {
+                    ret = String.format(OPERATOR_MAP.get(op), qualifiedName, getCustomAttributeIndexQueryValue(escapeIndexQueryValue, false));
+                } else {
+                    ret = String.format(OPERATOR_MAP.get(op), qualifiedName, escapeIndexQueryValue);
+                }
             }
         } catch (AtlasBaseException ex) {
             LOG.warn(ex.getMessage());
@@ -457,6 +606,28 @@ public abstract class SearchProcessor {
         return ret;
     }
 
+    private String getCustomAttributeIndexQueryValue(String attrValue, boolean forGraphQuery) {
+        String ret = null;
+
+        if (StringUtils.isNotEmpty(attrValue)) {
+            int    separatorIdx = attrValue.indexOf(CUSTOM_ATTR_SEPARATOR);
+            String key          = separatorIdx != -1 ? attrValue.substring(0, separatorIdx) : null;
+            String value        = key != null ? attrValue.substring(separatorIdx + 1) : null;
+
+            if (key != null && value != null) {
+                if (forGraphQuery) {
+                    ret = String.format(CUSTOM_ATTR_SEARCH_FORMAT_GRAPH, key, value);
+                } else {
+                    ret = String.format(CUSTOM_ATTR_SEARCH_FORMAT, key, value);
+                }
+            } else {
+                ret = attrValue;
+            }
+        }
+
+        return ret;
+    }
+
     private Predicate toInMemoryPredicate(AtlasStructType type, String attrName, SearchParameters.Operator op, String attrVal) {
         Predicate ret = null;
 
@@ -524,7 +695,10 @@ public abstract class SearchProcessor {
                     break;
             }
 
-            ret = predicate.generatePredicate(attribute.getQualifiedName(), attrValue, attrClass);
+            String vertexPropertyName = attribute.getVertexPropertyName();
+            ret = predicate.generatePredicate(
+                    StringUtils.isEmpty(vertexPropertyName) ? attribute.getQualifiedName() : vertexPropertyName,
+                    attrValue, attrClass);
         }
 
         return ret;
@@ -557,52 +731,55 @@ public abstract class SearchProcessor {
                 String                    attrValue = criteria.getAttributeValue();
                 SearchParameters.Operator operator  = criteria.getOperator();
 
-                try {
-                    final String qualifiedName = type.getQualifiedAttributeName(attrName);
+                //process attribute value and attribute operator for pipeSeperated fields
+                if (isPipeSeparatedSystemAttribute(attrName)) {
+                    FilterCriteria processedCriteria = processPipeSeperatedSystemAttribute(attrName, operator, attrValue);
+                    attrValue                        = processedCriteria.getAttributeValue();
+                    operator                         = processedCriteria.getOperator();
+                }
 
-                    switch (operator) {
-                        case LT:
-                            query.has(qualifiedName, AtlasGraphQuery.ComparisionOperator.LESS_THAN, attrValue);
-                            break;
-                        case LTE:
-                            query.has(qualifiedName, AtlasGraphQuery.ComparisionOperator.LESS_THAN_EQUAL, attrValue);
-                            break;
-                        case GT:
-                            query.has(qualifiedName, AtlasGraphQuery.ComparisionOperator.GREATER_THAN, attrValue);
-                            break;
-                        case GTE:
-                            query.has(qualifiedName, AtlasGraphQuery.ComparisionOperator.GREATER_THAN_EQUAL, attrValue);
-                            break;
-                        case EQ:
-                            query.has(qualifiedName, AtlasGraphQuery.ComparisionOperator.EQUAL, attrValue);
-                            break;
-                        case NEQ:
-                            query.has(qualifiedName, AtlasGraphQuery.ComparisionOperator.NOT_EQUAL, attrValue);
-                            break;
-                        case LIKE:
-                            query.has(qualifiedName, AtlasGraphQuery.MatchingOperator.REGEX, attrValue);
-                            break;
-                        case CONTAINS:
-                            query.has(qualifiedName, AtlasGraphQuery.MatchingOperator.REGEX, getContainsRegex(attrValue));
-                            break;
-                        case STARTS_WITH:
-                            query.has(qualifiedName, AtlasGraphQuery.MatchingOperator.PREFIX, attrValue);
-                            break;
-                        case ENDS_WITH:
-                            query.has(qualifiedName, AtlasGraphQuery.MatchingOperator.REGEX, getSuffixRegex(attrValue));
-                            break;
-                        case IS_NULL:
-                            query.has(qualifiedName, AtlasGraphQuery.ComparisionOperator.EQUAL, null);
-                            break;
-                        case NOT_NULL:
-                            query.has(qualifiedName, AtlasGraphQuery.ComparisionOperator.NOT_EQUAL, null);
-                            break;
-                        default:
-                            LOG.warn("{}: unsupported operator. Ignored", operator);
-                            break;
-                    }
-                } catch (AtlasBaseException e) {
-                    LOG.error("toGraphFilterQuery(): failed for attrName=" + attrName + "; operator=" + operator + "; attrValue=" + attrValue, e);
+                final String qualifiedName           =  type.getAttribute(attrName).getVertexPropertyName();
+
+                switch (operator) {
+                    case LT:
+                        query.has(qualifiedName, AtlasGraphQuery.ComparisionOperator.LESS_THAN, attrValue);
+                        break;
+                    case LTE:
+                        query.has(qualifiedName, AtlasGraphQuery.ComparisionOperator.LESS_THAN_EQUAL, attrValue);
+                        break;
+                    case GT:
+                        query.has(qualifiedName, AtlasGraphQuery.ComparisionOperator.GREATER_THAN, attrValue);
+                        break;
+                    case GTE:
+                        query.has(qualifiedName, AtlasGraphQuery.ComparisionOperator.GREATER_THAN_EQUAL, attrValue);
+                        break;
+                    case EQ:
+                        query.has(qualifiedName, AtlasGraphQuery.ComparisionOperator.EQUAL, attrValue);
+                        break;
+                    case NEQ:
+                        query.has(qualifiedName, AtlasGraphQuery.ComparisionOperator.NOT_EQUAL, attrValue);
+                        break;
+                    case LIKE:
+                        query.has(qualifiedName, AtlasGraphQuery.MatchingOperator.REGEX, attrValue);
+                        break;
+                    case CONTAINS:
+                        query.has(qualifiedName, AtlasGraphQuery.MatchingOperator.REGEX, getContainsRegex(attrValue));
+                        break;
+                    case STARTS_WITH:
+                        query.has(qualifiedName, AtlasGraphQuery.MatchingOperator.PREFIX, attrValue);
+                        break;
+                    case ENDS_WITH:
+                        query.has(qualifiedName, AtlasGraphQuery.MatchingOperator.REGEX, getSuffixRegex(attrValue));
+                        break;
+                    case IS_NULL:
+                        query.has(qualifiedName, AtlasGraphQuery.ComparisionOperator.EQUAL, null);
+                        break;
+                    case NOT_NULL:
+                        query.has(qualifiedName, AtlasGraphQuery.ComparisionOperator.NOT_EQUAL, null);
+                        break;
+                    default:
+                        LOG.warn("{}: unsupported operator. Ignored", operator);
+                        break;
                 }
             }
         }
@@ -672,6 +849,10 @@ public abstract class SearchProcessor {
         return ".*" + escapeRegExChars(attributeValue) + ".*";
     }
 
+    private static String getRegexString(String value) {
+        return ".*" + value.replace("*", ".*") + ".*";
+    }
+
     private static String getSuffixRegex(String attributeValue) {
         return ".*" + escapeRegExChars(attributeValue);
     }
@@ -755,7 +936,7 @@ public abstract class SearchProcessor {
         return false;
     }
 
-    protected List<AtlasVertex> getVerticesFromIndexQueryResult(Iterator<AtlasIndexQuery.Result> idxQueryResult, List<AtlasVertex> vertices) {
+    protected Collection<AtlasVertex> getVerticesFromIndexQueryResult(Iterator<AtlasIndexQuery.Result> idxQueryResult, Collection<AtlasVertex> vertices) {
         if (idxQueryResult != null) {
             while (idxQueryResult.hasNext()) {
                 AtlasVertex vertex = idxQueryResult.next().getVertex();
@@ -767,7 +948,7 @@ public abstract class SearchProcessor {
         return vertices;
     }
 
-    protected List<AtlasVertex> getVertices(Iterator<AtlasVertex> iterator, List<AtlasVertex> vertices) {
+    protected Collection<AtlasVertex> getVertices(Iterator<AtlasVertex> iterator, Collection<AtlasVertex> vertices) {
         if (iterator != null) {
             while (iterator.hasNext()) {
                 AtlasVertex vertex = iterator.next();
@@ -805,4 +986,29 @@ public abstract class SearchProcessor {
         return defaultValue;
     }
 
+    private static String getSortByAttribute(SearchContext context) {
+        final AtlasEntityType entityType = context.getEntityType();
+        String sortBy = context.getSearchParameters().getSortBy();
+        AtlasStructType.AtlasAttribute sortByAttribute = entityType != null ? entityType.getAttribute(sortBy) : null;
+        if (sortByAttribute != null) {
+            return sortByAttribute.getVertexPropertyName();
+        }
+        return null;
+    }
+
+    private static Order getSortOrderAttribute(SearchContext context) {
+        SortOrder sortOrder = context.getSearchParameters().getSortOrder();
+        if (sortOrder == null) sortOrder = ASCENDING;
+
+        return sortOrder == SortOrder.ASCENDING ? Order.asc : Order.desc;
+    }
+
+    protected static Iterator<AtlasIndexQuery.Result> executeIndexQuery(SearchContext context, AtlasIndexQuery indexQuery, int qryOffset, int limit) {
+        String sortBy = getSortByAttribute(context);
+        if (sortBy != null && !sortBy.isEmpty()) {
+            Order sortOrder = getSortOrderAttribute(context);
+            return indexQuery.vertices(qryOffset, limit, sortBy, sortOrder);
+        }
+        return indexQuery.vertices(qryOffset, limit);
+    }
 }
