@@ -28,19 +28,26 @@ import org.apache.atlas.model.instance.AtlasObjectId;
 import org.apache.atlas.model.instance.AtlasRelatedObjectId;
 import org.apache.atlas.model.instance.AtlasRelationship;
 import org.apache.atlas.model.instance.AtlasStruct;
+import org.apache.atlas.repository.graphdb.AtlasVertex;
 import org.apache.atlas.repository.ogm.DataAccess;
 import org.apache.atlas.repository.store.graph.AtlasRelationshipStore;
+import org.apache.atlas.repository.store.graph.v2.AtlasGraphUtilsV2;
 import org.apache.atlas.type.AtlasRelationshipType;
 import org.apache.atlas.type.AtlasTypeRegistry;
+import org.apache.atlas.util.FileUtils;
 import org.apache.commons.collections.CollectionUtils;
 import org.apache.commons.collections.MapUtils;
 import org.apache.commons.lang.StringUtils;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
+import java.util.ArrayList;
+import java.util.Arrays;
 import java.util.Collection;
 import java.util.Collections;
 import java.util.HashMap;
+import java.util.HashSet;
+import java.util.List;
 import java.util.Map;
 import java.util.Objects;
 import java.util.Set;
@@ -522,4 +529,205 @@ public class GlossaryTermUtils extends GlossaryUtils {
         }
     }
 
+    protected List<AtlasGlossaryTerm> getGlossaryTermDataList(List<String[]> fileData, List<String> failedTermMsgs) throws AtlasBaseException {
+        List<AtlasGlossaryTerm> glossaryTerms     = new ArrayList<>();
+        Map<String, String>     glossaryNameCache = new HashMap<>();
+
+        for (String[] record : fileData) {
+            AtlasGlossaryTerm glossaryTerm = new AtlasGlossaryTerm();
+
+            if ((record.length < 1) || StringUtils.isBlank(record[0])) {
+                LOG.error("The GlossaryName is blank for the record : ", Arrays.toString(record));
+                failedTermMsgs.add("The GlossaryName is blank for the record : " + Arrays.toString(record));
+            }
+
+            String glossaryName = record[0];
+            String glossaryGuid;
+
+            if (glossaryNameCache.get(glossaryName) != null) {
+                glossaryGuid = glossaryNameCache.get(glossaryName);
+
+            } else {
+                AtlasVertex vertex = AtlasGraphUtilsV2.findByTypeAndUniquePropertyName(GlossaryUtils.ATLAS_GLOSSARY_TYPENAME, GlossaryUtils.ATLAS_GLOSSARY_TYPENAME + "." + QUALIFIED_NAME_ATTR, glossaryName);
+
+                glossaryGuid = (vertex != null) ? AtlasGraphUtilsV2.getIdFromVertex(vertex) : null;
+            }
+
+            if (glossaryGuid == null) {
+                if (GlossaryService.isNameInvalid(glossaryName)) {
+                    LOG.error("The provided Glossary Name is invalid : " + glossaryName);
+                    failedTermMsgs.add("The provided Glossary Name is invalid : " + glossaryName);
+                } else {
+                    AtlasGlossary glossary = new AtlasGlossary();
+                    glossary.setQualifiedName(glossaryName);
+                    glossary.setName(glossaryName);
+
+                    glossary      = dataAccess.save(glossary);
+                    glossaryGuid  = glossary.getGuid();
+                }
+            }
+
+            if (glossaryGuid != null) {
+                glossaryNameCache.put(glossaryName, glossaryGuid);
+                glossaryTerm = populateGlossaryTermObject(failedTermMsgs, record, glossaryGuid);
+                glossaryTerms.add(glossaryTerm);
+            }
+        }
+
+        if (failedTermMsgs.size() == 0) {
+            return glossaryTerms;
+        } else {
+            throw new AtlasBaseException("The uploaded file has not been processed due to the following errors : " + "\n" + failedTermMsgs.toString());
+        }
+    }
+
+    public static String getGlossaryTermHeaders() {
+        List<String> ret = new ArrayList<>();
+
+        ret.add("GlossaryName");
+        ret.add("TermName");
+        ret.add("ShortDescription");
+        ret.add("LongDescription");
+        ret.add("Examples");
+        ret.add("Abbreviation");
+        ret.add("Usage");
+        ret.add("AdditionalAttributes");
+        ret.add("TranslationTerms");
+        ret.add("ValidValuesFor");
+        ret.add("Synonyms");
+        ret.add("ReplacedBy");
+        ret.add("ValidValues");
+        ret.add("ReplacementTerms");
+        ret.add("SeeAlso");
+        ret.add("TranslatedTerms");
+        ret.add("IsA");
+        ret.add("Antonyms");
+        ret.add("Classifies");
+        ret.add("PreferredToTerms");
+        ret.add("PreferredTerms");
+
+        return String.join(", ", ret);
+    }
+
+    protected Map getMapValue(String csvRecord, List<String> failedTermMsgs) {
+        Map ret = null;
+
+        if (StringUtils.isNotBlank(csvRecord)) {
+            ret                     = new HashMap<>();
+            String csvRecordArray[] = csvRecord.split(FileUtils.ESCAPE_CHARACTER + FileUtils.PIPE_CHARACTER);
+            String recordArray[];
+
+            for (String record : csvRecordArray) {
+                recordArray = record.split(FileUtils.COLON_CHARACTER);
+
+                if ((recordArray.length % 2) == 0) {
+                    ret.put(recordArray[0], recordArray[1]);
+                } else {
+                    failedTermMsgs.add("\n" + "The Data in the uploaded file is incorrectly specified  : " + csvRecord);
+                }
+            }
+        }
+
+        return ret;
+    }
+
+    protected List getListValue(String csvRecord) {
+        List ret = null;
+
+        if (StringUtils.isNotBlank(csvRecord)) {
+            ret =  Arrays.asList(csvRecord.split(FileUtils.ESCAPE_CHARACTER + FileUtils.PIPE_CHARACTER));
+        }
+
+        return ret;
+    }
+
+    protected Set getAtlasRelatedTermHeaderSet(String csvRecord, String termName, String glossaryName, List<String> failedTermMsgs) {
+        Set ret = null;
+
+        if (StringUtils.isNotBlank(csvRecord)) {
+            ret                                     = new HashSet();
+            String                 csvRecordArray[] = csvRecord.split(FileUtils.ESCAPE_CHARACTER + FileUtils.PIPE_CHARACTER);
+            AtlasRelatedTermHeader relatedTermHeader;
+
+            for (String data : csvRecordArray) {
+                AtlasVertex vertex      = null;
+                String      dataArray[] = data.split(FileUtils.ESCAPE_CHARACTER + FileUtils.COLON_CHARACTER);
+
+                if ((dataArray.length % 2) == 0) {
+                    vertex = AtlasGraphUtilsV2.findByTypeAndUniquePropertyName(GlossaryUtils.ATLAS_GLOSSARY_TERM_TYPENAME,
+                            GlossaryUtils.ATLAS_GLOSSARY_TERM_TYPENAME + invalidNameChars[1] + QUALIFIED_NAME_ATTR, dataArray[1] + invalidNameChars[0] + dataArray[0]);
+                } else {
+                    failedTermMsgs.add("\n" + "Either incorrect data specified for Term or Term does not exist : " +termName);
+                }
+
+                if (vertex != null) {
+                    String glossaryTermGuid = AtlasGraphUtilsV2.getIdFromVertex(vertex);
+                    relatedTermHeader       = new AtlasRelatedTermHeader();
+                    relatedTermHeader.setTermGuid(glossaryTermGuid);
+                    ret.add(relatedTermHeader);
+                } else {
+                    failedTermMsgs.add("\n" + "The provided Reference Glossary and TermName does not exist in the system " +
+                            dataArray[1] + FileUtils.COLON_CHARACTER + dataArray[0] + " for record with TermName  : " + termName + " and GlossaryName : " + glossaryName);
+                }
+            }
+
+            return ret;
+        }
+
+        return ret;
+    }
+
+    protected AtlasGlossaryTerm populateGlossaryTermObject(List<String> failedTermMsgList, String[] record, String glossaryGuid) {
+        AtlasGlossaryTerm ret    = new AtlasGlossaryTerm();
+        int               i      = 0;
+        int               length = record.length;
+
+        ret.setName((length > ++i) ? record[i] : null);
+
+        if (!StringUtils.isNotBlank(ret.getName())) {
+            failedTermMsgList.add("\n" + "The TermName is blank for provided record: " + Arrays.toString(record));
+        } else {
+            ret.setShortDescription((length > ++i) ? record[i] : null);
+
+            ret.setLongDescription((length > ++i) ? record[i] : null);
+
+            ret.setExamples((length > ++i) ? (List<String>) getListValue(record[i]) : null);
+
+            ret.setAbbreviation((length > ++i) ? record[i] : null);
+
+            ret.setUsage((length > ++i) ? record[i] : null);
+
+            ret.setAdditionalAttributes(((length > ++i) ? (Map<String, Object>) getMapValue(record[i], failedTermMsgList) : null));
+
+            ret.setTranslationTerms((length > ++i) ? (Set<AtlasRelatedTermHeader>) getAtlasRelatedTermHeaderSet(record[i], ret.getName(), record[0], failedTermMsgList) : null);
+
+            ret.setValidValuesFor((length > ++i) ? (Set<AtlasRelatedTermHeader>) getAtlasRelatedTermHeaderSet(record[i], ret.getName(), record[0], failedTermMsgList) : null);
+
+            ret.setSynonyms((length > ++i) ? (Set<AtlasRelatedTermHeader>) getAtlasRelatedTermHeaderSet(record[i], ret.getName(), record[0], failedTermMsgList) : null);
+
+            ret.setReplacedBy((length > ++i) ? (Set<AtlasRelatedTermHeader>) getAtlasRelatedTermHeaderSet(record[i], ret.getName(), record[0], failedTermMsgList) : null);
+
+            ret.setValidValues((length > ++i) ? (Set<AtlasRelatedTermHeader>) getAtlasRelatedTermHeaderSet(record[i], ret.getName(), record[0], failedTermMsgList) : null);
+
+            ret.setReplacementTerms((length > ++i) ? (Set<AtlasRelatedTermHeader>) getAtlasRelatedTermHeaderSet(record[i], ret.getName(), record[0], failedTermMsgList) : null);
+
+            ret.setSeeAlso((length > ++i) ? (Set<AtlasRelatedTermHeader>) getAtlasRelatedTermHeaderSet(record[i], ret.getName(), record[0], failedTermMsgList) : null);
+
+            ret.setTranslatedTerms((length > ++i) ? (Set<AtlasRelatedTermHeader>) getAtlasRelatedTermHeaderSet(record[i], ret.getName(), record[0], failedTermMsgList) : null);
+
+            ret.setIsA((length > ++i) ? (Set<AtlasRelatedTermHeader>) getAtlasRelatedTermHeaderSet(record[i], ret.getName(), record[0], failedTermMsgList) : null);
+
+            ret.setAnchor(new AtlasGlossaryHeader(glossaryGuid));
+
+            ret.setAntonyms((length > ++i) ? (Set<AtlasRelatedTermHeader>) getAtlasRelatedTermHeaderSet(record[i], ret.getName(), record[0], failedTermMsgList) : null);
+
+            ret.setClassifies((length > ++i) ? (Set<AtlasRelatedTermHeader>) getAtlasRelatedTermHeaderSet(record[i], ret.getName(), record[0], failedTermMsgList) : null);
+
+            ret.setPreferredToTerms((length > ++i) ? (Set<AtlasRelatedTermHeader>) getAtlasRelatedTermHeaderSet(record[i], ret.getName(), record[0], failedTermMsgList) : null);
+
+            ret.setPreferredTerms((length > ++i) ? (Set<AtlasRelatedTermHeader>) getAtlasRelatedTermHeaderSet(record[i], ret.getName(), record[0], failedTermMsgList) : null);
+        }
+
+        return ret;
+    }
 }
