@@ -43,16 +43,20 @@ import java.util.Map;
 import java.util.Objects;
 import java.util.Set;
 import java.util.TimeZone;
+import java.util.regex.Pattern;
 import java.util.stream.Collectors;
 import java.util.stream.Stream;
 
 import static org.apache.atlas.model.discovery.SearchParameters.ALL_CLASSIFICATIONS;
 import static org.apache.atlas.model.discovery.SearchParameters.NO_CLASSIFICATIONS;
+import static org.apache.atlas.type.AtlasStructType.AtlasAttribute.AtlasRelationshipEdgeDirection.IN;
+import static org.apache.atlas.type.AtlasStructType.AtlasAttribute.AtlasRelationshipEdgeDirection.OUT;
 
 public class GremlinQueryComposer {
     private static final Logger LOG                 = LoggerFactory.getLogger(GremlinQueryComposer.class);
     private static final String ISO8601_FORMAT      = "yyyy-MM-dd'T'HH:mm:ss.SSS'Z'";
     private static final String ISO8601_DATE_FORMAT = "yyyy-MM-dd";
+    private static final String REGEX_ALPHA_NUMERIC_PATTERN = "[a-zA-Z0-9]+";
 
     private static final ThreadLocal<DateFormat[]> DSL_DATE_FORMAT = ThreadLocal.withInitial(() -> {
         final String formats[] = {ISO8601_FORMAT, ISO8601_DATE_FORMAT};
@@ -117,9 +121,9 @@ public class GremlinQueryComposer {
             if (ia.isTrait()) {
                 String traitName = ia.get();
 
-                if (traitName.equalsIgnoreCase(ALL_CLASSIFICATIONS)) {
+                if (traitName.equals(ALL_CLASSIFICATIONS)) {
                     addTrait(GremlinClause.ANY_TRAIT, ia);
-                } else if (traitName.equalsIgnoreCase(NO_CLASSIFICATIONS)) {
+                } else if (traitName.equals(NO_CLASSIFICATIONS)) {
                     addTrait(GremlinClause.NO_TRAIT, ia);
                 } else {
                     addTrait(GremlinClause.TRAIT, ia);
@@ -171,12 +175,14 @@ public class GremlinQueryComposer {
         }
 
         String                currentType = context.getActiveTypeName();
+
         IdentifierHelper.Info org         = null;
         IdentifierHelper.Info lhsI        = createInfo(lhs);
         if (!lhsI.isPrimitive()) {
             introduceType(lhsI);
             org = lhsI;
             lhsI = createInfo(lhs);
+            lhsI.setTypeName(org.getTypeName());
         }
 
         if (!context.validator.isValidQualifiedName(lhsI.getQualifiedName(), lhsI.getRaw())) {
@@ -192,22 +198,38 @@ public class GremlinQueryComposer {
         rhs = addQuotesIfNecessary(lhsI, rhs);
         SearchParameters.Operator op = SearchParameters.Operator.fromString(operator);
         if (op == SearchParameters.Operator.LIKE) {
-            add(GremlinClause.TEXT_CONTAINS, lhsI.getQualifiedName(), IdentifierHelper.getFixedRegEx(rhs));
+            final AtlasStructType.AtlasAttribute attribute = context.getActiveEntityType().getAttribute(lhsI.getAttributeName());
+            final AtlasStructDef.AtlasAttributeDef.IndexType indexType = attribute.getAttributeDef().getIndexType();
+
+            if (indexType == AtlasStructDef.AtlasAttributeDef.IndexType.STRING || !containsNumberAndLettersOnly(rhs)) {
+                add(GremlinClause.STRING_CONTAINS, getPropertyForClause(lhsI), IdentifierHelper.getFixedRegEx(rhs));
+            } else {
+                add(GremlinClause.TEXT_CONTAINS, getPropertyForClause(lhsI), IdentifierHelper.getFixedRegEx(rhs));
+            }
         } else if (op == SearchParameters.Operator.IN) {
-            add(GremlinClause.HAS_OPERATOR, lhsI.getQualifiedName(), "within", rhs);
+            add(GremlinClause.HAS_OPERATOR, getPropertyForClause(lhsI), "within", rhs);
         } else {
-            add(GremlinClause.HAS_OPERATOR, lhsI.getQualifiedName(), op.getSymbols()[1], rhs);
+            add(GremlinClause.HAS_OPERATOR, getPropertyForClause(lhsI), op.getSymbols()[1], rhs);
         }
         // record that the attribute has been processed so that the select clause doesn't add a attr presence check
         attributesProcessed.add(lhsI.getQualifiedName());
 
         if (org != null && org.isReferredType()) {
             add(GremlinClause.DEDUP);
-            add(GremlinClause.IN, org.getEdgeLabel());
+            if (org.getEdgeDirection() != null) {
+                GremlinClause gremlinClauseForEdgeLabel = org.getEdgeDirection().equals(IN) ? GremlinClause.OUT : GremlinClause.IN;
+                add(gremlinClauseForEdgeLabel, org.getEdgeLabel());
+            } else {
+                add(GremlinClause.OUT, org.getEdgeLabel());
+            }
             context.registerActive(currentType);
         }
     }
 
+    private boolean containsNumberAndLettersOnly(String rhs) {
+        return Pattern.matches(REGEX_ALPHA_NUMERIC_PATTERN, IdentifierHelper.removeWildcards(rhs));
+    }
+
     private String parseNumber(String rhs, Context context) {
         return rhs.replace("'", "").replace("\"", "") + context.getNumericTypeFormatter();
     }
@@ -320,12 +342,12 @@ public class GremlinQueryComposer {
 
         IdentifierHelper.Info ia = createInfo(name);
         if (queryMetadata.hasSelect() && queryMetadata.hasGroupBy()) {
-            addSelectTransformation(this.context.selectClauseComposer, getQualifiedName(ia), isDesc);
+            addSelectTransformation(this.context.selectClauseComposer, getPropertyForClause(ia), isDesc);
         } else if (queryMetadata.hasGroupBy()) {
-            addOrderByClause(getQualifiedName(ia), isDesc);
+            addOrderByClause(ia, isDesc);
             moveToLast(GremlinClause.GROUP_BY);
         } else {
-            addOrderByClause(getQualifiedName(ia), isDesc);
+            addOrderByClause(ia, isDesc);
         }
     }
 
@@ -347,9 +369,17 @@ public class GremlinQueryComposer {
                        queryClauses.contains(GremlinClause.HAS_TYPE_WITHIN) != -1;
     }
 
-    private String getQualifiedName(IdentifierHelper.Info ia) {
-        return context.validator.isValidQualifiedName(ia.getQualifiedName(), ia.getRaw()) ?
-                       ia.getQualifiedName() : ia.getRaw();
+    private String getPropertyForClause(IdentifierHelper.Info ia) {
+        String vertexPropertyName = lookup.getVertexPropertyName(ia.getTypeName(), ia.getAttributeName());
+        if (StringUtils.isNotEmpty(vertexPropertyName)) {
+            return vertexPropertyName;
+        }
+
+        if (StringUtils.isNotEmpty(ia.getQualifiedName())) {
+            return ia.getQualifiedName();
+        }
+
+        return ia.getRaw();
     }
 
     private void addSelectAttrExistsCheck(final SelectClauseComposer selectClauseComposer) {
@@ -362,7 +392,7 @@ public class GremlinQueryComposer {
                 IdentifierHelper.Info idMetadata         = createInfo(qualifiedAttribute);
                 // Only primitive attributes need to be checked
                 if (idMetadata.isPrimitive() && !selectClauseComposer.isAggregatorIdx(i) && !attributesProcessed.contains(qualifiedAttribute)) {
-                    add(GremlinClause.HAS_PROPERTY, qualifiedAttribute);
+                    add(GremlinClause.HAS_PROPERTY, getPropertyForClause(idMetadata));
                 }
             }
             // All these checks should be done before the grouping happens (if any)
@@ -388,10 +418,10 @@ public class GremlinQueryComposer {
             }
 
             if (!scc.getItem(i).equals(scc.getLabel(i))) {
-                context.addAlias(scc.getLabel(i), getQualifiedName(ia));
+                context.addAlias(scc.getLabel(i), ia.getQualifiedName());
             }
 
-            if (scc.updateAsApplicable(i, getQualifiedName(ia))) {
+            if (scc.updateAsApplicable(i, getPropertyForClause(ia), ia.getQualifiedName())) {
                 continue;
             }
 
@@ -404,10 +434,10 @@ public class GremlinQueryComposer {
                 scc.incrementTypesIntroduced();
                 scc.isSelectNoop = !ia.hasParts();
                 if (ia.hasParts()) {
-                    scc.assign(i, getQualifiedName(createInfo(ia.get())), GremlinClause.INLINE_GET_PROPERTY);
+                    scc.assign(i, getPropertyForClause(createInfo(ia.get())), GremlinClause.INLINE_GET_PROPERTY);
                 }
             } else {
-                scc.assign(i, getQualifiedName(ia), GremlinClause.INLINE_GET_PROPERTY);
+                scc.assign(i, getPropertyForClause(ia), GremlinClause.INLINE_GET_PROPERTY);
                 scc.incrementPrimitiveType();
             }
         }
@@ -456,16 +486,16 @@ public class GremlinQueryComposer {
     private void addSelectTransformation(final SelectClauseComposer selectClauseComposer,
                                          final String orderByQualifiedAttrName,
                                          final boolean isDesc) {
-        GremlinClause fn;
+        GremlinClause gremlinClause;
         if (selectClauseComposer.isSelectNoop) {
-            fn = GremlinClause.SELECT_NOOP_FN;
+            gremlinClause = GremlinClause.SELECT_NOOP_FN;
         } else if (queryMetadata.hasGroupBy()) {
-            fn = selectClauseComposer.onlyAggregators() ? GremlinClause.SELECT_ONLY_AGG_GRP_FN : GremlinClause.SELECT_MULTI_ATTR_GRP_FN;
+            gremlinClause = selectClauseComposer.onlyAggregators() ? GremlinClause.SELECT_ONLY_AGG_GRP_FN : GremlinClause.SELECT_MULTI_ATTR_GRP_FN;
         } else {
-            fn = selectClauseComposer.onlyAggregators() ? GremlinClause.SELECT_ONLY_AGG_FN : GremlinClause.SELECT_FN;
+            gremlinClause = selectClauseComposer.onlyAggregators() ? GremlinClause.SELECT_ONLY_AGG_FN : GremlinClause.SELECT_FN;
         }
         if (StringUtils.isEmpty(orderByQualifiedAttrName)) {
-            add(0, fn,
+            add(0, gremlinClause,
                 selectClauseComposer.getLabelHeader(),
                 selectClauseComposer.hasAssignmentExpr() ? selectClauseComposer.getAssignmentExprString() : EMPTY_STRING,
                 selectClauseComposer.getItemsString(),
@@ -477,7 +507,7 @@ public class GremlinQueryComposer {
                 sortClause = isDesc ? GremlinClause.INLINE_TUPLE_SORT_DESC : GremlinClause.INLINE_TUPLE_SORT_ASC;
             }
             String idxStr = String.valueOf(itemIdx);
-            add(0, fn,
+            add(0, gremlinClause,
                 selectClauseComposer.getLabelHeader(),
                 selectClauseComposer.hasAssignmentExpr() ? selectClauseComposer.getAssignmentExprString() : EMPTY_STRING,
                 selectClauseComposer.getItemsString(),
@@ -552,7 +582,12 @@ public class GremlinQueryComposer {
 
     private boolean introduceType(IdentifierHelper.Info ia) {
         if (ia.isReferredType()) {
-            add(GremlinClause.OUT, ia.getEdgeLabel());
+            if (ia.getEdgeDirection() != null) {
+                GremlinClause gremlinClauseForEdgeLabel = ia.getEdgeDirection().equals(OUT) ? GremlinClause.OUT : GremlinClause.IN;
+                add(gremlinClauseForEdgeLabel, ia.getEdgeLabel());
+            } else {
+                add(GremlinClause.OUT, ia.getEdgeLabel());
+            }
             context.registerActive(ia);
         }
 
@@ -575,12 +610,11 @@ public class GremlinQueryComposer {
         }
     }
 
-    private void addOrderByClause(String name, boolean descr) {
+    private void addOrderByClause(IdentifierHelper.Info ia, boolean descr) {
         if (LOG.isDebugEnabled()) {
-            LOG.debug("addOrderByClause(name={})", name, descr);
+            LOG.debug("addOrderByClause(name={})", ia.getRaw(), descr);
         }
 
-        IdentifierHelper.Info ia = createInfo(name);
         add((!descr) ? GremlinClause.ORDER_BY : GremlinClause.ORDER_BY_DESC, ia);
     }
 
@@ -598,7 +632,7 @@ public class GremlinQueryComposer {
             return;
         }
 
-        add(clause, (idInfo.getQualifiedName() == null ? idInfo.get() : idInfo.getQualifiedName()));
+        add(clause, getPropertyForClause(idInfo));
     }
 
     private void add(GremlinClause clause, String... args) {
