@@ -24,6 +24,7 @@ import org.apache.atlas.model.instance.AtlasClassification;
 import org.apache.atlas.model.instance.AtlasEntity;
 import org.apache.atlas.model.typedef.AtlasStructDef;
 import org.apache.atlas.type.AtlasArrayType;
+import org.apache.atlas.type.AtlasBusinessMetadataType;
 import org.apache.atlas.type.AtlasClassificationType;
 import org.apache.atlas.type.AtlasEntityType;
 import org.apache.atlas.type.AtlasEnumType;
@@ -42,11 +43,9 @@ class ExportTypeProcessor {
     private static final Logger LOG = LoggerFactory.getLogger(ExportTypeProcessor.class);
 
     private AtlasTypeRegistry typeRegistry;
-    private final ExportService.ExportContext context;
 
-    ExportTypeProcessor(AtlasTypeRegistry typeRegistry, ExportService.ExportContext context) {
+    ExportTypeProcessor(AtlasTypeRegistry typeRegistry) {
         this.typeRegistry = typeRegistry;
-        this.context = context;
     }
 
     public void addTypes(AtlasEntity entity, ExportService.ExportContext context) {
@@ -105,12 +104,14 @@ class ExportTypeProcessor {
             addEntityType((AtlasEntityType)type, context);
         } else if (type instanceof AtlasClassificationType) {
             addClassificationType((AtlasClassificationType)type, context);
+        } else if (type instanceof AtlasRelationshipType) {
+            addRelationshipType(type.getTypeName(), context);
+        } else if (type instanceof AtlasBusinessMetadataType) {
+            addBusinessMetadataType((AtlasBusinessMetadataType) type, context);
         } else if (type instanceof AtlasStructType) {
             addStructType((AtlasStructType)type, context);
         } else if (type instanceof AtlasEnumType) {
             addEnumType((AtlasEnumType)type, context);
-        } else if (type instanceof AtlasRelationshipType) {
-            addRelationshipType(type.getTypeName(), context);
         }
     }
 
@@ -171,6 +172,14 @@ class ExportTypeProcessor {
         }
     }
 
+    private void addBusinessMetadataType(AtlasBusinessMetadataType businessMetadataType, ExportService.ExportContext context) {
+        if (!context.businessMetadataTypes.contains(businessMetadataType.getTypeName())) {
+            context.businessMetadataTypes.add(businessMetadataType.getTypeName());
+
+            addAttributeTypes(businessMetadataType, context);
+        }
+    }
+
     private void addAttributeTypes(AtlasStructType structType, ExportService.ExportContext context) {
         for (AtlasStructDef.AtlasAttributeDef attributeDef : structType.getStructDef().getAttributeDefs()) {
             addType(attributeDef.getTypeName(), context);
