@@ -22,6 +22,7 @@ import org.apache.atlas.model.impexp.AtlasImportResult;
 import org.apache.atlas.model.typedef.AtlasClassificationDef;
 import org.apache.atlas.model.typedef.AtlasEntityDef;
 import org.apache.atlas.model.typedef.AtlasEnumDef;
+import org.apache.atlas.model.typedef.AtlasRelationshipDef;
 import org.apache.atlas.model.typedef.AtlasStructDef;
 import org.apache.atlas.model.typedef.AtlasTypesDef;
 import org.apache.atlas.repository.store.bootstrap.AtlasTypeDefStoreInitializer;
@@ -42,14 +43,13 @@ public class ImportTypeDefProcessor {
     public void processTypes(AtlasTypesDef typeDefinitionMap, AtlasImportResult result) throws AtlasBaseException {
         setGuidToEmpty(typeDefinitionMap);
 
-        typeAttributeDifference.updateTypes(typeDefinitionMap, result);
-
         AtlasTypesDef typesToCreate = AtlasTypeDefStoreInitializer.getTypesToCreate(typeDefinitionMap, this.typeRegistry);
         if (!typesToCreate.isEmpty()) {
             typeDefStore.createTypesDef(typesToCreate);
             updateMetricsForTypesDef(typesToCreate, result);
-
         }
+
+        typeAttributeDifference.updateTypes(typeDefinitionMap, result);
     }
 
     private void setGuidToEmpty(AtlasTypesDef typesDef) {
@@ -68,6 +68,10 @@ public class ImportTypeDefProcessor {
         for (AtlasStructDef def : typesDef.getStructDefs()) {
             def.setGuid(null);
         }
+
+        for (AtlasRelationshipDef def : typesDef.getRelationshipDefs()) {
+            def.setGuid(null);
+        }
     }
 
     private void updateMetricsForTypesDef(AtlasTypesDef typeDefinitionMap, AtlasImportResult result) {
