@@ -19,9 +19,11 @@
 package org.apache.atlas.repository.store.graph.v1;
 
 import org.apache.atlas.RequestContext;
+import org.apache.atlas.exception.AtlasBaseException;
 import org.apache.atlas.model.instance.AtlasEntity.Status;
 import org.apache.atlas.repository.graph.GraphHelper;
 import org.apache.atlas.repository.graphdb.AtlasEdge;
+import org.apache.atlas.repository.graphdb.AtlasGraph;
 import org.apache.atlas.repository.graphdb.AtlasVertex;
 import org.apache.atlas.repository.store.graph.v2.AtlasGraphUtilsV2;
 import org.apache.atlas.type.AtlasTypeRegistry;
@@ -36,8 +38,8 @@ import static org.apache.atlas.repository.Constants.STATE_PROPERTY_KEY;
 public class SoftDeleteHandlerV1 extends DeleteHandlerV1 {
 
     @Inject
-    public SoftDeleteHandlerV1(AtlasTypeRegistry typeRegistry) {
-        super(typeRegistry, false, true);
+    public SoftDeleteHandlerV1(AtlasGraph graph, AtlasTypeRegistry typeRegistry) {
+        super(graph, typeRegistry, false, true);
     }
 
     @Override
@@ -60,12 +62,14 @@ public class SoftDeleteHandlerV1 extends DeleteHandlerV1 {
     }
 
     @Override
-    protected void deleteEdge(AtlasEdge edge, boolean force) {
+    protected void deleteEdge(AtlasEdge edge, boolean force) throws AtlasBaseException {
         if (LOG.isDebugEnabled()) {
             LOG.debug("==> SoftDeleteHandlerV1.deleteEdge({}, {})",GraphHelper.string(edge), force);
         }
 
         if (force) {
+            removeTagPropagation(edge);
+
             graphHelper.removeEdge(edge);
         } else {
             Status state = AtlasGraphUtilsV2.getState(edge);
