@@ -66,7 +66,7 @@ import static org.apache.atlas.repository.Constants.ENTITY_TEXT_PROPERTY_KEY;
 
 
 @Component
-public class AtlasEntityChangeNotifier {
+public class AtlasEntityChangeNotifier implements IAtlasEntityChangeNotifier {
     private static final Logger LOG = LoggerFactory.getLogger(AtlasEntityChangeNotifier.class);
 
     private final Set<EntityChangeListener>   entityChangeListeners;
@@ -91,8 +91,9 @@ public class AtlasEntityChangeNotifier {
         this.isV2EntityNotificationEnabled = AtlasRepositoryConfiguration.isV2EntityNotificationEnabled();
     }
 
+    @Override
     public void onEntitiesMutated(EntityMutationResponse entityMutationResponse, boolean isImport) throws AtlasBaseException {
-        if (CollectionUtils.isEmpty(entityChangeListeners) || instanceConverter == null) {
+        if (CollectionUtils.isEmpty(entityChangeListeners)) {
             return;
         }
 
@@ -102,6 +103,7 @@ public class AtlasEntityChangeNotifier {
         List<AtlasEntityHeader> updatedEntities          = entityMutationResponse.getUpdatedEntities();
         List<AtlasEntityHeader> partiallyUpdatedEntities = entityMutationResponse.getPartialUpdatedEntities();
         List<AtlasEntityHeader> deletedEntities          = entityMutationResponse.getDeletedEntities();
+        List<AtlasEntityHeader> purgedEntities           = entityMutationResponse.getPurgedEntities();
 
         // complete full text mapping before calling toReferenceables(), from notifyListners(), to
         // include all vertex updates in the current graph-transaction
@@ -113,12 +115,14 @@ public class AtlasEntityChangeNotifier {
         notifyListeners(updatedEntities, EntityOperation.UPDATE, isImport);
         notifyListeners(partiallyUpdatedEntities, EntityOperation.PARTIAL_UPDATE, isImport);
         notifyListeners(deletedEntities, EntityOperation.DELETE, isImport);
+        notifyListeners(purgedEntities, EntityOperation.PURGE, isImport);
 
         notifyPropagatedEntities();
     }
 
+    @Override
     public void notifyRelationshipMutation(AtlasRelationship relationship, EntityNotification.EntityNotificationV2.OperationType operationType) throws AtlasBaseException {
-        if (CollectionUtils.isEmpty(entityChangeListeners) || instanceConverter == null) {
+        if (CollectionUtils.isEmpty(entityChangeListeners)) {
             return;
         }
 
@@ -135,6 +139,7 @@ public class AtlasEntityChangeNotifier {
         }
     }
 
+    @Override
     public void onClassificationAddedToEntity(AtlasEntity entity, List<AtlasClassification> addedClassifications) throws AtlasBaseException {
         if (isV2EntityNotificationEnabled) {
             doFullTextMapping(entity.getGuid());
@@ -145,85 +150,159 @@ public class AtlasEntityChangeNotifier {
         } else {
             updateFullTextMapping(entity.getGuid(), addedClassifications);
 
-            Referenceable entityRef = toReferenceable(entity.getGuid());
-            List<Struct>  traits    = toStruct(addedClassifications);
+            if (instanceConverter != null) {
+                Referenceable entityRef = toReferenceable(entity.getGuid());
+                List<Struct>  traits    = toStruct(addedClassifications);
 
-            if (entity == null || CollectionUtils.isEmpty(traits)) {
-                return;
-            }
+                if (entityRef == null || CollectionUtils.isEmpty(traits)) {
+                    return;
+                }
 
-            for (EntityChangeListener listener : entityChangeListeners) {
-                try {
-                    listener.onTraitsAdded(entityRef, traits);
-                } catch (AtlasException e) {
-                    throw new AtlasBaseException(AtlasErrorCode.NOTIFICATION_FAILED, e, getListenerName(listener), "TraitAdd");
+                for (EntityChangeListener listener : entityChangeListeners) {
+                    try {
+                        listener.onTraitsAdded(entityRef, traits);
+                    } catch (AtlasException e) {
+                        throw new AtlasBaseException(AtlasErrorCode.NOTIFICATION_FAILED, e, getListenerName(listener), "TraitAdd");
+                    }
                 }
             }
         }
     }
 
-    public void onClassificationUpdatedToEntity(AtlasEntity entity, List<AtlasClassification> updatedClassifications) throws AtlasBaseException {
+    @Override
+    public void onClassificationsAddedToEntities(List<AtlasEntity> entities, List<AtlasClassification> addedClassifications) throws AtlasBaseException {
         if (isV2EntityNotificationEnabled) {
-            doFullTextMapping(entity.getGuid());
+            doFullTextMappingHelper(entities);
 
+            for (EntityChangeListenerV2 listener : entityChangeListenersV2) {
+                listener.onClassificationsAdded(entities, addedClassifications);
+            }
+        } else {
+            updateFullTextMapping(entities, addedClassifications);
+
+            if (instanceConverter != null) {
+                List<Struct> traits = toStruct(addedClassifications);
+
+                if (!CollectionUtils.isEmpty(traits)) {
+                    for(AtlasEntity entity : entities) {
+                        Referenceable entityRef = toReferenceable(entity.getGuid());
+
+                        if (entityRef == null) {
+                            LOG.warn("EntityRef with guid {} not found while adding classifications {} ", entity.getGuid(), addedClassifications);
+                            continue;
+                        }
+
+                        for (EntityChangeListener listener : entityChangeListeners) {
+                            try {
+                                listener.onTraitsAdded(entityRef, traits);
+                            } catch (AtlasException e) {
+                                throw new AtlasBaseException(AtlasErrorCode.NOTIFICATION_FAILED, e, getListenerName(listener), "TraitAdd");
+                            }
+                        }
+                    }
+                }
+            }
+        }
+    }
+
+    @Override
+    public void onClassificationUpdatedToEntity(AtlasEntity entity, List<AtlasClassification> updatedClassifications) throws AtlasBaseException {
+        doFullTextMapping(entity.getGuid());
+
+        if (isV2EntityNotificationEnabled) {
             for (EntityChangeListenerV2 listener : entityChangeListenersV2) {
                 listener.onClassificationsUpdated(entity, updatedClassifications);
             }
         } else {
-            doFullTextMapping(entity.getGuid());
+            if (instanceConverter != null) {
+                Referenceable entityRef = toReferenceable(entity.getGuid());
+                List<Struct>  traits    = toStruct(updatedClassifications);
 
-            Referenceable entityRef = toReferenceable(entity.getGuid());
-            List<Struct>  traits    = toStruct(updatedClassifications);
+                if (entityRef == null || CollectionUtils.isEmpty(traits)) {
+                    return;
+                }
 
-            if (entityRef == null || CollectionUtils.isEmpty(traits)) {
-                return;
-            }
-
-            for (EntityChangeListener listener : entityChangeListeners) {
-                try {
-                    listener.onTraitsUpdated(entityRef, traits);
-                } catch (AtlasException e) {
-                    throw new AtlasBaseException(AtlasErrorCode.NOTIFICATION_FAILED, e, getListenerName(listener), "TraitUpdate");
+                for (EntityChangeListener listener : entityChangeListeners) {
+                    try {
+                        listener.onTraitsUpdated(entityRef, traits);
+                    } catch (AtlasException e) {
+                        throw new AtlasBaseException(AtlasErrorCode.NOTIFICATION_FAILED, e, getListenerName(listener), "TraitUpdate");
+                    }
                 }
             }
         }
     }
 
+    @Override
     public void onClassificationDeletedFromEntity(AtlasEntity entity, List<AtlasClassification> deletedClassifications) throws AtlasBaseException {
-        if (isV2EntityNotificationEnabled) {
-            doFullTextMapping(entity.getGuid());
+        doFullTextMapping(entity.getGuid());
 
+        if (isV2EntityNotificationEnabled) {
             for (EntityChangeListenerV2 listener : entityChangeListenersV2) {
                 listener.onClassificationsDeleted(entity, deletedClassifications);
             }
         } else {
-            doFullTextMapping(entity.getGuid());
+            if (instanceConverter != null) {
+                Referenceable entityRef = toReferenceable(entity.getGuid());
+                List<Struct>  traits    = toStruct(deletedClassifications);
 
-            Referenceable entityRef = toReferenceable(entity.getGuid());
-            List<Struct>  traits    = toStruct(deletedClassifications);
+                if (entityRef == null || CollectionUtils.isEmpty(deletedClassifications)) {
+                    return;
+                }
 
-            if (entityRef == null || CollectionUtils.isEmpty(deletedClassifications)) {
-                return;
-            }
-
-            for (EntityChangeListener listener : entityChangeListeners) {
-                try {
-                    listener.onTraitsDeleted(entityRef, traits);
-                } catch (AtlasException e) {
-                    throw new AtlasBaseException(AtlasErrorCode.NOTIFICATION_FAILED, e, getListenerName(listener), "TraitDelete");
+                for (EntityChangeListener listener : entityChangeListeners) {
+                    try {
+                        listener.onTraitsDeleted(entityRef, traits);
+                    } catch (AtlasException e) {
+                        throw new AtlasBaseException(AtlasErrorCode.NOTIFICATION_FAILED, e, getListenerName(listener), "TraitDelete");
+                    }
                 }
             }
-
         }
     }
 
+    @Override
+    public void onClassificationsDeletedFromEntities(List<AtlasEntity> entities, List<AtlasClassification> deletedClassifications) throws AtlasBaseException {
+        doFullTextMappingHelper(entities);
+
+        if (isV2EntityNotificationEnabled) {
+            for (EntityChangeListenerV2 listener : entityChangeListenersV2) {
+                listener.onClassificationsDeleted(entities, deletedClassifications);
+            }
+        } else {
+            if (instanceConverter != null) {
+                List<Struct> traits = toStruct(deletedClassifications);
+
+                if(!CollectionUtils.isEmpty(deletedClassifications)) {
+                    for(AtlasEntity entity : entities) {
+                        Referenceable entityRef = toReferenceable(entity.getGuid());
+
+                        if (entityRef == null) {
+                            LOG.warn("EntityRef with guid {} not found while deleting classifications {} ", entity.getGuid(), deletedClassifications);
+                            continue;
+                        }
+
+                        for (EntityChangeListener listener : entityChangeListeners) {
+                            try {
+                                listener.onTraitsDeleted(entityRef, traits);
+                            } catch (AtlasException e) {
+                                throw new AtlasBaseException(AtlasErrorCode.NOTIFICATION_FAILED, e, getListenerName(listener), "TraitDelete");
+                            }
+                        }
+                    }
+                }
+            }
+        }
+    }
+
+    @Override
     public void onTermAddedToEntities(AtlasGlossaryTerm term, List<AtlasRelatedObjectId> entityIds) throws AtlasBaseException {
         // listeners notified on term-entity association only if v2 notifications are enabled
         if (isV2EntityNotificationEnabled) {
             for (EntityChangeListenerV2 listener : entityChangeListenersV2) {
                 listener.onTermAdded(term, entityIds);
             }
-        } else {
+        } else if (instanceConverter != null) {
             List<Referenceable> entityRefs = toReferenceables(entityIds);
 
             for (EntityChangeListener listener : entityChangeListeners) {
@@ -236,13 +315,14 @@ public class AtlasEntityChangeNotifier {
         }
     }
 
+    @Override
     public void onTermDeletedFromEntities(AtlasGlossaryTerm term, List<AtlasRelatedObjectId> entityIds) throws AtlasBaseException {
         // listeners notified on term-entity disassociation only if v2 notifications are enabled
         if (isV2EntityNotificationEnabled) {
             for (EntityChangeListenerV2 listener : entityChangeListenersV2) {
                 listener.onTermDeleted(term, entityIds);
             }
-        } else {
+        } else if (instanceConverter != null) {
             List<Referenceable> entityRefs = toReferenceables(entityIds);
 
             for (EntityChangeListener listener : entityChangeListeners) {
@@ -255,6 +335,21 @@ public class AtlasEntityChangeNotifier {
         }
     }
 
+    @Override
+    public void onLabelsUpdatedFromEntity(String entityGuid, Set<String> addedLabels, Set<String> deletedLabels) throws AtlasBaseException {
+        doFullTextMapping(entityGuid);
+
+        if (isV2EntityNotificationEnabled) {
+            AtlasEntity entity = instanceConverter.getAndCacheEntity(entityGuid);
+
+            for (EntityChangeListenerV2 listener : entityChangeListenersV2) {
+                listener.onLabelsDeleted(entity, deletedLabels);
+                listener.onLabelsAdded(entity, addedLabels);
+            }
+        }
+    }
+
+    @Override
     public void notifyPropagatedEntities() throws AtlasBaseException {
         RequestContext                         context             = RequestContext.get();
         Map<String, List<AtlasClassification>> addedPropagations   = context.getAddedPropagations();
@@ -264,6 +359,18 @@ public class AtlasEntityChangeNotifier {
         notifyPropagatedEntities(removedPropagations, PROPAGATED_CLASSIFICATION_DELETE);
     }
 
+    @Override
+    public void onBusinessAttributesUpdated(String entityGuid, Map<String, Map<String, Object>> updatedBusinessAttributes) throws AtlasBaseException{
+        if (isV2EntityNotificationEnabled) {
+            AtlasEntity entity = instanceConverter.getAndCacheEntity(entityGuid);
+
+            for (EntityChangeListenerV2 listener : entityChangeListenersV2) {
+                listener.onBusinessAttributesUpdated(entity, updatedBusinessAttributes);
+            }
+        }
+    }
+
+
     private void notifyPropagatedEntities(Map<String, List<AtlasClassification>> entityPropagationMap, EntityAuditActionV2 action) throws AtlasBaseException {
         if (MapUtils.isEmpty(entityPropagationMap) || action == null) {
             return;
@@ -277,7 +384,7 @@ public class AtlasEntityChangeNotifier {
                 continue;
             }
 
-            AtlasEntity entity = instanceConverter.getAndCacheEntity(guid);
+            AtlasEntity entity = fullTextMapperV2.getAndCacheEntity(guid);
 
             if (entity == null) {
                 continue;
@@ -300,11 +407,15 @@ public class AtlasEntityChangeNotifier {
             return;
         }
 
+        MetricRecorder metric = RequestContext.get().startMetricRecord("notifyListeners");
+
         if (isV2EntityNotificationEnabled) {
             notifyV2Listeners(entityHeaders, operation, isImport);
         } else {
             notifyV1Listeners(entityHeaders, operation, isImport);
         }
+
+        RequestContext.get().endMetricRecord(metric);
     }
 
     private void notifyRelationshipListeners(List<AtlasRelationship> relationships, EntityOperation operation, boolean isImport) throws AtlasBaseException {
@@ -322,24 +433,26 @@ public class AtlasEntityChangeNotifier {
 
 
     private void notifyV1Listeners(List<AtlasEntityHeader> entityHeaders, EntityOperation operation, boolean isImport) throws AtlasBaseException {
-        List<Referenceable> typedRefInsts = toReferenceables(entityHeaders, operation);
+        if (operation != EntityOperation.PURGE && instanceConverter != null) {
+            List<Referenceable> typedRefInsts = toReferenceables(entityHeaders, operation);
 
-        for (EntityChangeListener listener : entityChangeListeners) {
-            try {
-                switch (operation) {
-                    case CREATE:
-                        listener.onEntitiesAdded(typedRefInsts, isImport);
-                        break;
-                    case UPDATE:
-                    case PARTIAL_UPDATE:
-                        listener.onEntitiesUpdated(typedRefInsts, isImport);
-                        break;
-                    case DELETE:
-                        listener.onEntitiesDeleted(typedRefInsts, isImport);
-                        break;
+            for (EntityChangeListener listener : entityChangeListeners) {
+                try {
+                    switch (operation) {
+                        case CREATE:
+                            listener.onEntitiesAdded(typedRefInsts, isImport);
+                            break;
+                        case UPDATE:
+                        case PARTIAL_UPDATE:
+                            listener.onEntitiesUpdated(typedRefInsts, isImport);
+                            break;
+                        case DELETE:
+                            listener.onEntitiesDeleted(typedRefInsts, isImport);
+                            break;
+                    }
+                } catch (AtlasException e) {
+                    throw new AtlasBaseException(AtlasErrorCode.NOTIFICATION_FAILED, e, getListenerName(listener), operation.toString());
                 }
-            } catch (AtlasException e) {
-                throw new AtlasBaseException(AtlasErrorCode.NOTIFICATION_FAILED, e, getListenerName(listener), operation.toString());
             }
         }
     }
@@ -361,6 +474,10 @@ public class AtlasEntityChangeNotifier {
                 case DELETE:
                     listener.onEntitiesDeleted(entities, isImport);
                     break;
+
+                case PURGE:
+                    listener.onEntitiesPurged(entities);
+                    break;
             }
         }
     }
@@ -379,21 +496,26 @@ public class AtlasEntityChangeNotifier {
                 case DELETE:
                     listener.onRelationshipsDeleted(relationships, isImport);
                     break;
+                case PURGE:
+                    listener.onRelationshipsPurged(relationships);
+                    break;
             }
         }
     }
 
-        private List<Referenceable> toReferenceables(List<AtlasEntityHeader> entityHeaders, EntityOperation operation) throws AtlasBaseException {
+    private List<Referenceable> toReferenceables(List<AtlasEntityHeader> entityHeaders, EntityOperation operation) throws AtlasBaseException {
         List<Referenceable> ret = new ArrayList<>(entityHeaders.size());
 
-        // delete notifications don't need all attributes. Hence the special handling for delete operation
-        if (operation == EntityOperation.DELETE) {
-            for (AtlasEntityHeader entityHeader : entityHeaders) {
-                ret.add(new Referenceable(entityHeader.getGuid(), entityHeader.getTypeName(), entityHeader.getAttributes()));
-            }
-        } else {
-            for (AtlasEntityHeader entityHeader : entityHeaders) {
-                ret.add(toReferenceable(entityHeader.getGuid()));
+        if (instanceConverter != null) {
+            // delete notifications don't need all attributes. Hence the special handling for delete operation
+            if (operation == EntityOperation.DELETE) {
+                for (AtlasEntityHeader entityHeader : entityHeaders) {
+                    ret.add(new Referenceable(entityHeader.getGuid(), entityHeader.getTypeName(), entityHeader.getAttributes()));
+                }
+            } else {
+                for (AtlasEntityHeader entityHeader : entityHeaders) {
+                    ret.add(toReferenceable(entityHeader.getGuid()));
+                }
             }
         }
 
@@ -403,7 +525,7 @@ public class AtlasEntityChangeNotifier {
     private List<Referenceable> toReferenceables(List<AtlasRelatedObjectId> entityIds) throws AtlasBaseException {
         List<Referenceable> ret = new ArrayList<>();
 
-        if (CollectionUtils.isNotEmpty(entityIds)) {
+        if (instanceConverter != null && CollectionUtils.isNotEmpty(entityIds)) {
             for (AtlasRelatedObjectId relatedObjectId : entityIds) {
                 String entityGuid = relatedObjectId.getGuid();
 
@@ -417,17 +539,17 @@ public class AtlasEntityChangeNotifier {
     private Referenceable toReferenceable(String entityId) throws AtlasBaseException {
         Referenceable ret = null;
 
-        if (StringUtils.isNotEmpty(entityId)) {
+        if (instanceConverter != null && StringUtils.isNotEmpty(entityId)) {
             ret = instanceConverter.getReferenceable(entityId);
         }
 
         return ret;
     }
 
-        private List<Struct> toStruct(List<AtlasClassification> classifications) throws AtlasBaseException {
+    private List<Struct> toStruct(List<AtlasClassification> classifications) throws AtlasBaseException {
         List<Struct> ret = null;
 
-        if (classifications != null) {
+        if (instanceConverter != null && classifications != null) {
             ret = new ArrayList<>(classifications.size());
 
             for (AtlasClassification classification : classifications) {
@@ -463,12 +585,12 @@ public class AtlasEntityChangeNotifier {
                 final AtlasEntity entity;
 
                 // delete notifications don't need all attributes. Hence the special handling for delete operation
-                if (operation == EntityOperation.DELETE) {
+                if (operation == EntityOperation.DELETE || operation == EntityOperation.PURGE) {
                     entity = new AtlasEntity(entityHeader);
                 } else {
                     String entityGuid = entityHeader.getGuid();
 
-                    entity = instanceConverter.getAndCacheEntity(entityGuid);
+                    entity = fullTextMapperV2.getAndCacheEntity(entityGuid);
                 }
 
                 if (entity != null) {
@@ -481,16 +603,12 @@ public class AtlasEntityChangeNotifier {
     }
 
     private void doFullTextMapping(List<AtlasEntityHeader> entityHeaders) {
-        if (CollectionUtils.isEmpty(entityHeaders)) {
+        if(AtlasRepositoryConfiguration.isFreeTextSearchEnabled() || !AtlasRepositoryConfiguration.isFullTextSearchEnabled()) {
             return;
         }
 
-        try {
-            if(!AtlasRepositoryConfiguration.isFullTextSearchEnabled()) {
-                return;
-            }
-        } catch (AtlasException e) {
-            LOG.warn("Unable to determine if FullText is disabled. Proceeding with FullText mapping");
+        if (CollectionUtils.isEmpty(entityHeaders)) {
+            return;
         }
 
         MetricRecorder metric = RequestContext.get().startMetricRecord("fullTextMapping");
@@ -520,12 +638,8 @@ public class AtlasEntityChangeNotifier {
     }
 
     private void updateFullTextMapping(String entityId, List<AtlasClassification> classifications) {
-        try {
-            if(!AtlasRepositoryConfiguration.isFullTextSearchEnabled()) {
-                return;
-            }
-        } catch (AtlasException e) {
-            LOG.warn("Unable to determine if FullText is disabled. Proceeding with FullText mapping");
+        if(AtlasRepositoryConfiguration.isFreeTextSearchEnabled() || !AtlasRepositoryConfiguration.isFullTextSearchEnabled()) {
+            return;
         }
 
         if (StringUtils.isEmpty(entityId) || CollectionUtils.isEmpty(classifications)) {
@@ -552,13 +666,29 @@ public class AtlasEntityChangeNotifier {
         RequestContext.get().endMetricRecord(metric);
     }
 
+    private void updateFullTextMapping(List<AtlasEntity> entities, List<AtlasClassification> classifications) {
+        for (AtlasEntity entity : entities) {
+            updateFullTextMapping(entity.getGuid(), classifications);
+        }
+    }
+
     private void doFullTextMapping(String guid) {
+        if(AtlasRepositoryConfiguration.isFreeTextSearchEnabled() || !AtlasRepositoryConfiguration.isFullTextSearchEnabled()) {
+            return;
+        }
+
         AtlasEntityHeader entityHeader = new AtlasEntityHeader();
         entityHeader.setGuid(guid);
 
         doFullTextMapping(Collections.singletonList(entityHeader));
     }
 
+    private void doFullTextMappingHelper(List<AtlasEntity> entities) {
+        for (AtlasEntity entity : entities) {
+            doFullTextMapping(entity.getGuid());
+        }
+    }
+
     private void pruneResponse(EntityMutationResponse resp) {
         if (LOG.isDebugEnabled()) {
             LOG.debug("==> pruneResponse()");
@@ -568,12 +698,23 @@ public class AtlasEntityChangeNotifier {
         List<AtlasEntityHeader> updatedEntities        = resp.getUpdatedEntities();
         List<AtlasEntityHeader> partialUpdatedEntities = resp.getPartialUpdatedEntities();
         List<AtlasEntityHeader> deletedEntities        = resp.getDeletedEntities();
+        List<AtlasEntityHeader> purgedEntities        = resp.getPurgedEntities();
 
         // remove entities with DELETED status from created & updated lists
         purgeDeletedEntities(createdEntities);
         purgeDeletedEntities(updatedEntities);
         purgeDeletedEntities(partialUpdatedEntities);
 
+        // remove entities purged in this mutation from created & updated lists
+        if (purgedEntities != null) {
+            for (AtlasEntityHeader entity : purgedEntities) {
+                purgeEntity(entity.getGuid(), deletedEntities);
+                purgeEntity(entity.getGuid(), createdEntities);
+                purgeEntity(entity.getGuid(), updatedEntities);
+                purgeEntity(entity.getGuid(), partialUpdatedEntities);
+            }
+        }
+
         // remove entities deleted in this mutation from created & updated lists
         if (deletedEntities != null) {
             for (AtlasEntityHeader entity : deletedEntities) {
@@ -634,5 +775,4 @@ public class AtlasEntityChangeNotifier {
             }
         }
     }
-
 }
