@@ -27,19 +27,25 @@ import org.apache.atlas.discovery.SearchProcessor;
 import org.apache.atlas.exception.AtlasBaseException;
 import org.apache.atlas.model.TypeCategory;
 import org.apache.atlas.model.instance.AtlasEntity;
+import org.apache.atlas.model.instance.AtlasEntity.Status;
 import org.apache.atlas.model.typedef.AtlasBaseTypeDef;
+import org.apache.atlas.model.typedef.AtlasEnumDef;
 import org.apache.atlas.repository.Constants;
-import org.apache.atlas.repository.graph.AtlasGraphProvider;
 import org.apache.atlas.repository.graph.GraphHelper;
 import org.apache.atlas.repository.graphdb.AtlasEdge;
 import org.apache.atlas.repository.graphdb.AtlasElement;
+import org.apache.atlas.repository.graphdb.AtlasGraph;
 import org.apache.atlas.repository.graphdb.AtlasGraphQuery;
 import org.apache.atlas.repository.graphdb.AtlasIndexQuery;
+import org.apache.atlas.repository.graphdb.AtlasIndexQuery.Result;
 import org.apache.atlas.repository.graphdb.AtlasVertex;
 import org.apache.atlas.type.AtlasEntityType;
+import org.apache.atlas.type.AtlasEnumType;
 import org.apache.atlas.type.AtlasStructType;
 import org.apache.atlas.type.AtlasStructType.AtlasAttribute;
 import org.apache.atlas.type.AtlasType;
+import org.apache.atlas.util.FileUtils;
+import org.apache.atlas.utils.AtlasPerfMetrics;
 import org.apache.atlas.utils.AtlasPerfMetrics.MetricRecorder;
 import org.apache.commons.collections.MapUtils;
 import org.apache.commons.configuration.Configuration;
@@ -47,8 +53,9 @@ import org.apache.commons.lang.StringUtils;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
+import java.text.SimpleDateFormat;
 import java.util.ArrayList;
-import java.util.Collection;
+import java.util.Arrays;
 import java.util.Collections;
 import java.util.Date;
 import java.util.Iterator;
@@ -56,9 +63,17 @@ import java.util.List;
 import java.util.Map;
 import java.util.Set;
 
+import static org.apache.atlas.repository.Constants.CLASSIFICATION_NAMES_KEY;
+import static org.apache.atlas.repository.Constants.ENTITY_TYPE_PROPERTY_KEY;
 import static org.apache.atlas.repository.Constants.INDEX_SEARCH_VERTEX_PREFIX_DEFAULT;
 import static org.apache.atlas.repository.Constants.INDEX_SEARCH_VERTEX_PREFIX_PROPERTY;
-import static org.apache.atlas.repository.graphdb.AtlasGraphQuery.SortOrder.*;
+import static org.apache.atlas.repository.Constants.PROPAGATED_CLASSIFICATION_NAMES_KEY;
+import static org.apache.atlas.repository.Constants.STATE_PROPERTY_KEY;
+import static org.apache.atlas.repository.Constants.TYPENAME_PROPERTY_KEY;
+import static org.apache.atlas.repository.Constants.TYPE_NAME_PROPERTY_KEY;
+import static org.apache.atlas.repository.graph.AtlasGraphProvider.getGraphInstance;
+import static org.apache.atlas.repository.graphdb.AtlasGraphQuery.SortOrder.ASC;
+import static org.apache.atlas.repository.graphdb.AtlasGraphQuery.SortOrder.DESC;
 
 /**
  * Utility methods for Graph.
@@ -73,7 +88,7 @@ public class AtlasGraphUtilsV2 {
     public static final String VERTEX_TYPE                 = "typeSystem";
 
     private static boolean USE_INDEX_QUERY_TO_FIND_ENTITY_BY_UNIQUE_ATTRIBUTES = false;
-    private static boolean USE_UNIQUE_INDEX_PROPERTY_TO_FIND_ENTITY            = false;
+    private static boolean USE_UNIQUE_INDEX_PROPERTY_TO_FIND_ENTITY            = true;
     private static String  INDEX_SEARCH_PREFIX;
 
     static {
@@ -115,7 +130,7 @@ public class AtlasGraphUtilsV2 {
     }
 
     public static String getTypeName(AtlasElement element) {
-        return element.getProperty(Constants.ENTITY_TYPE_PROPERTY_KEY, String.class);
+        return element.getProperty(ENTITY_TYPE_PROPERTY_KEY, String.class);
     }
 
     public static String getEdgeLabel(String fromNode, String toNode) {
@@ -141,6 +156,10 @@ public class AtlasGraphUtilsV2 {
         return StringUtils.isNotEmpty(getIdFromVertex(vertex)) && StringUtils.isNotEmpty(getTypeName(vertex));
     }
 
+    public static boolean isTypeVertex(AtlasVertex vertex) {
+        return vertex.getProperty(TYPENAME_PROPERTY_KEY, String.class) != null;
+    }
+
     public static boolean isReference(AtlasType type) {
         return isReference(type.getTypeCategory());
     }
@@ -206,12 +225,11 @@ public class AtlasGraphUtilsV2 {
 
         Object existingValue = element.getProperty(propertyName, Object.class);
 
-        if (value == null || (value instanceof Collection && ((Collection)value).isEmpty())) {
+        if (value == null) {
             if (existingValue != null) {
                 if (LOG.isDebugEnabled()) {
                     LOG.debug("Removing property {} from {}", propertyName, toString(element));
                 }
-
                 element.removeProperty(propertyName);
             }
         } else {
@@ -251,9 +269,12 @@ public class AtlasGraphUtilsV2 {
 
         return returnType.cast(property);
     }
-
     public static AtlasVertex getVertexByUniqueAttributes(AtlasEntityType entityType, Map<String, Object> attrValues) throws AtlasBaseException {
-        AtlasVertex vertex = findByUniqueAttributes(entityType, attrValues);
+        return getVertexByUniqueAttributes(getGraphInstance(), entityType, attrValues);
+    }
+
+    public static AtlasVertex getVertexByUniqueAttributes(AtlasGraph graph, AtlasEntityType entityType, Map<String, Object> attrValues) throws AtlasBaseException {
+        AtlasVertex vertex = findByUniqueAttributes(graph, entityType, attrValues);
 
         if (vertex == null) {
             throw new AtlasBaseException(AtlasErrorCode.INSTANCE_BY_UNIQUE_ATTRIBUTE_NOT_FOUND, entityType.getTypeName(),
@@ -264,11 +285,20 @@ public class AtlasGraphUtilsV2 {
     }
 
     public static String getGuidByUniqueAttributes(AtlasEntityType entityType, Map<String, Object> attrValues) throws AtlasBaseException {
-        AtlasVertex vertexByUniqueAttributes = getVertexByUniqueAttributes(entityType, attrValues);
+        AtlasVertex vertexByUniqueAttributes = getVertexByUniqueAttributes(getGraphInstance(), entityType, attrValues);
+        return getIdFromVertex(vertexByUniqueAttributes);
+    }
+
+    public static String getGuidByUniqueAttributes(AtlasGraph graph, AtlasEntityType entityType, Map<String, Object> attrValues) throws AtlasBaseException {
+        AtlasVertex vertexByUniqueAttributes = getVertexByUniqueAttributes(graph, entityType, attrValues);
         return getIdFromVertex(vertexByUniqueAttributes);
     }
 
     public static AtlasVertex findByUniqueAttributes(AtlasEntityType entityType, Map<String, Object> attrValues) {
+        return findByUniqueAttributes(getGraphInstance(), entityType, attrValues);
+    }
+
+    public static AtlasVertex findByUniqueAttributes(AtlasGraph graph, AtlasEntityType entityType, Map<String, Object> attrValues) {
         MetricRecorder metric = RequestContext.get().startMetricRecord("findByUniqueAttributes");
 
         AtlasVertex vertex = null;
@@ -283,22 +313,22 @@ public class AtlasGraphUtilsV2 {
                     continue;
                 }
 
-                if (canUseIndexQuery(entityType, attribute.getName())) {
-                    vertex = AtlasGraphUtilsV2.getAtlasVertexFromIndexQuery(entityType, attribute, attrValue);
+                if (canUseIndexQuery(graph, entityType, attribute.getName())) {
+                    vertex = AtlasGraphUtilsV2.getAtlasVertexFromIndexQuery(graph, entityType, attribute, attrValue);
                 } else {
                     if (USE_UNIQUE_INDEX_PROPERTY_TO_FIND_ENTITY && attribute.getVertexUniquePropertyName() != null) {
-                        vertex = AtlasGraphUtilsV2.findByTypeAndUniquePropertyName(entityType.getTypeName(), attribute.getVertexUniquePropertyName(), attrValue);
+                        vertex = AtlasGraphUtilsV2.findByTypeAndUniquePropertyName(graph, entityType.getTypeName(), attribute.getVertexUniquePropertyName(), attrValue);
 
                         // if no instance of given typeName is found, try to find an instance of type's sub-type
                         if (vertex == null && !entityType.getAllSubTypes().isEmpty()) {
-                            vertex = AtlasGraphUtilsV2.findBySuperTypeAndUniquePropertyName(entityType.getTypeName(), attribute.getVertexUniquePropertyName(), attrValue);
+                            vertex = AtlasGraphUtilsV2.findBySuperTypeAndUniquePropertyName(graph, entityType.getTypeName(), attribute.getVertexUniquePropertyName(), attrValue);
                         }
                     } else {
-                        vertex = AtlasGraphUtilsV2.findByTypeAndPropertyName(entityType.getTypeName(), attribute.getVertexPropertyName(), attrValue);
+                        vertex = AtlasGraphUtilsV2.findByTypeAndPropertyName(graph, entityType.getTypeName(), attribute.getVertexPropertyName(), attrValue);
 
                         // if no instance of given typeName is found, try to find an instance of type's sub-type
                         if (vertex == null && !entityType.getAllSubTypes().isEmpty()) {
-                            vertex = AtlasGraphUtilsV2.findBySuperTypeAndPropertyName(entityType.getTypeName(), attribute.getVertexPropertyName(), attrValue);
+                            vertex = AtlasGraphUtilsV2.findBySuperTypeAndPropertyName(graph, entityType.getTypeName(), attribute.getVertexPropertyName(), attrValue);
                         }
                     }
 
@@ -321,11 +351,37 @@ public class AtlasGraphUtilsV2 {
     }
 
     public static AtlasVertex findByGuid(String guid) {
+        return findByGuid(getGraphInstance(), guid);
+    }
+
+    public static AtlasVertex findByGuid(AtlasGraph graph, String guid) {
+        AtlasPerfMetrics.MetricRecorder metric = RequestContext.get().startMetricRecord("findByGuid");
+
         AtlasVertex ret = GraphTransactionInterceptor.getVertexFromCache(guid);
 
         if (ret == null) {
-            AtlasGraphQuery query = AtlasGraphProvider.getGraphInstance().query()
-                    .has(Constants.GUID_PROPERTY_KEY, guid);
+            AtlasGraphQuery query = graph.query().has(Constants.GUID_PROPERTY_KEY, guid);
+
+            Iterator<AtlasVertex> results = query.vertices().iterator();
+
+            ret = results.hasNext() ? results.next() : null;
+
+            if (ret != null) {
+                GraphTransactionInterceptor.addToVertexCache(guid, ret);
+            }
+        }
+
+        RequestContext.get().endMetricRecord(metric);
+        return ret;
+    }
+
+    public static AtlasVertex findDeletedByGuid(AtlasGraph graph, String guid) {
+        AtlasVertex ret = GraphTransactionInterceptor.getVertexFromCache(guid);
+
+        if (ret == null) {
+            AtlasGraphQuery query = graph.query()
+                    .has(Constants.GUID_PROPERTY_KEY, guid)
+                    .has(STATE_PROPERTY_KEY, Status.DELETED.name());
 
             Iterator<AtlasVertex> results = query.vertices().iterator();
 
@@ -339,22 +395,25 @@ public class AtlasGraphUtilsV2 {
         return ret;
     }
 
-    public static String getTypeNameFromGuid(String guid) {
+    public static String getTypeNameFromGuid(AtlasGraph graph, String guid) {
         String ret = null;
 
         if (StringUtils.isNotEmpty(guid)) {
-            AtlasVertex vertex = AtlasGraphUtilsV2.findByGuid(guid);
+            AtlasVertex vertex = AtlasGraphUtilsV2.findByGuid(graph, guid);
 
             ret = (vertex != null) ? AtlasGraphUtilsV2.getTypeName(vertex) : null;
         }
 
         return ret;
     }
-
     public static boolean typeHasInstanceVertex(String typeName) throws AtlasBaseException {
-        AtlasGraphQuery query = AtlasGraphProvider.getGraphInstance()
+        return typeHasInstanceVertex(getGraphInstance(), typeName);
+    }
+
+    public static boolean typeHasInstanceVertex(AtlasGraph graph, String typeName) throws AtlasBaseException {
+        AtlasGraphQuery query = graph
                 .query()
-                .has(Constants.TYPE_NAME_PROPERTY_KEY, AtlasGraphQuery.ComparisionOperator.EQUAL, typeName);
+                .has(TYPE_NAME_PROPERTY_KEY, AtlasGraphQuery.ComparisionOperator.EQUAL, typeName);
 
         Iterator<AtlasVertex> results = query.vertices().iterator();
 
@@ -368,10 +427,14 @@ public class AtlasGraphUtilsV2 {
     }
 
     public static AtlasVertex findByTypeAndUniquePropertyName(String typeName, String propertyName, Object attrVal) {
+        return findByTypeAndUniquePropertyName(getGraphInstance(), typeName, propertyName, attrVal);
+    }
+
+    public static AtlasVertex findByTypeAndUniquePropertyName(AtlasGraph graph, String typeName, String propertyName, Object attrVal) {
         MetricRecorder metric = RequestContext.get().startMetricRecord("findByTypeAndUniquePropertyName");
 
-        AtlasGraphQuery query = AtlasGraphProvider.getGraphInstance().query()
-                                                    .has(Constants.ENTITY_TYPE_PROPERTY_KEY, typeName)
+        AtlasGraphQuery query = graph.query()
+                                                    .has(ENTITY_TYPE_PROPERTY_KEY, typeName)
                                                     .has(propertyName, attrVal);
 
         Iterator<AtlasVertex> results = query.vertices().iterator();
@@ -383,10 +446,10 @@ public class AtlasGraphUtilsV2 {
         return vertex;
     }
 
-    public static AtlasVertex findBySuperTypeAndUniquePropertyName(String typeName, String propertyName, Object attrVal) {
+    public static AtlasVertex findBySuperTypeAndUniquePropertyName(AtlasGraph graph, String typeName, String propertyName, Object attrVal) {
         MetricRecorder metric = RequestContext.get().startMetricRecord("findBySuperTypeAndUniquePropertyName");
 
-        AtlasGraphQuery query = AtlasGraphProvider.getGraphInstance().query()
+        AtlasGraphQuery query = graph.query()
                                                     .has(Constants.SUPER_TYPES_PROPERTY_KEY, typeName)
                                                     .has(propertyName, attrVal);
 
@@ -399,13 +462,13 @@ public class AtlasGraphUtilsV2 {
         return vertex;
     }
 
-    public static AtlasVertex findByTypeAndPropertyName(String typeName, String propertyName, Object attrVal) {
+    public static AtlasVertex findByTypeAndPropertyName(AtlasGraph graph, String typeName, String propertyName, Object attrVal) {
         MetricRecorder metric = RequestContext.get().startMetricRecord("findByTypeAndPropertyName");
 
-        AtlasGraphQuery query = AtlasGraphProvider.getGraphInstance().query()
-                                                    .has(Constants.ENTITY_TYPE_PROPERTY_KEY, typeName)
+        AtlasGraphQuery query = graph.query()
+                                                    .has(ENTITY_TYPE_PROPERTY_KEY, typeName)
                                                     .has(propertyName, attrVal)
-                                                    .has(Constants.STATE_PROPERTY_KEY, AtlasEntity.Status.ACTIVE.name());
+                                                    .has(STATE_PROPERTY_KEY, AtlasEntity.Status.ACTIVE.name());
 
         Iterator<AtlasVertex> results = query.vertices().iterator();
 
@@ -416,13 +479,13 @@ public class AtlasGraphUtilsV2 {
         return vertex;
     }
 
-    public static AtlasVertex findBySuperTypeAndPropertyName(String typeName, String propertyName, Object attrVal) {
+    public static AtlasVertex findBySuperTypeAndPropertyName(AtlasGraph graph, String typeName, String propertyName, Object attrVal) {
         MetricRecorder metric = RequestContext.get().startMetricRecord("findBySuperTypeAndPropertyName");
 
-        AtlasGraphQuery query = AtlasGraphProvider.getGraphInstance().query()
+        AtlasGraphQuery query = graph.query()
                                                     .has(Constants.SUPER_TYPES_PROPERTY_KEY, typeName)
                                                     .has(propertyName, attrVal)
-                                                    .has(Constants.STATE_PROPERTY_KEY, AtlasEntity.Status.ACTIVE.name());
+                                                    .has(STATE_PROPERTY_KEY, AtlasEntity.Status.ACTIVE.name());
 
         Iterator<AtlasVertex> results = query.vertices().iterator();
 
@@ -434,8 +497,12 @@ public class AtlasGraphUtilsV2 {
     }
 
     public static List<String> findEntityGUIDsByType(String typename, SortOrder sortOrder) {
-        AtlasGraphQuery query = AtlasGraphProvider.getGraphInstance().query()
-                                                  .has(Constants.ENTITY_TYPE_PROPERTY_KEY, typename);
+        return findEntityGUIDsByType(getGraphInstance(), typename, sortOrder);
+    }
+
+    public static List<String> findEntityGUIDsByType(AtlasGraph graph, String typename, SortOrder sortOrder) {
+        AtlasGraphQuery query = graph.query()
+                                                  .has(ENTITY_TYPE_PROPERTY_KEY, typename);
         if (sortOrder != null) {
             AtlasGraphQuery.SortOrder qrySortOrder = sortOrder == SortOrder.ASCENDING ? ASC : DESC;
             query.orderBy(Constants.QUALIFIED_NAME, qrySortOrder);
@@ -455,14 +522,26 @@ public class AtlasGraphUtilsV2 {
         return ret;
     }
 
-    public static List<String> findEntityGUIDsByType(String typename) {
-        return findEntityGUIDsByType(typename, null);
+    public static List<String> findEntityGUIDsByType(AtlasGraph graph, String typename) {
+        return findEntityGUIDsByType(graph, typename, null);
+    }
+
+    public static Iterator<AtlasVertex> findActiveEntityVerticesByType(AtlasGraph graph, String typename) {
+        AtlasGraphQuery query = graph.query()
+                                          .has(ENTITY_TYPE_PROPERTY_KEY, typename)
+                                          .has(STATE_PROPERTY_KEY, Status.ACTIVE.name());
+
+        return query.vertices().iterator();
     }
 
     public static boolean relationshipTypeHasInstanceEdges(String typeName) throws AtlasBaseException {
-        AtlasGraphQuery query = AtlasGraphProvider.getGraphInstance()
+        return relationshipTypeHasInstanceEdges(getGraphInstance(), typeName);
+    }
+
+    public static boolean relationshipTypeHasInstanceEdges(AtlasGraph graph, String typeName) throws AtlasBaseException {
+        AtlasGraphQuery query = graph
                 .query()
-                .has(Constants.TYPE_NAME_PROPERTY_KEY, AtlasGraphQuery.ComparisionOperator.EQUAL, typeName);
+                .has(TYPE_NAME_PROPERTY_KEY, AtlasGraphQuery.ComparisionOperator.EQUAL, typeName);
 
         Iterator<AtlasEdge> results = query.edges().iterator();
 
@@ -526,10 +605,10 @@ public class AtlasGraphUtilsV2 {
     }
 
     public static String getStateAsString(AtlasElement element) {
-        return element.getProperty(Constants.STATE_PROPERTY_KEY, String.class);
+        return element.getProperty(STATE_PROPERTY_KEY, String.class);
     }
 
-    private static boolean canUseIndexQuery(AtlasEntityType entityType, String attributeName) {
+    private static boolean canUseIndexQuery(AtlasGraph graph, AtlasEntityType entityType, String attributeName) {
         boolean ret = false;
 
         if (USE_INDEX_QUERY_TO_FIND_ENTITY_BY_UNIQUE_ATTRIBUTES) {
@@ -538,9 +617,9 @@ public class AtlasGraphUtilsV2 {
             ret = typeAndSubTypesQryStr.length() <= SearchProcessor.MAX_QUERY_STR_LENGTH_TYPES;
 
             if (ret) {
-                Set<String> indexSet = AtlasGraphProvider.getGraphInstance().getVertexIndexKeys();
+                Set<String> indexSet = graph.getVertexIndexKeys();
                 try {
-                    ret = indexSet.contains(entityType.getQualifiedAttributeName(attributeName));
+                    ret = indexSet.contains(entityType.getVertexPropertyName(attributeName));
                 }
                 catch (AtlasBaseException ex) {
                     ret = false;
@@ -551,13 +630,13 @@ public class AtlasGraphUtilsV2 {
         return ret;
     }
 
-    private static AtlasVertex getAtlasVertexFromIndexQuery(AtlasEntityType entityType, AtlasAttribute attribute, Object attrVal) {
+    private static AtlasVertex getAtlasVertexFromIndexQuery(AtlasGraph graph, AtlasEntityType entityType, AtlasAttribute attribute, Object attrVal) {
         String          propertyName = attribute.getVertexPropertyName();
-        AtlasIndexQuery query        = getIndexQuery(entityType, propertyName, attrVal.toString());
+        AtlasIndexQuery query        = getIndexQuery(graph, entityType, propertyName, attrVal.toString());
 
-        for (Iterator<AtlasIndexQuery.Result> iter = query.vertices(); iter.hasNext(); ) {
-            AtlasIndexQuery.Result result = iter.next();
-            AtlasVertex            vertex = result.getVertex();
+        for (Iterator<Result> iter = query.vertices(); iter.hasNext(); ) {
+            Result      result = iter.next();
+            AtlasVertex vertex = result.getVertex();
 
             // skip non-entity vertices, if any got returned
             if (vertex == null || !vertex.getPropertyKeys().contains(Constants.GUID_PROPERTY_KEY)) {
@@ -590,19 +669,160 @@ public class AtlasGraphUtilsV2 {
         return null;
     }
 
-    private static AtlasIndexQuery getIndexQuery(AtlasEntityType entityType, String propertyName, String value) {
+    private static AtlasIndexQuery getIndexQuery(AtlasGraph graph, AtlasEntityType entityType, String propertyName, String value) {
         StringBuilder sb = new StringBuilder();
 
-        sb.append(INDEX_SEARCH_PREFIX + "\"").append(Constants.TYPE_NAME_PROPERTY_KEY).append("\":").append(entityType.getTypeAndAllSubTypesQryStr())
+        sb.append(INDEX_SEARCH_PREFIX + "\"").append(TYPE_NAME_PROPERTY_KEY).append("\":").append(entityType.getTypeAndAllSubTypesQryStr())
                 .append(" AND ")
                 .append(INDEX_SEARCH_PREFIX + "\"").append(propertyName).append("\":").append(AtlasAttribute.escapeIndexQueryValue(value))
                 .append(" AND ")
-                .append(INDEX_SEARCH_PREFIX + "\"").append(Constants.STATE_PROPERTY_KEY).append("\":ACTIVE");
+                .append(INDEX_SEARCH_PREFIX + "\"").append(STATE_PROPERTY_KEY).append("\":ACTIVE");
 
-        return AtlasGraphProvider.getGraphInstance().indexQuery(Constants.VERTEX_INDEX, sb.toString());
+        return graph.indexQuery(Constants.VERTEX_INDEX, sb.toString());
     }
 
     public static String getIndexSearchPrefix() {
         return INDEX_SEARCH_PREFIX;
     }
-}
+
+    public static List<String> getClassificationNames(AtlasVertex entityVertex) {
+        return getClassificationNamesHelper(entityVertex, CLASSIFICATION_NAMES_KEY);
+    }
+
+    public static List<String> getPropagatedClassificationNames(AtlasVertex entityVertex) {
+        return getClassificationNamesHelper(entityVertex, PROPAGATED_CLASSIFICATION_NAMES_KEY);
+    }
+
+    private static List<String> getClassificationNamesHelper(AtlasVertex entityVertex, String propertyKey) {
+        List<String> classificationNames = null;
+        String classificationNamesString =  entityVertex.getProperty(propertyKey, String.class);
+        if (StringUtils.isNotEmpty(classificationNamesString)) {
+            classificationNames = Arrays.asList(StringUtils.split(classificationNamesString, "\\|"));
+        }
+        return classificationNames;
+    }
+
+    public static List<Date> dateParser(String[] arr, List failedTermMsgList, int lineIndex) {
+
+        List<Date> ret = new ArrayList();
+        for (String s : arr) {
+            try{
+                SimpleDateFormat formatter = new SimpleDateFormat("MM/dd/yyyy");
+                Date date = formatter.parse(s);
+                ret.add(date);
+            }
+            catch(Exception e){
+                LOG.error("Provided value "+s+" is not of Date type at line #"+lineIndex);
+                failedTermMsgList.add("Provided value "+s+" is not of Date type at line #"+lineIndex);
+            }
+        }
+        return ret;
+    }
+
+    public static List<Boolean> booleanParser(String[] arr, List failedTermMsgList, int lineIndex) {
+
+        List<Boolean> ret = new ArrayList();
+        for (String s : arr) {
+            try{
+                ret.add(Boolean.parseBoolean(s));
+            }
+            catch(Exception e){
+                LOG.error("Provided value "+s+" is not of Boolean type at line #"+lineIndex);
+                failedTermMsgList.add("Provided value "+s+" is not of Boolean type at line #"+lineIndex);
+            }
+        }
+        return ret;
+    }
+
+    public static List<Double> doubleParser(String[] arr, List failedTermMsgList, int lineIndex) {
+
+        List<Double> ret = new ArrayList();
+        for (String s : arr) {
+            try{
+                ret.add(Double.parseDouble(s));
+            }
+            catch(Exception e){
+                LOG.error("Provided value "+s+" is not of Double type at line #"+lineIndex);
+                failedTermMsgList.add("Provided value "+s+" is not of Double type at line #"+lineIndex);
+            }
+        }
+        return ret;
+    }
+
+    public static List<Short> shortParser(String[] arr, List failedTermMsgList, int lineIndex) {
+
+        List<Short> ret = new ArrayList();
+        for (String s : arr) {
+            try{
+                ret.add(Short.parseShort(s));
+            }
+            catch(Exception e){
+                LOG.error("Provided value "+s+" is not of Short type at line #"+lineIndex);
+                failedTermMsgList.add("Provided value "+s+" is not of Short type at line #"+lineIndex);
+            }
+        }
+        return ret;
+    }
+
+    public static List<Long> longParser(String[] arr, List failedTermMsgList, int lineIndex) {
+
+        List<Long> ret = new ArrayList();
+        for (String s : arr) {
+            try{
+                ret.add(Long.parseLong(s));
+            }
+            catch(Exception e){
+                LOG.error("Provided value "+s+" is not of Long type at line #"+lineIndex);
+                failedTermMsgList.add("Provided value "+s+" is not of Long type at line #"+lineIndex);
+            }
+        }
+        return ret;
+    }
+
+    public static List<Integer> intParser(String[] arr, List failedTermMsgList, int lineIndex) {
+
+        List<Integer> ret = new ArrayList();
+        for (String s : arr) {
+            try{
+                ret.add(Integer.parseInt(s));
+            }
+            catch(Exception e){
+                LOG.error("Provided value "+s+" is not of Integer type at line #"+lineIndex);
+                failedTermMsgList.add("Provided value "+s+" is Integer of Long type at line #"+lineIndex);
+            }
+        }
+        return ret;
+    }
+
+    public static List<Float> floatParser(String[] arr, List failedTermMsgList, int lineIndex) {
+
+        List<Float> ret = new ArrayList();
+        for (String s : arr) {
+            try{
+                ret.add(Float.parseFloat(s));
+            }
+            catch(Exception e){
+                LOG.error("Provided value "+s+" is Float of Long type at line #"+lineIndex);
+                failedTermMsgList.add("Provided value "+s+" is Float of Long type at line #"+lineIndex);
+            }
+        }
+        return ret;
+    }
+
+    public static List assignEnumValues(String bmAttributeValues, AtlasEnumType enumType, List<String> failedTermMsgList, int lineIndex) {
+        List<String> ret = new ArrayList<>();
+        String[] arr = bmAttributeValues.split(FileUtils.ESCAPE_CHARACTER + FileUtils.PIPE_CHARACTER);
+        AtlasEnumDef.AtlasEnumElementDef atlasEnumDef;
+        for(String s : arr){
+            atlasEnumDef = enumType.getEnumElementDef(s);
+            if(atlasEnumDef==null){
+                LOG.error("Provided value "+s+" is Enumeration of Long type at line #"+lineIndex);
+                failedTermMsgList.add("Provided value "+s+" is Enumeration of Long type at line #"+lineIndex);
+            }else{
+                ret.add(s);
+            }
+        }
+        return ret;
+    }
+
+}
\ No newline at end of file
