@@ -19,6 +19,7 @@
 package org.apache.atlas.repository.store.graph.v2;
 
 import org.apache.atlas.AtlasErrorCode;
+import org.apache.atlas.RequestContext;
 import org.apache.atlas.annotation.GraphTransaction;
 import org.apache.atlas.authorize.AtlasAuthorizationUtils;
 import org.apache.atlas.authorize.AtlasPrivilege;
@@ -40,6 +41,7 @@ import org.apache.atlas.repository.RepositoryException;
 import org.apache.atlas.repository.graph.GraphHelper;
 import org.apache.atlas.repository.graphdb.AtlasEdge;
 import org.apache.atlas.repository.graphdb.AtlasEdgeDirection;
+import org.apache.atlas.repository.graphdb.AtlasGraph;
 import org.apache.atlas.repository.graphdb.AtlasVertex;
 import org.apache.atlas.repository.store.graph.AtlasRelationshipStore;
 import org.apache.atlas.repository.store.graph.v1.DeleteHandlerDelegate;
@@ -48,6 +50,7 @@ import org.apache.atlas.type.AtlasRelationshipType;
 import org.apache.atlas.type.AtlasStructType.AtlasAttribute;
 import org.apache.atlas.type.AtlasTypeRegistry;
 import org.apache.atlas.type.AtlasTypeUtil;
+import org.apache.atlas.utils.AtlasPerfMetrics;
 import org.apache.commons.collections.CollectionUtils;
 import org.apache.commons.collections.MapUtils;
 import org.apache.commons.lang.StringUtils;
@@ -66,9 +69,13 @@ import java.util.Map;
 import java.util.Objects;
 import java.util.Set;
 import java.util.UUID;
+import java.util.function.Function;
 
+import static org.apache.atlas.AtlasConfiguration.NOTIFICATION_RELATIONSHIPS_ENABLED;
 import static org.apache.atlas.model.instance.AtlasEntity.Status.ACTIVE;
 import static org.apache.atlas.model.instance.AtlasEntity.Status.DELETED;
+import static org.apache.atlas.model.typedef.AtlasRelationshipDef.PropagateTags.BOTH;
+import static org.apache.atlas.model.typedef.AtlasRelationshipDef.PropagateTags.NONE;
 import static org.apache.atlas.model.typedef.AtlasRelationshipDef.PropagateTags.ONE_TO_TWO;
 import static org.apache.atlas.model.typedef.AtlasRelationshipDef.PropagateTags.TWO_TO_ONE;
 import static org.apache.atlas.repository.Constants.ENTITY_TYPE_PROPERTY_KEY;
@@ -77,14 +84,10 @@ import static org.apache.atlas.repository.Constants.PROVENANCE_TYPE_KEY;
 import static org.apache.atlas.repository.Constants.RELATIONSHIPTYPE_TAG_PROPAGATION_KEY;
 import static org.apache.atlas.repository.Constants.RELATIONSHIP_GUID_PROPERTY_KEY;
 import static org.apache.atlas.repository.Constants.VERSION_PROPERTY_KEY;
-import static org.apache.atlas.AtlasConfiguration.NOTIFICATION_RELATIONSHIPS_ENABLED;
-
-
 import static org.apache.atlas.repository.graph.GraphHelper.getBlockedClassificationIds;
 import static org.apache.atlas.repository.graph.GraphHelper.getClassificationEntityGuid;
 import static org.apache.atlas.repository.graph.GraphHelper.getClassificationName;
-import static org.apache.atlas.repository.graph.GraphHelper.getClassificationVertices;
-import static org.apache.atlas.repository.graph.GraphHelper.getIncomingEdgesByLabel;
+import static org.apache.atlas.repository.graph.GraphHelper.getPropagatableClassifications;
 import static org.apache.atlas.repository.graph.GraphHelper.getPropagateTags;
 import static org.apache.atlas.repository.store.graph.v2.AtlasGraphUtilsV2.getState;
 import static org.apache.atlas.repository.store.graph.v2.AtlasGraphUtilsV2.getTypeName;
@@ -92,20 +95,23 @@ import static org.apache.atlas.repository.store.graph.v2.AtlasGraphUtilsV2.getTy
 @Component
 public class AtlasRelationshipStoreV2 implements AtlasRelationshipStore {
     private static final Logger LOG = LoggerFactory.getLogger(AtlasRelationshipStoreV2.class);
-
     private static final Long DEFAULT_RELATIONSHIP_VERSION = 0L;
+
+    private final AtlasGraph graph;
     private boolean notificationsEnabled = NOTIFICATION_RELATIONSHIPS_ENABLED.getBoolean();
 
     private final AtlasTypeRegistry         typeRegistry;
     private final EntityGraphRetriever      entityRetriever;
     private final DeleteHandlerDelegate     deleteDelegate;
-    private final GraphHelper               graphHelper = GraphHelper.getInstance();
-    private final AtlasEntityChangeNotifier entityChangeNotifier;
+    private final GraphHelper               graphHelper;
+    private final IAtlasEntityChangeNotifier entityChangeNotifier;
 
     @Inject
-    public AtlasRelationshipStoreV2(AtlasTypeRegistry typeRegistry, DeleteHandlerDelegate deleteDelegate, AtlasEntityChangeNotifier entityChangeNotifier) {
+    public AtlasRelationshipStoreV2(AtlasGraph graph, AtlasTypeRegistry typeRegistry, DeleteHandlerDelegate deleteDelegate, IAtlasEntityChangeNotifier entityChangeNotifier) {
+        this.graph                = graph;
         this.typeRegistry         = typeRegistry;
-        this.entityRetriever      = new EntityGraphRetriever(typeRegistry);
+        this.graphHelper          = new GraphHelper(graph);
+        this.entityRetriever      = new EntityGraphRetriever(graph, typeRegistry);
         this.deleteDelegate       = deleteDelegate;
         this.entityChangeNotifier = entityChangeNotifier;
     }
@@ -192,7 +198,16 @@ public class AtlasRelationshipStoreV2 implements AtlasRelationshipStore {
             }
         }
 
-        validateRelationship(end1Vertex, end2Vertex, edgeType, relationship.getAttributes());
+        boolean relationshipTypeNotExists = false;
+        if (StringUtils.isEmpty(relationship.getTypeName())) {
+            relationship.setTypeName(edgeType);
+            relationshipTypeNotExists = true;
+        }
+        validateRelationship(end1Vertex, end2Vertex, relationship);
+
+        if (relationshipTypeNotExists) {
+            relationship.setTypeName(null);
+        }
 
         AtlasRelationship ret = updateRelationship(edge, relationship);
         sendNotifications(ret, OperationType.RELATIONSHIP_UPDATE);
@@ -266,7 +281,7 @@ public class AtlasRelationshipStoreV2 implements AtlasRelationshipStore {
             throw new AtlasBaseException(AtlasErrorCode.RELATIONSHIP_ALREADY_DELETED, guid);
         }
 
-        String            relationShipType = GraphHelper.getTypeName(edge);
+        String            relationShipType = graphHelper.getTypeName(edge);
         AtlasEntityHeader end1Entity       = entityRetriever.toAtlasEntityHeaderWithClassifications(edge.getOutVertex());
         AtlasEntityHeader end2Entity       = entityRetriever.toAtlasEntityHeaderWithClassifications(edge.getInVertex());
 
@@ -337,7 +352,7 @@ public class AtlasRelationshipStoreV2 implements AtlasRelationshipStore {
         AtlasEdge ret;
 
         try {
-            validateRelationship(end1Vertex, end2Vertex, relationship.getTypeName(), relationship.getAttributes());
+            validateRelationship(end1Vertex, end2Vertex, relationship);
 
             String relationshipLabel = getRelationshipEdgeLabel(end1Vertex, end2Vertex, relationship.getTypeName());
 
@@ -444,60 +459,53 @@ public class AtlasRelationshipStoreV2 implements AtlasRelationshipStore {
         return entityRetriever.mapEdgeToAtlasRelationship(relationshipEdge);
     }
 
-    private void handleBlockedClassifications(AtlasEdge edge, Set<AtlasClassification> blockedPropagatedClassifications) throws AtlasBaseException {
-        if (blockedPropagatedClassifications != null) {
-            List<AtlasVertex> propagatedClassificationVertices               = getClassificationVertices(edge);
-            List<String>      currentClassificationIds                       = getBlockedClassificationIds(edge);
-            List<AtlasVertex> currentBlockedPropagatedClassificationVertices = getBlockedClassificationVertices(propagatedClassificationVertices, currentClassificationIds);
-            List<AtlasVertex> updatedBlockedPropagatedClassificationVertices = new ArrayList<>();
-            List<String>      updatedClassificationIds                       = new ArrayList<>();
+    private void handleBlockedClassifications(AtlasEdge edge, Set<AtlasClassification> blockedClassifications) throws AtlasBaseException {
+        if (blockedClassifications != null) {
+            List<AtlasVertex> propagatableClassifications  = getPropagatableClassifications(edge);
+            List<String>      currBlockedClassificationIds = getBlockedClassificationIds(edge);
+            List<AtlasVertex> currBlockedClassifications   = getVerticesForIds(propagatableClassifications, currBlockedClassificationIds);
+            List<AtlasVertex> classificationsToBlock       = new ArrayList<>();
+            List<String>      classificationIdsToBlock     = new ArrayList<>();
 
-            for (AtlasClassification classification : blockedPropagatedClassifications) {
-                AtlasVertex classificationVertex = validateBlockedPropagatedClassification(propagatedClassificationVertices, classification);
+            for (AtlasClassification blockedClassification : blockedClassifications) {
+                AtlasVertex classificationVertex = validateBlockedPropagatedClassification(propagatableClassifications, blockedClassification);
 
-                // ignore invalid blocked propagated classification
-                if (classificationVertex == null) {
-                    continue;
+                if (classificationVertex != null) {
+                    classificationsToBlock.add(classificationVertex);
+                    classificationIdsToBlock.add(classificationVertex.getIdForDisplay());
+                }
+            }
+
+            setBlockedClassificationIds(edge, classificationIdsToBlock);
+
+            List<AtlasVertex> propagationChangedClassifications = (List<AtlasVertex>) CollectionUtils.disjunction(classificationsToBlock, currBlockedClassifications);
+
+            for (AtlasVertex classificationVertex : propagationChangedClassifications) {
+                List<AtlasVertex> propagationsToRemove = new ArrayList<>();
+                List<AtlasVertex> propagationsToAdd    = new ArrayList<>();
+
+                entityRetriever.evaluateClassificationPropagation(classificationVertex, propagationsToAdd, propagationsToRemove);
+
+                if (CollectionUtils.isNotEmpty(propagationsToAdd)) {
+                    deleteDelegate.getHandler().addTagPropagation(classificationVertex, propagationsToAdd);
                 }
 
-                updatedBlockedPropagatedClassificationVertices.add(classificationVertex);
-
-                String classificationId = classificationVertex.getIdForDisplay();
-
-                updatedClassificationIds.add(classificationId);
-            }
-
-            addToBlockedClassificationIds(edge, updatedClassificationIds);
-
-            // remove propagated tag for added entry
-            List<AtlasVertex> addedBlockedClassifications = (List<AtlasVertex>) CollectionUtils.subtract(updatedBlockedPropagatedClassificationVertices, currentBlockedPropagatedClassificationVertices);
-
-            for (AtlasVertex classificationVertex : addedBlockedClassifications) {
-                List<AtlasVertex> removePropagationFromVertices = graphHelper.getPropagatedEntityVertices(classificationVertex);
-
-                deleteDelegate.getHandler().removeTagPropagation(classificationVertex, removePropagationFromVertices);
-            }
-
-            // add propagated tag for removed entry
-            List<AtlasVertex> removedBlockedClassifications = (List<AtlasVertex>) CollectionUtils.subtract(currentBlockedPropagatedClassificationVertices, updatedBlockedPropagatedClassificationVertices);
-
-            for (AtlasVertex classificationVertex : removedBlockedClassifications) {
-                List<AtlasVertex> addPropagationToVertices = graphHelper.getPropagatedEntityVertices(classificationVertex);
-
-                deleteDelegate.getHandler().addTagPropagation(classificationVertex, addPropagationToVertices);
+                if (CollectionUtils.isNotEmpty(propagationsToRemove)) {
+                    deleteDelegate.getHandler().removeTagPropagation(classificationVertex, propagationsToRemove);
+                }
             }
         }
     }
 
-    private List<AtlasVertex> getBlockedClassificationVertices(List<AtlasVertex> classificationVertices, List<String> blockedClassificationIds) {
+    private List<AtlasVertex> getVerticesForIds(List<AtlasVertex> vertices, List<String> vertexIds) {
         List<AtlasVertex> ret = new ArrayList<>();
 
-        if (CollectionUtils.isNotEmpty(blockedClassificationIds)) {
-            for (AtlasVertex classificationVertex : classificationVertices) {
-                String classificationId = classificationVertex.getIdForDisplay();
+        if (CollectionUtils.isNotEmpty(vertexIds)) {
+            for (AtlasVertex vertex : vertices) {
+                String vertexId = vertex.getIdForDisplay();
 
-                if (blockedClassificationIds.contains(classificationId)) {
-                    ret.add(classificationVertex);
+                if (vertexIds.contains(vertexId)) {
+                    ret.add(vertex);
                 }
             }
         }
@@ -506,7 +514,6 @@ public class AtlasRelationshipStoreV2 implements AtlasRelationshipStore {
     }
 
     // propagated classifications should contain blocked propagated classification
-
     private AtlasVertex validateBlockedPropagatedClassification(List<AtlasVertex> classificationVertices, AtlasClassification classification) {
         AtlasVertex ret = null;
 
@@ -522,7 +529,8 @@ public class AtlasRelationshipStoreV2 implements AtlasRelationshipStore {
 
         return ret;
     }
-    private void addToBlockedClassificationIds(AtlasEdge edge, List<String> classificationIds) {
+
+    private void setBlockedClassificationIds(AtlasEdge edge, List<String> classificationIds) {
         if (edge != null) {
             if (classificationIds.isEmpty()) {
                 edge.removeProperty(Constants.RELATIONSHIPTYPE_BLOCKED_PROPAGATED_CLASSIFICATIONS_KEY);
@@ -537,15 +545,15 @@ public class AtlasRelationshipStoreV2 implements AtlasRelationshipStore {
         PropagateTags newTagPropagation = relationship.getPropagateTags();
 
         if (newTagPropagation != oldTagPropagation) {
-            List<AtlasVertex>                   currentClassificationVertices = getClassificationVertices(edge);
-            Map<AtlasVertex, List<AtlasVertex>> currentClassificationsMap     = graphHelper.getClassificationPropagatedEntitiesMapping(currentClassificationVertices);
+            List<AtlasVertex>                   currentClassificationVertices = getPropagatableClassifications(edge);
+            Map<AtlasVertex, List<AtlasVertex>> currentClassificationsMap     = entityRetriever.getClassificationPropagatedEntitiesMapping(currentClassificationVertices);
 
             // Update propagation edge
             AtlasGraphUtilsV2.setEncodedProperty(edge, RELATIONSHIPTYPE_TAG_PROPAGATION_KEY, newTagPropagation.name());
 
-            List<AtlasVertex>                   updatedClassificationVertices = getClassificationVertices(edge);
+            List<AtlasVertex>                   updatedClassificationVertices = getPropagatableClassifications(edge);
             List<AtlasVertex>                   classificationVerticesUnion   = (List<AtlasVertex>) CollectionUtils.union(currentClassificationVertices, updatedClassificationVertices);
-            Map<AtlasVertex, List<AtlasVertex>> updatedClassificationsMap     = graphHelper.getClassificationPropagatedEntitiesMapping(classificationVerticesUnion);
+            Map<AtlasVertex, List<AtlasVertex>> updatedClassificationsMap     = entityRetriever.getClassificationPropagatedEntitiesMapping(classificationVerticesUnion);
 
             // compute add/remove propagations list
             Map<AtlasVertex, List<AtlasVertex>> addPropagationsMap    = new HashMap<>();
@@ -623,7 +631,10 @@ public class AtlasRelationshipStoreV2 implements AtlasRelationshipStore {
         validateAndNormalize(relationship);
     }
 
-    private void validateRelationship(AtlasVertex end1Vertex, AtlasVertex end2Vertex, String relationshipName, Map<String, Object> attributes) throws AtlasBaseException {
+    private void validateRelationship(AtlasVertex end1Vertex, AtlasVertex end2Vertex,  AtlasRelationship relationship) throws AtlasBaseException {
+
+        String relationshipName = relationship.getTypeName();
+
         AtlasRelationshipType relationshipType = typeRegistry.getRelationshipTypeByName(relationshipName);
 
         if (relationshipType == null) {
@@ -653,8 +664,35 @@ public class AtlasRelationshipStoreV2 implements AtlasRelationshipStore {
             throw new AtlasBaseException(AtlasErrorCode.INVALID_RELATIONSHIP_END_TYPE, relationshipName, relationshipType.getEnd2Type().getTypeName(), end1TypeName);
         }
 
+        PropagateTags typePropagation = relationshipType.getRelationshipDef().getPropagateTags();
+        PropagateTags edgePropagation = relationship.getPropagateTags();
+
+        if (typePropagation == null) {
+            typePropagation = NONE;
+        }
+
+        if (edgePropagation == null) {
+            edgePropagation = NONE;
+        }
+
+        /*
+          +-------------+----------------------------------------+
+          |     type    |                edge                    |
+          +-------------+-------+------------+------------+------+
+          |             | NONE  | ONE_TO_TWO | TWO_TO_ONE | BOTH |
+          |-------------+-------+------------+------------+------|
+          | NONE        |   Y   |     N      |      N     |   N  |
+          | ONE_TO_TWO  |   Y   |     Y      |      N     |   N  |
+          | TWO_TO_ONE  |   Y   |     N      |      Y     |   N  |
+          | BOTH        |   Y   |     Y      |      Y     |   Y  |
+          +-------------+-------+------------+------------+------+
+         */
+
+        if (edgePropagation != NONE && typePropagation != BOTH && edgePropagation != typePropagation) {
+            throw new AtlasBaseException(AtlasErrorCode.INVALID_PROPAGATION_TYPE, edgePropagation.toString(), relationshipName, typePropagation.toString());
+        }
+
         List<String>      messages     = new ArrayList<>();
-        AtlasRelationship relationship = new AtlasRelationship(relationshipName, attributes);
 
         relationshipType.validateValue(relationship, relationshipName, messages);
 
@@ -690,7 +728,7 @@ public class AtlasRelationshipStoreV2 implements AtlasRelationshipStore {
             String              guid             = end.getGuid();
             String              typeName         = end.getTypeName();
             Map<String, Object> uniqueAttributes = end.getUniqueAttributes();
-            AtlasVertex         endVertex        = AtlasGraphUtilsV2.findByGuid(guid);
+            AtlasVertex         endVertex        = AtlasGraphUtilsV2.findByGuid(this.graph, guid);
 
             if (!AtlasTypeUtil.isValidGuid(guid) || endVertex == null) {
                 throw new AtlasBaseException(AtlasErrorCode.INSTANCE_GUID_NOT_FOUND, guid);
@@ -698,7 +736,7 @@ public class AtlasRelationshipStoreV2 implements AtlasRelationshipStore {
             } else if (MapUtils.isNotEmpty(uniqueAttributes)) {
                 AtlasEntityType entityType = typeRegistry.getEntityTypeByName(typeName);
 
-                if (AtlasGraphUtilsV2.findByUniqueAttributes(entityType, uniqueAttributes) == null) {
+                if (AtlasGraphUtilsV2.findByUniqueAttributes(this.graph, entityType, uniqueAttributes) == null) {
                     throw new AtlasBaseException(AtlasErrorCode.INSTANCE_BY_UNIQUE_ATTRIBUTE_NOT_FOUND, typeName, uniqueAttributes.toString());
                 }
             } else {
@@ -741,22 +779,15 @@ public class AtlasRelationshipStoreV2 implements AtlasRelationshipStore {
     }
 
     public AtlasEdge getRelationshipEdge(AtlasVertex fromVertex, AtlasVertex toVertex, String relationshipLabel) {
-        AtlasEdge           ret           = null;
-        Iterator<AtlasEdge> edgesIterator = getIncomingEdgesByLabel(toVertex, relationshipLabel);
+        AtlasPerfMetrics.MetricRecorder metric = RequestContext.get().startMetricRecord("getRelationshipEdge");
 
-        while (edgesIterator != null && edgesIterator.hasNext()) {
-            AtlasEdge edge = edgesIterator.next();
+        AtlasEdge ret = null;
 
-            if (edge != null) {
-                Status status = graphHelper.getStatus(edge);
-
-                if ((status == null || status == ACTIVE) && edge.getOutVertex().equals(fromVertex)) {
-                    ret = edge;
-                    break;
-                }
-            }
+        if (toVertex.hasEdges(AtlasEdgeDirection.IN, relationshipLabel) && fromVertex.hasEdges(AtlasEdgeDirection.OUT, relationshipLabel)) {
+            ret = graph.getEdgeBetweenVertices(fromVertex, toVertex, relationshipLabel);
         }
 
+        RequestContext.get().endMetricRecord(metric);
         return ret;
     }
 
@@ -770,11 +801,11 @@ public class AtlasRelationshipStoreV2 implements AtlasRelationshipStore {
         AtlasVertex ret = null;
 
         if (StringUtils.isNotEmpty(endPoint.getGuid())) {
-            ret = AtlasGraphUtilsV2.findByGuid(endPoint.getGuid());
+            ret = AtlasGraphUtilsV2.findByGuid(this.graph, endPoint.getGuid());
         } else if (StringUtils.isNotEmpty(endPoint.getTypeName()) && MapUtils.isNotEmpty(endPoint.getUniqueAttributes())) {
             AtlasEntityType entityType = typeRegistry.getEntityTypeByName(endPoint.getTypeName());
 
-            ret = AtlasGraphUtilsV2.findByUniqueAttributes(entityType, endPoint.getUniqueAttributes());
+            ret = AtlasGraphUtilsV2.findByUniqueAttributes(this.graph, entityType, endPoint.getUniqueAttributes());
         }
 
         return ret;
@@ -857,7 +888,7 @@ public class AtlasRelationshipStoreV2 implements AtlasRelationshipStore {
         String typeName = objectId.getTypeName();
 
         if (StringUtils.isBlank(typeName)) {
-            typeName = AtlasGraphUtilsV2.getTypeNameFromGuid(objectId.getGuid());
+            typeName = AtlasGraphUtilsV2.getTypeNameFromGuid(this.graph, objectId.getGuid());
         }
 
         return typeName;
