@@ -539,11 +539,15 @@ public class AtlasStructDefStoreV2 extends AtlasAbstractDefStoreV2<AtlasStructDe
         attribInfo.put("reverseAttributeName", attribute.getInverseRefAttributeName());
         attribInfo.put("defaultValue", attributeDef.getDefaultValue());
         attribInfo.put("description", attributeDef.getDescription());
+        attribInfo.put("searchWeight", attributeDef.getSearchWeight());
+        attribInfo.put("indexType", attributeDef.getIndexType());
 
         if(attributeDef.getOptions() != null) {
             attribInfo.put("options", AtlasType.toJson(attributeDef.getOptions()));
         }
 
+        attribInfo.put("displayName", attributeDef.getDisplayName());
+
         final int lower;
         final int upper;
 
@@ -589,6 +593,8 @@ public class AtlasStructDefStoreV2 extends AtlasAbstractDefStoreV2<AtlasStructDe
             ret.setOptions(AtlasType.fromJson((String) attribInfo.get("options"), Map.class));
         }
 
+        ret.setDisplayName((String) attribInfo.get("displayName"));
+
         if ((Boolean)attribInfo.get("isComposite")) {
             ret.addConstraint(new AtlasConstraintDef(AtlasConstraintDef.CONSTRAINT_TYPE_OWNED_REF));
         }
@@ -627,6 +633,17 @@ public class AtlasStructDefStoreV2 extends AtlasAbstractDefStoreV2<AtlasStructDe
             ret.setValuesMaxCount(maxCount.intValue());
         }
 
+        Number searchWeight = (Number) attribInfo.get("searchWeight");
+        if( searchWeight != null ) {
+            ret.setSearchWeight(searchWeight.intValue());
+        } else {
+            ret.setSearchWeight(-1);
+        }
+
+        String indexType = (String) attribInfo.get("indexType");
+        if(!StringUtils.isEmpty(indexType)) {
+            ret.setIndexType(AtlasAttributeDef.IndexType.valueOf(indexType));
+        }
         return ret;
     }
 
@@ -644,7 +661,9 @@ public class AtlasStructDefStoreV2 extends AtlasAbstractDefStoreV2<AtlasStructDe
         ret.setReverseAttributeName(attribute.getInverseRefAttributeName());
         ret.setDescription(attrDef.getDescription());
         ret.setDefaultValue(attrDef.getDefaultValue());
-
+        ret.setSearchWeight(attrDef.getSearchWeight());
+        ret.setIndexType(attrDef.getIndexType());
         return ret;
     }
 }
+
