@@ -17,6 +17,7 @@
  */
 package org.apache.atlas.query;
 
+import org.apache.atlas.BasicTestSetup;
 import org.apache.atlas.TestModules;
 import org.apache.atlas.discovery.EntityDiscoveryService;
 import org.apache.atlas.exception.AtlasBaseException;
@@ -134,6 +135,14 @@ public class DSLQueriesTest extends BasicTestSetup {
     @DataProvider(name = "comparisonQueriesProvider")
     private Object[][] comparisonQueriesProvider() {
         return new Object[][] {
+                {"Person where (name = \"Julius\" )", 1},
+                {"Person where (name like \"Jul*\" )", 1},
+                {"Person where (name like \"J*\" )", 3},
+                {"Person where (name like \"*us\" )", 1},
+                {"Person where (name like \"*uli*\" )", 1},
+                {"Person where (name like \"Julius\" )", 1},
+                {"Person where (name like \"Jul\" )", 0},
+
                 {"Person where (birthday < \"1950-01-01T02:35:58.440Z\" )", 0},
                 {"Person where (birthday > \"1975-01-01T02:35:58.440Z\" )", 2},
                 {"Person where (birthday >= \"1975-01-01T02:35:58.440Z\" )", 2},
@@ -196,6 +205,8 @@ public class DSLQueriesTest extends BasicTestSetup {
     @DataProvider(name = "basicProvider")
     private Object[][] basicQueries() {
         return new Object[][]{
+                {"hive_column where table.name = \"sales_fact_daily_mv\"", 4},
+                {"hive_table where columns.name = \"app_id\"", 2},
                 {"from hive_db", 3},
                 {"hive_db", 3},
                 {"hive_db as d select d", 3},
@@ -454,6 +465,11 @@ public class DSLQueriesTest extends BasicTestSetup {
     @DataProvider(name = "likeQueriesProvider")
     private Object[][] likeQueries() {
         return new Object[][]{
+                {"hive_table qualifiedName like \"*time_dim*\"", 1},
+                {"hive_db where qualifiedName like \"qualified:R*\"", 1},
+                {"hive_table db.name=\"Sales\"", 4},
+                {"hive_table qualifiedName =\"Sales.time_dim\" AND db.name=\"Sales\"", 1},
+                {"hive_table qualifiedName like \"*time_dim*\" AND db.name=\"Sales\"", 1},
                 {"hive_table where name like \"sa?es*\"", 3},
                 {"hive_db where name like \"R*\"", 1},
                 {"hive_db where hive_db.name like \"R???rt?*\" or hive_db.name like \"S?l?s\" or hive_db.name like\"Log*\"", 3},
