@@ -22,6 +22,7 @@ import org.apache.atlas.model.instance.AtlasClassification;
 import org.apache.atlas.model.instance.AtlasEntity;
 import org.apache.atlas.model.instance.AtlasEntity.AtlasEntityWithExtInfo;
 import org.apache.atlas.model.instance.AtlasEntityHeader;
+import org.apache.atlas.model.instance.AtlasObjectId;
 import org.apache.atlas.store.DeleteType;
 import org.apache.atlas.utils.AtlasPerfMetrics;
 import org.apache.atlas.utils.AtlasPerfMetrics.MetricRecorder;
@@ -29,7 +30,16 @@ import org.apache.commons.lang.StringUtils;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
-import java.util.*;
+import java.util.Collection;
+import java.util.List;
+import java.util.Set;
+import java.util.Map;
+import java.util.ArrayList;
+import java.util.HashSet;
+import java.util.HashMap;
+
+import static org.apache.atlas.model.instance.AtlasObjectId.KEY_GUID;
+
 
 public class RequestContext {
     private static final Logger METRICS = LoggerFactory.getLogger("METRICS");
@@ -38,26 +48,29 @@ public class RequestContext {
     private static final Set<RequestContext>         ACTIVE_REQUESTS = new HashSet<>();
     private static final boolean                     isMetricsEnabled = METRICS.isDebugEnabled();
 
-    private final long                                   requestTime         = System.currentTimeMillis();
-    private final Map<String, AtlasEntityHeader>         updatedEntities     = new HashMap<>();
-    private final Map<String, AtlasEntityHeader>         deletedEntities     = new HashMap<>();
-    private final Map<String, AtlasEntity>               entityCache         = new HashMap<>();
-    private final Map<String, AtlasEntityWithExtInfo>    entityExtInfoCache  = new HashMap<>();
-    private final Map<String, List<AtlasClassification>> addedPropagations   = new HashMap<>();
-    private final Map<String, List<AtlasClassification>> removedPropagations = new HashMap<>();
-    private final AtlasPerfMetrics                       metrics             = isMetricsEnabled ? new AtlasPerfMetrics() : null;
-    private       List<EntityGuidPair>                   entityGuidInRequest = null;
+    private final long                                   requestTime          = System.currentTimeMillis();
+    private final Map<String, AtlasEntityHeader>         updatedEntities      = new HashMap<>();
+    private final Map<String, AtlasEntityHeader>         deletedEntities      = new HashMap<>();
+    private final Map<String, AtlasEntity>               entityCache          = new HashMap<>();
+    private final Map<String, AtlasEntityWithExtInfo>    entityExtInfoCache   = new HashMap<>();
+    private final Map<String, List<AtlasClassification>> addedPropagations    = new HashMap<>();
+    private final Map<String, List<AtlasClassification>> removedPropagations  = new HashMap<>();
+    private final AtlasPerfMetrics                       metrics              = isMetricsEnabled ? new AtlasPerfMetrics() : null;
+    private       List<EntityGuidPair>                   entityGuidInRequest  = null;
+    private final Set<String>                            entitiesToSkipUpdate = new HashSet<>();
 
-    private String      user;
-    private Set<String> userGroups;
-    private String      clientIPAddress;
-    private DeleteType  deleteType   = DeleteType.DEFAULT;
+    private String       user;
+    private Set<String>  userGroups;
+    private String       clientIPAddress;
+    private List<String> forwardedAddresses;
+    private DeleteType   deleteType   = DeleteType.DEFAULT;
+    private boolean     isPurgeRequested = false;
     private int         maxAttempts  = 1;
     private int         attemptCount = 1;
     private boolean     isImportInProgress = false;
     private boolean     isInNotificationProcessing = false;
     private boolean     isInTypePatching           = false;
-
+    private boolean     createShellEntityForNonExistingReference = false;
 
     private RequestContext() {
     }
@@ -100,6 +113,7 @@ public class RequestContext {
         this.entityExtInfoCache.clear();
         this.addedPropagations.clear();
         this.removedPropagations.clear();
+        this.entitiesToSkipUpdate.clear();
 
         if (metrics != null && !metrics.isEmpty()) {
             METRICS.debug(metrics.toString());
@@ -166,6 +180,10 @@ public class RequestContext {
         isImportInProgress = importInProgress;
     }
 
+    public boolean isPurgeRequested() { return isPurgeRequested; }
+
+    public void setPurgeRequested(boolean isPurgeRequested) { this.isPurgeRequested = isPurgeRequested; }
+
     public boolean isInNotificationProcessing() {
         return isInNotificationProcessing;
     }
@@ -182,13 +200,26 @@ public class RequestContext {
         isInTypePatching = inTypePatching;
     }
 
+    public boolean isCreateShellEntityForNonExistingReference() {
+        return createShellEntityForNonExistingReference;
+    }
+
+    public void setCreateShellEntityForNonExistingReference(boolean createShellEntityForNonExistingReference) {
+        this.createShellEntityForNonExistingReference = createShellEntityForNonExistingReference;
+    }
 
     public void recordEntityUpdate(AtlasEntityHeader entity) {
-        if (entity != null && entity.getGuid() != null) {
+        if (entity != null && entity.getGuid() != null && ! entitiesToSkipUpdate.contains(entity.getGuid())) {
             updatedEntities.put(entity.getGuid(), entity);
         }
     }
 
+    public void recordEntityToSkip(String guid) {
+        if(! StringUtils.isEmpty(guid)) {
+            entitiesToSkipUpdate.add(guid);
+        }
+    }
+
     public void recordEntityDelete(AtlasEntityHeader entity) {
         if (entity != null && entity.getGuid() != null) {
             deletedEntities.put(entity.getGuid(), entity);
@@ -302,8 +333,6 @@ public class RequestContext {
         return deletedEntities.containsKey(guid);
     }
 
-
-
     public MetricRecorder startMetricRecord(String name) { return metrics != null ? metrics.getMetricRecorder(name) : null; }
 
     public void endMetricRecord(MetricRecorder recorder) {
@@ -313,11 +342,23 @@ public class RequestContext {
     }
 
     public void recordEntityGuidUpdate(AtlasEntity entity, String guidInRequest) {
+        recordEntityGuidUpdate(new EntityGuidPair(entity, guidInRequest));
+    }
+
+    public void recordEntityGuidUpdate(AtlasObjectId entity, String guidInRequest) {
+        recordEntityGuidUpdate(new EntityGuidPair(entity, guidInRequest));
+    }
+
+    public void recordEntityGuidUpdate(Map entity, String guidInRequest) {
+        recordEntityGuidUpdate(new EntityGuidPair(entity, guidInRequest));
+    }
+
+    public void recordEntityGuidUpdate(EntityGuidPair record) {
         if (entityGuidInRequest == null) {
             entityGuidInRequest = new ArrayList<>();
         }
 
-        entityGuidInRequest.add(new EntityGuidPair(entity, guidInRequest));
+        entityGuidInRequest.add(record);
     }
 
     public void resetEntityGuidUpdates() {
@@ -329,16 +370,40 @@ public class RequestContext {
     }
 
     public class EntityGuidPair {
-        private final AtlasEntity entity;
-        private final String      guid;
+        private final Object entity;
+        private final String guid;
 
         public EntityGuidPair(AtlasEntity entity, String guid) {
             this.entity = entity;
             this.guid   = guid;
         }
 
+        public EntityGuidPair(AtlasObjectId entity, String guid) {
+            this.entity = entity;
+            this.guid   = guid;
+        }
+
+        public EntityGuidPair(Map entity, String guid) {
+            this.entity = entity;
+            this.guid   = guid;
+        }
+
         public void resetEntityGuid() {
-            entity.setGuid(guid);
+            if (entity instanceof AtlasEntity) {
+                ((AtlasEntity) entity).setGuid(guid);
+            } else if (entity instanceof AtlasObjectId) {
+                ((AtlasObjectId) entity).setGuid(guid);
+            } else if (entity instanceof Map) {
+                ((Map) entity).put(KEY_GUID, guid);
+            }
         }
     }
+
+    public List<String> getForwardedAddresses() {
+        return forwardedAddresses;
+    }
+
+    public void setForwardedAddresses(List<String> forwardedAddresses) {
+        this.forwardedAddresses = forwardedAddresses;
+    }
 }
