@@ -96,11 +96,21 @@ public class EntityNotificationListenerV2 implements EntityChangeListenerV2 {
         notifyEntityEvents(entities, ENTITY_DELETE);
     }
 
+    @Override
+    public void onEntitiesPurged(List<AtlasEntity> entities) throws AtlasBaseException {
+        // do nothing -> notification not sent out for term purged from entities as its been sent in case of delete
+    }
+
     @Override
     public void onClassificationsAdded(AtlasEntity entity, List<AtlasClassification> classifications) throws AtlasBaseException {
         notifyEntityEvents(Collections.singletonList(entity), CLASSIFICATION_ADD);
     }
 
+    @Override
+    public void onClassificationsAdded(List<AtlasEntity> entities, List<AtlasClassification> classifications) throws AtlasBaseException {
+        notifyEntityEvents(entities, CLASSIFICATION_ADD);
+    }
+
     @Override
     public void onClassificationsUpdated(AtlasEntity entity, List<AtlasClassification> classifications) throws AtlasBaseException {
         Map<String, List<AtlasClassification>> addedPropagations   = RequestContext.get().getAddedPropagations();
@@ -118,6 +128,11 @@ public class EntityNotificationListenerV2 implements EntityChangeListenerV2 {
         notifyEntityEvents(Collections.singletonList(entity), CLASSIFICATION_DELETE);
     }
 
+    @Override
+    public void onClassificationsDeleted(List<AtlasEntity> entities, List<AtlasClassification> classifications) throws AtlasBaseException {
+        notifyEntityEvents(entities, CLASSIFICATION_DELETE);
+    }
+
     @Override
     public void onTermAdded(AtlasGlossaryTerm term, List<AtlasRelatedObjectId> entities) {
         // do nothing -> notification not sent out for term assignment to entities
@@ -128,6 +143,16 @@ public class EntityNotificationListenerV2 implements EntityChangeListenerV2 {
         // do nothing -> notification not sent out for term removal from entities
     }
 
+    @Override
+    public void onLabelsDeleted(AtlasEntity entity, Set<String> labels) throws AtlasBaseException {
+        // do nothing -> notification not sent out for label removal to entities
+    }
+
+    @Override
+    public void onLabelsAdded(AtlasEntity entity, Set<String> labels) throws AtlasBaseException {
+        // do nothing -> notification not sent out for label assignment to entities
+    }
+
     private void notifyEntityEvents(List<AtlasEntity> entities, OperationType operationType) throws AtlasBaseException {
         MetricRecorder metric = RequestContext.get().startMetricRecord("entityNotification");
         List<EntityNotificationV2> messages = new ArrayList<>();
@@ -177,6 +202,8 @@ public class EntityNotificationListenerV2 implements EntityChangeListenerV2 {
 
         ret.setGuid(entity.getGuid());
         ret.setStatus(entity.getStatus());
+        ret.setIsIncomplete(entity.getIsIncomplete());
+
         setAttribute(ret, NAME, name);
         setAttribute(ret, DESCRIPTION, entity.getAttribute(DESCRIPTION));
         setAttribute(ret, OWNER, entity.getAttribute(OWNER));
@@ -284,4 +311,14 @@ public class EntityNotificationListenerV2 implements EntityChangeListenerV2 {
     public void onRelationshipsDeleted(List<AtlasRelationship> relationships, boolean isImport) throws AtlasBaseException {
         notifyRelationshipEvents(relationships, RELATIONSHIP_DELETE);
     }
+
+    @Override
+    public void onRelationshipsPurged(List<AtlasRelationship> relationships) throws AtlasBaseException {
+        // do nothing -> notification not sent out for term purged from entities as its been sent in case of delete
+    }
+
+    @Override
+    public void onBusinessAttributesUpdated(AtlasEntity entity, Map<String, Map<String, Object>> updatedBusinessAttributes) throws AtlasBaseException{
+        // do nothing -> notification not sent out for business metadata attribute updation from entities
+    }
 }
\ No newline at end of file
