@@ -31,6 +31,7 @@ public abstract class EntityPreprocessor {
     public static final String TYPE_HIVE_COLUMN_LINEAGE = "hive_column_lineage";
     public static final String TYPE_HIVE_PROCESS        = "hive_process";
     public static final String TYPE_HIVE_STORAGEDESC    = "hive_storagedesc";
+    public static final String TYPE_HIVE_DB             = "hive_db";
     public static final String TYPE_HIVE_TABLE          = "hive_table";
     public static final String TYPE_RDBMS_INSTANCE      = "rdbms_instance";
     public static final String TYPE_RDBMS_DB            = "rdbms_db";
@@ -54,6 +55,8 @@ public abstract class EntityPreprocessor {
     public static final String ATTRIBUTE_INDEXES        = "indexes";
     public static final String ATTRIBUTE_FOREIGN_KEYS   = "foreign_keys";
     public static final String ATTRIBUTE_INSTANCE       = "instance";
+    public static final String ATTRIBUTE_START_TIME     = "startTime";
+    public static final String ATTRIBUTE_END_TIME       = "endTime";
 
     public static final char   QNAME_SEP_CLUSTER_NAME = '@';
     public static final char   QNAME_SEP_ENTITY_NAME  = '.';
@@ -67,6 +70,7 @@ public abstract class EntityPreprocessor {
 
     static {
         EntityPreprocessor[] hivePreprocessors = new EntityPreprocessor[] {
+                                                                    new HivePreprocessor.HiveDbPreprocessor(),
                                                                     new HivePreprocessor.HiveTablePreprocessor(),
                                                                     new HivePreprocessor.HiveColumnPreprocessor(),
                                                                     new HivePreprocessor.HiveProcessPreprocessor(),
@@ -114,6 +118,12 @@ public abstract class EntityPreprocessor {
         return obj != null ? obj.toString() : null;
     }
 
+    public static String getName(AtlasEntity entity) {
+        Object obj = entity != null ? entity.getAttribute(ATTRIBUTE_NAME) : null;
+
+        return obj != null ? obj.toString() : null;
+    }
+
     public String getTypeName(Object obj) {
         Object ret = null;
 
